/***********************************************************************************
AWTDisplay.java - Gerando uma GUI com AWT

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Gerando uma GUI com AWT

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel AWT com a imagem original.

Desenvolvido por:

    Jos Iguelmar Miranda


Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.Frame;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class AWTDisplay extends Frame {
   // Define um objeto BufferedImage para encapsular a imagem
   private BufferedImage imagem;

   public static void main(String args[]) {

     // Certifica a passagem da imagem como parmetro
     if (args.length != 1) {
        System.out.println("Uso: java -cp . AWTDisplay <arquivo_imagem>");
        System.exit(0);
     }

     // Chama o construtor passando o nome do arquivo imagem
     AWTDisplay d = new AWTDisplay(args[0]);
     d.setVisible(true);
   }

   public AWTDisplay(String filename) {

     // L imagem
     File file = new File(filename);
     try {
       imagem = ImageIO.read(file);
     } catch(Exception e) {
       System.out.println("Imagem '" + filename + "' nao existe.");
       System.exit(0);
     }

     // Atribui nome ao frame
     setTitle("AWTDisplay: " + file.getName());

     setSize(imagem.getWidth(), imagem.getHeight());
   }

   // Sobrecarrega o metodo grfico paint() para desenhar a imagem
   public void paint(Graphics g) {
     Graphics2D g2 = (Graphics2D)g;

     // Translada origem da imagem e desenha
     g2.translate(this.getInsets().left,this.getInsets().top);
     g2.drawImage(imagem, 0, 0, this);
   }

   // Encerra a aplicao clicando no "close"
   public boolean handleEvent(Event event) {
     if (event.id == Event.WINDOW_DESTROY)
        System.exit(0);
        return false;
   }

}
