/***********************************************************************************
Binariza.java - Binariza uma imagem usando um valor teto ou limiar

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Binariza uma imagem usando um valor teto ou limiar

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. valor do limiar.

Sada:

  Painel com a imagem original e imagem binria.

Desenvolvido por:

    Jos Iguelmar Miranda


Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.ByteLookupTable;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Binariza extends JFrame {

	public static void main(String args[]) {
		int limiar = 0;

    // Certifica a passagem da imagem e valor teto
    if (args.length != 2) {
         String msga = "Uso: java -cp . Binariza <imagem> <limiar>";
         System.out.println(msga);
         System.exit(0);
    }

    try {
        limiar = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'limiar' invalido");
        System.exit(0);
    }

    if (limiar <= 0) {
        System.out.println("limiar: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando os parmetros
    Binariza bina = new Binariza(args[0], limiar);

    // Encerra a aplicao clicando no "close"
    bina.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    bina.setVisible(true);
  }

  public Binariza(String aFile, int limiar) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
		int w, h;
		JLabel img1, img2;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Binariza imagem: " + file.getName());

    dest = binOp(src, limiar);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		setSize(2*w, h);
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
	 }

	// Binariza imagem usando uma tabela de cores - lookup table
  public BufferedImage binOp(BufferedImage src, int teto) {
		int binMax = 255, binMin = 0;
    byte[] tabela = new byte[256];

    BufferedImage out = null; 
    Raster srcR = src.getRaster(); 
    WritableRaster destWR = src.copyData(null); 

    // Constri a nova tabela de cores
    for (int i = 0; i < 256; i++) {
			if (i <= teto)
				tabela[i] = (byte)binMin;
      else 
				tabela[i] = (byte)binMax;
    }

		ByteLookupTable binTab = new ByteLookupTable(0, tabela);
    LookupOp binaOp = new LookupOp(binTab, null);
		destWR = binaOp.filter(srcR, destWR); 
	  out = new BufferedImage(src.getColorModel(), destWR, false, null);

    return out;
  }

}
