/***********************************************************************************
BinarizaEntropia.java - Binariza uma imagem usando entropia do conjunto difuso.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Binariza uma imagem usando entropia do conjunto difuso.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa. A imagem *no* pode
     ser binria, seno, o programa no funcionar.

Sada:

  Painel com a imagem original e imagem binria.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.text.DecimalFormat;
import java.io.File;

// pacotes do AWT
import java.awt.image.ByteLookupTable;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class BinarizaEntropia extends JFrame {

  public static void main(String args[]) {
	
		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    BinarizaEntropia bina = new BinarizaEntropia(file);

    // Encerra a aplicao clicando no "close"
    bina.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    bina.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

  public BinarizaEntropia(File file) {

    // Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
    
    // Armazena arquivo imagem numa BufferedImage
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Binariza fuzzy: " + file.getName());

    dest = binOp(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  // Binariza imagem usando uma tabela de cores - lookup table
  public BufferedImage binOp(BufferedImage src) {
		
		int binMax = 255, binMin = 0, limiar = 0, w, h, melhorLimiar= -1, 
			mu0, mu1, soma, minSoma, ponto1, ponto2, tempLimiar;
    byte[] tabela = new byte[256];
		double[] hist = new double[256];

		// Variveis para acumular histograma de cada pixel
		// e seu complemento
		double[] acumHisto = new double[256];
		double[] acumCompHisto = new double[256];

		// Variveis para acumular peso de cada pixel
		// e seu complemento
		double[] pesoPixel = new double[256];
		double[] compPesoPixel = new double[256];

		double[] fuzzyNumber = new double[256];
		double maxFuzzyNumber = 0.0, delta;
		long MN;

    BufferedImage out = null; 
    Raster srcR = src.getRaster(); 
    WritableRaster destWR = src.copyData(null); 

		w = src.getWidth();
		h = src.getHeight();

    System.out.println("binOp:: w,h " + w+", "+h);

		// Calcula o histograma
		for (int i = 0; i < 256; i++) 
			hist[i] = 0.0;
		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++)
				hist[srcR.getSample(j, i, 0)] += 1.0;

		MN = h*w;

		// Histograma acumulativo
		acumHisto[0] = hist[0]; 
		pesoPixel[0] = 0;
		for (int i = 1; i < 256; i++) {
			acumHisto[i] = acumHisto[i-1] + hist[i];
			pesoPixel[i] = pesoPixel[i-1] + i*hist[i];
		}

		// Inverte histograma acumulativo
		acumCompHisto[255] = 0; 
		compPesoPixel[255] = 0;
		for (int i = 254; i >= 0; i--) {
			acumCompHisto[i] = acumCompHisto[i+1] + hist[i+1];
			compPesoPixel[i] = compPesoPixel[i+1] + (i+1)*hist[i+1];
		}

		for (tempLimiar = 1; tempLimiar < 255; tempLimiar++) {
			if (hist[tempLimiar] == 0.0)
				continue;
			if (acumHisto[tempLimiar] == 0.0)
				continue;
			if (acumCompHisto[tempLimiar] == 0.0)
				continue;

			// Calcula mdias
			mu0 = (int)(pesoPixel[tempLimiar]/acumHisto[tempLimiar] + 0.5);
			mu1 = (int)(compPesoPixel[tempLimiar]/acumCompHisto[tempLimiar] + 0.5);

			// Calcula grau de pertinncia difusa
			fuzzyNumber[tempLimiar] = fuzzy(hist, mu0, mu1, tempLimiar)/MN;

			// Mantem o menor nmero fuzzy
			maxFuzzyNumber = Math.max(fuzzyNumber[tempLimiar], maxFuzzyNumber);
			if (melhorLimiar < 0)
				melhorLimiar = tempLimiar;
			else
				if (fuzzyNumber[tempLimiar] < fuzzyNumber[melhorLimiar])
					melhorLimiar = tempLimiar;
		}

		// Encontra o melhor limiar
    System.out.println("binOp::melhorLimiar " + melhorLimiar);
		delta = fuzzyNumber[melhorLimiar] + (maxFuzzyNumber - fuzzyNumber[melhorLimiar])*0.05;
		ponto1 = (int)(melhorLimiar - delta);
		if (ponto1 <= 0)
			ponto1 = 1;
		ponto2 = (int)(melhorLimiar + delta);
		if (ponto2 >= 255)
			ponto2 = 254;
		minSoma = Integer.MAX_VALUE;

		for (int i = ponto1; i <= ponto2; i++) {
			soma = (int)(hist[i-1] + hist[i] + hist[i+1]);
			if (soma < minSoma) {
				tempLimiar = i;
				minSoma = soma;
			}
		}

		limiar = tempLimiar;
		System.out.println("Limiar:" + limiar);

		// Constri a nova tabela de cores
		for (int i = 0; i < 256; i++) {
			if (i < limiar)
				tabela[i] = (byte)binMin;
			else 
				tabela[i] = (byte)binMax;
		}

    ByteLookupTable binTab = new ByteLookupTable(0, tabela);
    LookupOp binaOp = new LookupOp(binTab, null);
		destWR = binaOp.filter(srcR, destWR); 
		out = new BufferedImage(src.getColorModel(), destWR, false, null);

    return out;
	}

	public double fuzzy(double[] hist, int mu0, int mu1, int limiar) {
		double E = 0;

		for (int i = 0; i < 255; i++) {
				E += Shannon(distMedia(i, mu0, mu1, limiar))*hist[i];
		}
		return E;
	}

	public double Shannon(double x) {

		if (x > 0.0 && x < 1.0)
			return (double)(-x*Math.log((double)x) - (1.0-x)*Math.log((double)(1.0-x)));
		else
			return 0.0;
	}

	public double distMedia(int g, int mu0, int mu1, int limiar) {
		double distancia, x;

		if (g <= limiar)	{
			x = 1.0 + ((double)Math.abs(g - mu0))/255.0;
			distancia = 1.0/x;
		} else {
			x = 1.0 + ((double)Math.abs(g - mu1))/255.0;
			distancia = 1.0/x;
		}
		if (distancia < 0.5 || distancia > 1.0)
			System.out.println("distMedia = " + distancia);
		return distancia;
	}

}
