/***********************************************************************************
BinarizaHisto.java - Binariza uma imagem usando histograma.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Binariza uma imagem usando histograma.
    Fonte: OTSU, N. A threshold selection method from grey-level histograms. IEEE 
           Transactions on Systems, Man, and Cybernetics. Vol. 9, no. 1, 
           pp. 377-393, 1979.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem binria.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.ByteLookupTable;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class BinarizaHisto extends JFrame {

  public static void main(String args[]) {
	
		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

    // Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    BinarizaHisto bina = new BinarizaHisto(file);

    // Encerra a aplicao clicando no "close"
    bina.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    bina.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

  public BinarizaHisto(File file) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Armazena arquivo imagem numa BufferedImage
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Binariza histo: " + file.getName());

    dest = binOp(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  // Binariza imagem usando uma tabela de cores - lookup table
  public BufferedImage binOp(BufferedImage src) {
		
		int binMax = 255, binMin = 0, limiar = 0, w, h, nc, m, t, indice1, indice2;
    byte[] tabela = new byte[256];
    int[]  histo  = new int[260];
		double y, z, mediaTotal, varianciaTotal;
		double[] propab = new double[260];
		long nroPixel;

    BufferedImage out = null; 
    Raster srcR = src.getRaster(); 
    WritableRaster destWR = src.copyData(null); 

		w = src.getWidth();
		h = src.getHeight();

		nroPixel = w*h;
		for (int i = 0; i < 260; i++) {
			histo[i] = 0;
			propab[i] = 0.0;
		}

		// Calcula o histograma
		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				nc = srcR.getSample(j, i, 0);
				histo[nc+1] += 1;
			}

		// Normaliza na distribuio
		for (int i = 1; i <= 256; i++)
			propab[i] = (float)histo[i]/(float)nroPixel;

		// Calcula mdia e varincia total
		mediaTotal = media(propab, 256);
		varianciaTotal = 0.0;
		for (int i = 1; i <= 256; i++)
			varianciaTotal += (i-mediaTotal)*(i-mediaTotal)*propab[i];

		// Acha primeiro e ltimo indices
		indice1 = -1; indice2 = -1;
		for (int i = 1; i <= 256; i++) {
			if ((indice1 < 0) && (propab[i] > 0.0))
				indice1 = i;
			if (propab[i] > 0.0)
				indice2 = i;
		}
		z = -1.0;
		m = -1;

		// Calcula o valor de NU
		for (int i = indice1; i <= indice2; i++) {
			y = nu(propab, i, mediaTotal, varianciaTotal);
			if (y >= z) {
				z = y;
				m = i;
			}
		}

		limiar = m;
		System.out.println("Limiar:" + limiar);

		// Constri a nova tabela de cores
		for (int i = 0; i < 256; i++) {
			if (i < limiar)
				tabela[i] = (byte)binMin;
			else 
				tabela[i] = (byte)binMax;
		}

    ByteLookupTable binTab = new ByteLookupTable(0, tabela);
    LookupOp binaOp = new LookupOp(binTab, null);
		destWR = binaOp.filter(srcR, destWR); 
		out = new BufferedImage(src.getColorModel(), destWR, false, null);

    return out;
	}

	public double media(double[] p, int k) {
		double x=0.0;

		for (int i = 1; i <= k; i++)
			x += (float)i*p[i];
		return x;
	}

	public double nu(double[] p, int k, double mediaTot, double varianciaTot) {
		double varianciaInterClasses, omegaZero;

		omegaZero = acumProb(p, k);
		varianciaInterClasses = mediaTot*omegaZero - media(p, k);
		varianciaInterClasses = varianciaInterClasses*varianciaInterClasses;
		omegaZero = omegaZero*(1.0 - omegaZero);
		if (omegaZero > 0)
			varianciaInterClasses = varianciaInterClasses/omegaZero;
	  else 
			varianciaInterClasses = 0.0;
		return varianciaInterClasses/varianciaTot;
	}

	public double acumProb(double[] p, int k) {
		double x = 0.0;

		for (int i = 1; i <= k; i++)
			x += p[i];
		return x;
	}

}
