/***********************************************************************************
BinarizaIterativo.java - Binariza uma imagem usando mtodo iterativo.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Binariza uma imagem usando mtodo iterativo.
    Fonte: RIDLER, T. W.; CALVARD, S. Picture thresholding using an iterative 
           selection method. IEEE Transactions on Systems, Man, and Cybernetics. 
           Vol. 8, no. 1, pp. 630-632, 1983.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem binria.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.text.DecimalFormat;
import java.io.File;

// pacotes do AWT
import java.awt.image.ByteLookupTable;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class BinarizaIterativo extends JFrame {

  public static void main(String args[]) {
	
		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando o parmetro
    BinarizaIterativo bina = new BinarizaIterativo(file);

    // Encerra a aplicao clicando no "close"
    bina.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    bina.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

  public BinarizaIterativo(File file) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Armazena arquivo imagem numa BufferedImage
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Binariza iterativo: " + file.getName());

    dest = binOp(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  // Binariza imagem usando uma tabela de cores - lookup table
  public BufferedImage binOp(BufferedImage src) {

		int binMax = 255, binMin = 0, limiar = 0, w, h;
    byte[] tabela = new byte[256];

    BufferedImage out = null; 
    Raster srcR = src.getRaster(); 
    WritableRaster destWR = src.copyData(null); 

		w = src.getWidth();
		h = src.getHeight();

		// Algoritmo #1 - mais rpido
		// variveis
		long Told, media, a, b, c, d, N;
		int hist[];

		// Calcula mdia e histograma
		hist = new int[257];
		for (int i = 0; i < 256; i++)
			hist[i] = 0;

		N = w*h;
		media = 0;
		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				hist[srcR.getSample(j, i, 0)] += 1;
				media = media + (srcR.getSample(j, i, 0));
	  }
		media = media/N;

		do {
			Told = media;
			a = 0; b = 0;
			for ( int i = 0; i <= Told; i++) {
				a += i*hist[i];
				b += hist[i];
			}
			b += b;
			c = 0; d = 0;
			for (int i = (int)Told + 1; i < 256; i++) {
				c += i*hist[i];
				d += hist[i];
			}
			d += d;
			if (b == 0)
				b = 1;
			if (d == 0)
				d = 1;
			media = a/b + c/d;
		} while (media != Told);
		// fim do Algoritmo #1

		// Algoritmo #2
		// variveis
		/*
		double	media, 
						tb, 				// mdia dos pixels abaixo do limiar inicial
						total,
						t2;					// varivel auxiliar
		long  nroPixel,			// nmero de pixels da imagem
					no,						// pixels com valor maior que limiar inicial
					nb;						// pixels com valor menor que limiar inicial;

		nroPixel = w*h;	total = 0.0;
		for (int i = 0; i < h; i++) 
			for (int j = 0; j < w; j++)
				total = total + srcR.getSample(j, i, 0);
		media = total/nroPixel;

		while (nroPixel > 0) {
			no = 0; nb = 0; tb = 0.0; total = 0.0;
			for (int i = 0; i < h; i++) 
				for (int j = 0; j < w; j++) {
					if ((float)srcR.getSample(j, i, 0) >= media ) {
						total = total + (float)srcR.getSample(j, i, 0);
						no++;
					} else {
						tb = tb + (float)srcR.getSample(j, i, 0);
						nb++;
					}
				}

			if (no == 0) 
				no = 1;
			if (nb == 0) 
				nb = 1;
			t2 = (tb/(float)nb + total/(float)no )/2.0;
			if (t2 == media) 
				nroPixel = 0;
			media = t2;
		}
		fim do Algoritmo #2 */

		limiar = (int) media;
		System.out.println("Limiar:" + limiar);

    // Constri a nova tabela de cores
    for (int i = 0; i < 256; i++) {
			if (i < limiar)
				tabela[i] = (byte)binMin;
      else 
				tabela[i] = (byte)binMax;
    }

    ByteLookupTable binTab = new ByteLookupTable(0, tabela);
    LookupOp binaOp = new LookupOp(binTab, null);
		destWR = binaOp.filter(srcR, destWR); 
		out = new BufferedImage(src.getColorModel(), destWR, false, null);

    return out;
	}
}
