/***********************************************************************************
BinarizaZadeh.java - 'defuzzify' uma imagem usando lgica difusa de Zadeh

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    'defuzzify' uma imagem usando lgica difusa de Zadeh.

Parmetros de entrada:

  1. nome do arquivo imagem e nmero de interaes. O parmetro 'interaes' 
     pode ser 3.

Sada:

  Painel com a imagem original e imagem 'defuzzified'.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.GridLayout;
import java.awt.image.Raster;
import java.awt.image.LookupOp;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.image.ByteLookupTable;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class BinarizaZadeh extends JFrame {

  public static void main(String args[]) {

		int interacao = 1;

		// Certifica a passagem dos parmetros
    if (args.length != 2) {
			String msga = "Uso: java -cp . BinarizaZadeh <arquivo_imagem> <interacoes>";
      System.out.println(msga);
      System.exit(0);
    }

    try {
       interacao = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
       System.out.println("Valor do parametro 'interacoes' invalido");
       System.exit(0);
    }

    if (interacao <= 0) {
       System.out.println("interacoes: valor invalido. Deve ser > 0");
       System.exit(0);
    }

		// Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    BinarizaZadeh bina = new BinarizaZadeh(args[0], interacao);

    // Encerra a aplicao clicando no "close"
    bina.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    bina.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

  public BinarizaZadeh(String aFile, int interacao) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Binariza Zadeh: " + file.getName());

    dest = Zadeh(src, interacao);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  // Binariza imagem usando uma tabela de cores - lookup table
	public BufferedImage Zadeh(BufferedImage src, int interacoes) {
		
		double[][] buf1, buf2;
		int w, h;
		w = src.getWidth();
		h = src.getHeight();

    BufferedImage out = null; 

		buf1 = normaliza(src);
		buf2 = intensificaZadeh(buf1, interacoes, w, h);
		out  = binariza(buf2, w, h);

		// Salva arquivo formato PNG
		/* Se voc quiser gravar o arquivo, modificar o diretrio
		try {
      System.out.println("Gravando arquivo 'binZadeh.png' em 'C:\\temp\\'");
			ImageIO.write(out, "png", new File("C:\\temp\\binZadeh.png"));
		} catch (IOException e) {
			System.out.println("Problema gravando arquivo ");
      System.exit(0);
		}
    */
	
    return out;
	}

	public double[][] normaliza(BufferedImage src) {

		int w, h;
		double[][] mu;

		Raster srcR = src.getRaster();
		w = src.getWidth();
		h = src.getHeight();
		mu = new double[h][w];

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
	      mu[i][j] = (double)srcR.getSample(j, i, 0)/255;
	    }
		return mu;
	}

	public double[][] intensificaZadeh(double[][] mu, int interacoes,
		int w, int h) {

		double val;
		double Bp = 0.5;

		for (int n = 1; n <= interacoes; n++) {
			for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					val = mu[i][j];
		      if (val > Bp)
						mu[i][j] = 1 - 2*Math.pow((1-val), 2);
		      else
						mu[i][j] = 2*Math.pow(val, 2);
				}
			}
		}
		return mu;
	}
	
	public BufferedImage binariza(double[][] mu, int w, int h) {

		int tipo;
		long val;

		tipo = BufferedImage.TYPE_BYTE_GRAY;
		BufferedImage destImage =	new BufferedImage(w, h, tipo);
		WritableRaster destWR = destImage.getRaster();

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				val = Math.round(mu[i][j]*255);
				destWR.setSample(j, i, 0, val);
	    }
		return destImage;
	}	
		
}
