/***********************************************************************************
Bivariada.java - Calcula estatstica bivariada para imagens monocromaticas.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula estatstica bivariada para imagens monocromaticas.

Desenvolvido por:

    Jos Iguelmar Miranda


Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/
//
// Calcula estatstica bivariada para imagens monocromaticas.
//

import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grafica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Bivariada extends JFrame {

   public static void main(String args[]) {

      // Certifica a passagem da imagem
      if (args.length != 2) {
         String msg = "Uso: java -cp . Bivariada <imagem1> <imagem2>";
         System.out.println(msg);
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
      JFrame f = new Bivariada(args[0], args[1]);

      // Encerra a aplicao clicando no "close"
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.show();
   }

   public Bivariada(String aFile1, String aFile2) {

      // Define objeto BufferedImage para encasular a imagem
      BufferedImage src1 = null, src2 = null;

      // Certifica que as imagens existem
			File file1 = new File(aFile1);
      try {
         src1 = ImageIO.read(file1);
      } catch (Exception e) {
         System.out.println("Erro lendo arquivo: " + file1);
         System.exit(0);
      }

			File file2 = new File(aFile2);
      try {
         src2 = ImageIO.read(file2);
      } catch (Exception e) {
         System.out.println("Erro lendo arquivo: " + file2);
         System.exit(0);
      }

      if ((src1.getWidth() != src2.getWidth()) || (src1.getHeight() != src2.getHeight())) {
         System.out.println("Imagens com dimensoes diferentes");
         System.exit(0);
      }
		
      // Atribui nome ao frame
      setTitle("Estatistica Bivariada: " + file1.getName() + ", " + file2.getName());

      // Chama funo

      String msgBi = "\nEstatistica bivariada das imagens: ";
      System.out.println(msgBi + file1.getName() + ", " + file2.getName());
      imagemEstatBi(src1, src2);

      // Cria gridLayout de 1 x 2
      getContentPane().setLayout(new GridLayout(1, 2));

      // Imagem 1
      // Define GUI com objetos do Swing
      JLabel img1 = new JLabel(new ImageIcon(src1));
      getContentPane().add(new JScrollPane(img1));

      // Imagem 2
      // Define GUI com objetos do Swing
      JLabel img2 = new JLabel(new ImageIcon(src2));
      getContentPane().add(new JScrollPane(img2));
      setSize(src1.getWidth(), src1.getHeight());
   }

   public void imagemEstatBi(BufferedImage aSrc1, BufferedImage aSrc2) {
    
      // Variveis para a banda 1 (aSrc1)
      int    nc_b1, min_b1 = 255, max_b1 = 0;
      long   sum_b1 = 0, sum_sqr_b1 = 0;
      double dVAR_b1 = 0.0, dMEAN_b1 = 0.0, dDESVP_b1 = 0.0, dSECOND_b1 = 0.0;

      // Variveis para a banda 2 (aSrc2)
      int    nc_b2, min_b2 = 255, max_b2 = 0;
      long   sum_b2 = 0, sum_sqr_b2 = 0;
      double dVAR_b2 = 0.0, dMEAN_b2 = 0.0, dDESVP_b2 = 0.0, dSECOND_b2 = 0.0;
		
      // Variveis globais
      long   sum_b1xb2 = 0, total = 0;
      double dCOVAR = 0.0, dCORREL = 0.0, dSUMPROD = 0.0;

      Raster src1R = aSrc1.getRaster();
      Raster src2R = aSrc2.getRaster();
      int h = aSrc1.getHeight();
      int w = aSrc1.getWidth();		
      for (int i = 0; i < h; i++) {
        for (int j = 0; j < w; j++) {
            // Pega pixels da banda 1
            nc_b1 = src1R.getSample(j, i, 0);
            
            // Pega pixels da banda 2
            nc_b2 = src2R.getSample(j, i, 0);

            // Acumula valores para clculos univariados da banda 1
            sum_b1  += nc_b1;
            sum_sqr_b1 += nc_b1*nc_b1;

            // Acumula valores para clculos univariados da banda 2
            sum_b2  += nc_b2;
            sum_sqr_b2 += nc_b2*nc_b2;

            // Acumula valores para calculo da covarincia
            sum_b1xb2 += nc_b1*nc_b2;
            total++;

            // Acha mximos e mnimos
            min_b1 = Math.min(nc_b1, min_b1);
            max_b1 = Math.max(nc_b1, max_b1);
            min_b2 = Math.min(nc_b2, min_b2);
            max_b2 = Math.max(nc_b2, max_b2);
        }
      }

      // Valores univariados da banda 1
      dMEAN_b1   = (double)sum_b1/total;
      dSECOND_b1 = (double)sum_sqr_b1/total;
      dVAR_b1    = dSECOND_b1 - dMEAN_b1*dMEAN_b1;
      dDESVP_b1  = Math.sqrt(dVAR_b1);
		
      // Valores univariados da banda 2
      dMEAN_b2   = (double)sum_b2/total;
      dSECOND_b2 = (double)sum_sqr_b2/total;
      dVAR_b2    = dSECOND_b2 - dMEAN_b2*dMEAN_b2;
      dDESVP_b2  = Math.sqrt(dVAR_b2);

      // Valores bivariados
      dSUMPROD = (double)(sum_b1xb2 - (sum_b1*sum_b2)/total);
      dCOVAR = (double)(dSUMPROD/(total - 1));
      dCORREL = dCOVAR/(dDESVP_b1*dDESVP_b2);
		
      String padrao = "##.##";
      DecimalFormat formato = new DecimalFormat(padrao);
      String media_b1 = formato.format(dMEAN_b1);
      String media_b2 = formato.format(dMEAN_b2);
      String varia_b1 = formato.format(dVAR_b1);
      String varia_b2 = formato.format(dVAR_b2);
      String desvp_b1 = formato.format(dDESVP_b1);
      String desvp_b2 = formato.format(dDESVP_b2);
      String covar_bb = formato.format(dCOVAR);
      String correl_b = formato.format(dCORREL);
		
      System.out.print("\nEstatisticas univariadas:\n"); 
      System.out.print("\nBanda\t1\t2");
      System.out.print("\nmedia\t" + media_b1 + "\t" + media_b2);
      System.out.print("\nvar.\t" + varia_b1 + "\t" + varia_b2);
      System.out.print("\ndesv.\t" + desvp_b1 + "\t" + desvp_b2);
      System.out.print("\nminimo\t" + min_b1   + "\t" + min_b2);
      System.out.print("\nmaximo\t" + max_b1   + "\t" + max_b2);
      System.out.println("\n\nEstatisticas bivariadas:");
      System.out.println("\nMatriz de variancia-covariancia");
      System.out.print("\nBanda\t1\t2");
      System.out.print("\n1\t" + varia_b1);
      System.out.print("\n2\t" + covar_bb + "\t" + varia_b2 + "\n\n");
      System.out.println("Matriz de correlacao");
      System.out.print("\nBanda\t1\t2");
      System.out.print("\n1\t1");
      System.out.print("\n2\t" + correl_b + "\t1\n\n");
   }

}
