/***********************************************************************************
ContornoI.java - Implementa operao morfolgica de contorno interno de um objeto.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa operao morfolgica de contorno interno de um objeto.
    O elemento estruturante (mFilter[][]) utiliza a forma "arredondada".
    Para obter efeitos diferentes, mudar os valores '1' nesse filtro.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. tamanho do elemento estruturante: 3 (3x3), 5 (5x5) ou 7 (7x7).

Sada:

  Painel com a imagem original e contorno da imagem.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.text.DecimalFormat;
import java.io.File;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.GridLayout;
import java.awt.Color;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ContornoI extends JFrame {

	public static void main(String args[]) {

		int dimFiltro = 3;

		// Certifica a passagem dos parmetros
		if (args.length != 2) {
			String msg1 = "Uso: java -cp . ContornoI <arquivo_imagem> <filtro>";
			String msg2 = "<filtro> = 3 para dimensao 3x3";
			String msg3 = "<filtro> = 5 para dimensao 5x5";
			String msg4 = "<filtro> = 7 para dimensao 7x7";
			System.out.println(msg1);
			System.out.println(msg2);
			System.out.println(msg3);
			System.out.println(msg4);
			System.exit(0);
    }

		// Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// L valor do filtro
    try {
        dimFiltro = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro <filtro> invalido");
        System.exit(0);
    }

    if ((dimFiltro != 3) && (dimFiltro != 5) && (dimFiltro != 7)) {
        System.out.println("<filtro>: valor invalido. Deve ser [3|5|7]");
        System.exit(0);
    }

    // Chama o construtor passando os parmetros
    JFrame f = new ContornoI(args[0], dimFiltro);
 
		// Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
  }

  public ContornoI(String aFile, int dimFiltro) {

    // Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;

		// Abre arquivo imagem
    File file = new File(aFile);
		src = abreImagem(file);

    // Atribui nome ao frame
    setTitle("Morfologia - Contorno interno: " + file.getName());

    // Chama funo
    dest = contornoOp(src, dimFiltro);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  public BufferedImage contornoOp(BufferedImage src, int dimFiltro) {
  
		int srcPixel, src_red, fltPixel, flt_red, red, w, h, tipo, 
				mFilter[][] = new int[1][1];
		Color c_src, c_flt;
		BufferedImage gray = null, dst = null;

		// Define elemento estruturante
		switch(dimFiltro){
			case 3:
				mFilter = new int[][]{
					{0,1,0},
					{1,1,1},
					{0,1,0}
				};
				break;
			case 5:
				mFilter = new int[][]{
					{0,1,1,1,0},
					{1,1,1,1,1},
					{1,1,1,1,1},
					{1,1,1,1,1},
					{0,1,1,1,0}
				};
				break;
			case 7:
				mFilter = new int[][]{
					{0,0,0,1,0,0,0},
					{0,1,1,1,1,1,0},
					{1,1,1,1,1,1,1},
					{1,1,1,1,1,1,1},
					{1,1,1,1,1,1,1},
					{0,1,1,1,1,1,0},
					{0,0,0,1,0,0,0},
			 	};
		}		

		h = src.getHeight();
		w = src.getWidth();

		tipo = BufferedImage.TYPE_BYTE_GRAY;
    dst = new BufferedImage(w, h, tipo);

		// Aplica filtro de eroso na imagem original
    System.out.println("Calculando filtro de erosao...");
    BufferedImage filtro = erodeOp(src, mFilter);

    // Calcula contorno interno = src - filtro eroso
		System.out.println("Calculando contorno interno...");
    for (int i = 0; i < h; i++) {
			for (int j = 0; j < w; j++) {
				srcPixel  = src.getRGB(j, i);
        c_src     = new Color(srcPixel);
        src_red   = c_src.getRed();

        fltPixel  = filtro.getRGB(j, i);
        c_flt     = new Color(fltPixel);
        flt_red   = c_flt.getRed();
        red   = src_red - flt_red;
				if (red < 0) {
					red = 0;
				}
        dst.setRGB(j, i, new Color(red, red, red).getRGB());
      }
    }

    // Retorna imagem com contorno interno
    return dst;
   }
  
   public BufferedImage erodeOp(BufferedImage in, int afiltro[][]) {

		// Como imagem deve ser binria, ou mesmo em nvel de cinza,
		// basta pegar o valor do pixel de um canal.
    int min_red, srcPixel, red, tipo, w, h, limite;
    Color c;

		h = in.getHeight();
		w = in.getWidth();
		limite = afiltro.length/2;
		tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage dst = new BufferedImage(w, h, tipo);

		for (int i = 0; i < h; i++) {
			for (int j = 0; j < w; j++) {
    		min_red = 255;
				for (int k = -limite; k <= limite; k++) {
					for (int n = -limite; n <= limite; n++) {
						if (afiltro[n+limite][k+limite] == 1) {
							if (((j+n >= 0) && (i+k >= 0)) && ((j+n < w) && (i+k < h)))	{
        				srcPixel = in.getRGB(j+n, i+k);
								c     = new Color(srcPixel);
								red   = c.getRed();
								min_red = Math.min(red, min_red);
							}
						}
					}
          dst.setRGB(j, i, new Color(min_red, min_red, min_red).getRGB());
				}
			}
    }

    return dst;
  }

  public BufferedImage abreImagem(File arquivoImagem) {
	
		BufferedImage src = null;

    // Certifica que a imagem existe
    try {
        src = ImageIO.read(arquivoImagem);
    } catch(Exception e) {
        System.out.println("Imagem '" + arquivoImagem + "' nao existe.");
        System.exit(0);
    }
		return src;
	}

}
