/***********************************************************************************
Equaliza.java - Implementa vrias transformaes no lineares e uma linear

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa vrias transformaes no lineares e uma linear 
    (equao 8.24 do livro), o mtodo mini-maxi.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem transformada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.image.Raster;
import java.awt.BorderLayout;
import java.awt.Graphics2D;
import java.awt.Graphics;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JFrame;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Equaliza extends JComponent {

	// Define objeto BufferedImage para encapsular a imagem
  BufferedImage src = null;
  int w, h, hist[] = new int[256], max_hist = 0;
  private JComboBox opcoes;

  public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando o nome do arquivo imagem
    JFrame f = new JFrame("Equaliza"); 
    f.getContentPane().add(new Equaliza(file));

    // Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(300, 320);
    f.setVisible(true);
	}
  
  public Equaliza(File file) {
		int nc;

    // Armazena arquivo imagem numa BufferedImage
    try {
			src = ImageIO.read(file);
    } catch (Exception e) {
			System.out.println("Erro lendo arquivo: " + file);
      System.exit(0);
    }
	
		Raster srcR = src.getRaster();
    w = src.getWidth();
    h = src.getHeight();

		// Calcula histograma
    for (int i = 0; i < h; i++) {
			for (int j = 0; j < w; j++) {
				nc = srcR.getSample(j, i, 0);
        hist[nc]++;
      }
    }
		
    for (int i = 0; i < 256; i++) {
			if (hist[i] > max_hist)
				max_hist = hist[i];
    }

    // Define GUI com objetos do Swing
    setLayout(new BorderLayout());

    // Cria um painel para hospedar o combo box
    JPanel controle = new JPanel();

    // Cria o combo box com os nomes das opes
    opcoes = new JComboBox(new String[] {"Imagem", "Equaliza", 
                              "MiniMaxi", "Log", "Raiz", "Porcentagem"});

    // Processa mudana de opo
    opcoes.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
				repaint();
      }
    });
    controle.add(opcoes);
    add(controle, BorderLayout.SOUTH);
  }

	public void paintComponent(Graphics gc) {
		super.paintComponent(gc);
    Graphics2D g2 = (Graphics2D) gc;
    int nc, pixelOut;

    // Recupera a opo do combo box
    String opcao = (String)opcoes.getSelectedItem();
    Raster srcR = src.getRaster();

    // Transformao no linear - Equalizao de histograma
    //
    if (opcao.equals("Equaliza")) {
			int dim, histoEq[] = new int[256], tipo;
      long eq_time = System.currentTimeMillis();
      double soma = 0.0, normHisto[] = new double[256];

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage eqlImage = new BufferedImage(w, h, tipo);
      WritableRaster eqlWR = eqlImage.getRaster();
      dim = w*h;
      for (int i = 0; i < 256; i++) {
				soma = 0.0;
				for (int j = 0; j <= i; j++) {
					// se no usar o (double), valores = 0
          soma += (double)hist[j]/dim;
        }
        histoEq[i] = (int)(255*soma + .5);
      }
			
      for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
          pixelOut = histoEq[nc];
					eqlWR.setSample(j, i, 0, pixelOut);
				}
      }

			g2.drawImage(eqlImage, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Equaliza: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
		}

    // Transformao linear - mtodo mini-maxi
    // 
    if (opcao.equals("MiniMaxi")) {
			double autoScale = 0.0;
      long ec_time = System.currentTimeMillis();
      int grayMax = 255, grayMin = 0, sMin = 255, sMax = 0, tipo;
			
      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage autoImage = new BufferedImage(w, h, tipo);
      WritableRaster autoWR = autoImage.getRaster();
      for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
					sMax = Math.max(nc, sMax);
					sMin = Math.min(nc, sMin);
        }
      }
      if ((sMax - sMin) != 0)
				autoScale = (double)(grayMax - grayMin)/(sMax - sMin);
      else
        autoScale = 1.0;
      for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
          pixelOut = (int)(autoScale*(nc - sMin) + grayMin);
          autoWR.setSample(j, i, 0, pixelOut);
        }
      }

			g2.drawImage(autoImage, 0, 0, null);
      ec_time = System.currentTimeMillis() - ec_time;
      String msg = "MiniMaxi: tempo de execucao ";
      System.out.println( msg + ec_time + " milisseg.");
      return;
    }

    // Transformao no linear - logaritmo
	  //
    if (opcao.equals("Log")) {
      long el_time = System.currentTimeMillis();
      int grayMax = 255, grayMin = 0, tipo;
      double val;

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage logImage = new BufferedImage(w, h, tipo);
      WritableRaster logWR = logImage.getRaster();
			
      for (int i = 0; i < h; i++) {
        for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
          val = 31.875*log2((double)(nc + 1));
          pixelOut = (int)val;
          if (pixelOut < grayMin)
						pixelOut = grayMin;
          else if (pixelOut > grayMax)
						pixelOut = grayMax;
          logWR.setSample(j, i, 0, pixelOut);
        }
      }

			g2.drawImage(logImage, 0, 0, null);
      el_time = System.currentTimeMillis() - el_time;
      String msg = "Log: tempo de execucao ";
      System.out.println(msg + el_time + " milisseg.");
      return;
    }

    // Transformao no linear - raiz quadrada
	  //
    if (opcao.equals("Raiz")) {
      long el_time = System.currentTimeMillis();
      int grayMax = 255, grayMin = 0, tipo;
      double val;

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage raizImage = new BufferedImage(w, h, tipo);
      WritableRaster raizWR = raizImage.getRaster();
			
      for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
          val = 16*Math.sqrt((double)(nc));
          pixelOut = (int)val;
          if (pixelOut < grayMin)
						pixelOut = grayMin;
          else if (pixelOut > grayMax)
            pixelOut = grayMax;
          raizWR.setSample(j, i, 0, pixelOut);
				}
      }

			g2.drawImage(raizImage, 0, 0, null);
      el_time = System.currentTimeMillis() - el_time;
      String msg = "Raiz: tempo de execucao ";
      System.out.println(msg + el_time + " milisseg.");
      return;
    }

    // Transformao no linear - porcentagem -1 e +1 desvio padro
    //
    if (opcao.equals("Porcentagem")) {
			long ep_time = System.currentTimeMillis();
      int sMax = 0, sMin = 255, grayMax = 255, denom, tipo;
      long 	total, sum, sum_sqr;
      double dSECOND, dVAR = 0.0, dMEAN, dDESVP, val;
      sum = sum_sqr = total = 0;

      for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
					nc = srcR.getSample(j, i, 0);
					sMax = Math.max(nc, sMax);
					sMin = Math.min(nc, sMin);
          sum  += nc;
          sum_sqr += nc*nc;
          total++;
        }
      }
      dMEAN   = (double)sum/total;
      dSECOND = (double)sum_sqr/total;
      dVAR    = dSECOND - dMEAN*dMEAN;
      dDESVP  = Math.sqrt(dVAR);
      sMax = (int)(sMax - dDESVP);
      sMin = (int)(sMin + dDESVP);
			
      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage percentImage = new BufferedImage(w, h, tipo);
      WritableRaster percentWR = percentImage.getRaster();
			
      denom = sMax - sMin;
      for (int i = 0; i < h; i++) {
         for (int j = 0; j < w; j++) {
            nc = srcR.getSample(j, i, 0);
						if (nc < sMin)
							nc = sMin;
						else if (nc > sMax)
							nc = sMax;
            val = (double)grayMax*(nc - sMin)/denom;
            pixelOut = (int)val;
            percentWR.setSample(j, i, 0, pixelOut);
         }
      }

			g2.drawImage(percentImage, 0, 0, null);
      ep_time = System.currentTimeMillis() - ep_time;
      String msg = "Porcentagem: tempo de execucao ";
      System.out.println(msg + ep_time + " milisseg.");
      return;
    }

    if (opcao.equals("Imagem")) {
       g2.drawImage(src, 0, 0, null);
		}
	}
	
  public static double log2(double parm) {
     return Math.log(parm)/Math.log(2.0);
  }
}
