/***********************************************************************************
Esqueleto.java - Implementa o algoritmo de esqueletizao de uma imagem binria

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa o algoritmo de esqueletizao de uma imagem binria.
    Valores pretos (0) indicam o objeto e branco (1 = 255) o fundo.
    Fonte: Parker, J.R. Algorithms for image processing and computer vision. 
           New York, NY: John Wiley & Sons, 1997. 417p. pp. 203-218.
           
    Para gerar imagens binrias, usar o programa "Paint" do Microsoft Windows e 
    escolher as opcoes:
      Imagem --> Atributos --> Cores --> "Preto e branco"

    Os algoritmos de pr-processamento esto comentados, pois eles 
    pouco influenciaram no resultado final de esqueletizao.
    1. pre_smooth: se a imagem tem pequenas ramificaes nas bordas,
		   a sua funo  limp-las.  Executar o programa com e sem pre_smooth 
       na imagem.
    2. aae: reala ngulos agudos, mas s em caracteres de pequena
		   espessura. Exemplo, executar o programa com e sem aae.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e esqueleto da imagem.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.text.DecimalFormat;
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;
import java.awt.Color;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Esqueleto extends JFrame {

	int NORTH = 1, SOUTH = 3;

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
		Esqueleto skull = new Esqueleto(file);

		// Encerra a aplio clicando no "close"
		skull.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		skull.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		DecimalFormat df = new DecimalFormat("0.00");
		System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

	public Esqueleto(File file) {

		// Define objeto BufferedImage para encapsular a imagem
		BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Armazena arquivo imagem numa BufferedImage
   	try {
     		src = ImageIO.read(file);
   	} catch(Exception e) {
     		System.out.println("Imagem '" + file + "' nao existe.");
     		System.exit(0);
	 	}

		// Atribui nome ao frame
		setTitle("Esqueleto: " + file.getName());

		// Chama funo
 		System.out.println("Calculando esqueletizacao... ");
    dest = filtro(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

	public BufferedImage filtro(BufferedImage src) {
	
		int w, h, r, pixel, tipo;
		Color cor;
		BufferedImage srcCopy = null;

		w = src.getWidth();
		h = src.getHeight();
		tipo = BufferedImage.TYPE_INT_RGB;
		srcCopy = new BufferedImage(w, h, tipo);

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				pixel = src.getRGB(j, i);
				cor = new Color(pixel);
				r = cor.getRed();
				// Imagem preto e branco, no gerada com o Paint, o resultado
				// pode ser estranho.  Mudar if (r > 50).
				// Embora no seja visvel, existem nuances de cores no entorno
				// dos objetos da imagem. Testar os dois ifs com diferentes
				// arquivos de imagem
				// if (r > 0)
				if (r > 50)
					srcCopy.setRGB(j, i, new Color(255,255,255).getRGB());
				else
					srcCopy.setRGB(j, i, new Color(0,0,0).getRGB());
			}

		// pre-processamento da imagem 
		// pre_smooth(srcCopy);
		// aae(srcCopy);
		// Esqueletizao
		esqueleto(srcCopy);
		return srcCopy;
	}     

	// mtodo Stentiford de suavizao das bordas 
	public void pre_smooth(BufferedImage im) {

		int w, h, r, pixel;
		Color cor;
		
		w = im.getWidth();
		h = im.getHeight();

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				pixel = im.getRGB(j, i);
				cor = new Color(pixel);
				r = cor.getRed();
				if (r == 0)
					if (snays(im, i, j) <= 2 && Yokoi(im, i, j) < 2)
						im.setRGB(j, i, new Color(2,2,2).getRGB());
			}

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				pixel = im.getRGB(j, i);
				cor = new Color(pixel);
				r = cor.getRed();
				if (r == 2)
					im.setRGB(j, i, new Color(1,1,1).getRGB());
			}

	}

	// enfatiza ngulos agudos cf Stentiford
	public void aae(BufferedImage im) {

		int w, h, again = 0, pixel, r;
		Color cor;

		w = im.getWidth();
		h = im.getHeight();

		for (int k = 5; k >= 1; k -= 2) {
			for (int i = 2; i < h-2; i++)
				for (int j = 2; j < w-2; j++) {
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					r = cor.getRed();
					if (r == 0)
						match_du(im, i, j, k);
				}

			for (int i = 2; i < h-2; i++)
				for (int j = 2; j < w-2; j++) {
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					r = cor.getRed();
					if (r == 2) {
						again = 1;
						im.setRGB(j, i, new Color(1,1,1).getRGB());
					}
				}
			if (again == 0)
				break;
		}

	}

	// nmero de vizinhos 0 do pixel
	public int snays(BufferedImage im, int r, int c) {

		int red, pixel, k = 0;
		Color cor;

		for (int i = r - 1; i <= r + 1; i++)
			for (int j = c - 1; j <= c + 1; j++) {
				pixel = im.getRGB(j, i);
				cor = new Color(pixel);
				red = cor.getRed();
				if ((i != r) || (j != c))
					if (red == 0)
						k++;
			}
		return k;
	}

	// medida de conectividade de Yokoi
	public int Yokoi(BufferedImage im, int r, int c) {

		// Armazena os pixels de uma janela 3x3.
		// Comea no centro e gira no sentido anti-horrio.
    int[][] janela = {
			{0,0}, {1,0}, {1,-1}, {0,-1}, {-1,-1}, {-1,0}, {-1,1}, {0,1}, {1,1}
		};

		int[] N = new int[9];
		int k, i1, i2, red, pixel, lin, col;
		Color cor;

		for (int v = 0; v < 9; v++){
			col = c + janela[v][0]; //coluna    		
			lin = r + janela[v][1]; //linha
			pixel = im.getRGB(col, lin);
			cor = new Color(pixel);
			red = cor.getRed();
			if (red != 0)
				N[v] = red;
		}	

		k = 0;
		for (int i = 1; i < 8; i += 2) {
			i1 = i + 1;
			if (i1 > 8)
				i1 -= 8;
			i2 = i + 2;
			if (i2 > 8)
				i2 -= 8;
			k += (N[i] - N[i]*N[i1]*N[i2]);
		}
		return k;

	}

	// algoritmo de Zhang-Suen com remoo de efeitos dente-de-serra
	// de Holt
	public void esqueleto(BufferedImage im) {

		// Armazena quatro vizinhos perpendiculares ao pixel central
		// Sentido anti-horrio.
    int[][] janelaAH = {
			{1,0}, {0,-1}, {-1,0}, {0,1}
		};

    // Sentido horrio.
    int[][] janelaH = {
			{0,-1}, {1,0},  {0,1}, {-1,0}
		};
		int ared[] = new int[6];
		int w, h, k, pixel, red, lin, col, iJanela;

		Color cor;
		boolean again = true;

		w = im.getWidth();
		h = im.getHeight();

		//BufferedImage tmp = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_GRAY);
		BufferedImage tmp = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				pixel = im.getRGB(j, i);
				cor = new Color(pixel);
				red = cor.getRed();
				if (red > 0)
					im.setRGB(j, i, new Color(0,0,0).getRGB());
				else
					im.setRGB(j, i, new Color(1,1,1).getRGB());
				tmp.setRGB(j, i, new Color(0,0,0).getRGB());
			}

		// marca e apaga
		while (again) {
			again = false;

			// segunda sub-iterao
			for (int i = 1; i < h-1; i++)
				for (int j = 1; j < w-1; j++) {
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					red = cor.getRed();
					if (red != 1)
						continue;
					k = nays8(im, i, j);
					iJanela = 0;
					if ((k >= 2 && k <= 6) && connectivity(im, i, j) == 1) {
						for (int v = 0; v < 6; v++){
							col = j + janelaAH[iJanela][0]; //coluna    		
							lin = i + janelaAH[iJanela][1]; //linha
							pixel = im.getRGB(col, lin);
							cor = new Color(pixel);
							ared[v] = cor.getRed();
							iJanela++;
							if (v == 2) {
								iJanela = 1;
							}
						}
						if (ared[0]*ared[1]*ared[2] == 0 && 
								ared[3]*ared[4]*ared[5] == 0) {
							tmp.setRGB(j, i, new Color(1,1,1).getRGB());
							again = true;	
						}
					}		// if ((k >= 2...
				}	// for (j = 1;...

				delete(im, tmp);
				if (again == false)
					break;

			// primeira sub-iterao
			for (int i = 1; i < h-1; i++)
				for (int j = 1; j < w-1; j++) {
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					red = cor.getRed();
					if (red != 1)
						continue;
    			k = nays8(im, i, j);
					iJanela = 0;
    			if ((k >= 2 && k <= 6) && connectivity(im, i, j) == 1) {
						for (int v = 0; v < 6; v++){
							col = j + janelaH[iJanela][0]; //coluna    		
							lin = i + janelaH[iJanela][1]; //linha
							pixel = im.getRGB(col, lin);
							cor = new Color(pixel);
							ared[v] = cor.getRed();
							iJanela++;
							if (v == 2) {
								iJanela = 1;
							}
						}
						if (ared[0]*ared[1]*ared[2] == 0 && 
							ared[3]*ared[4]*ared[5] == 0) {
							tmp.setRGB(j, i, new Color(1,1,1).getRGB());
							again = true;	
						}
	    		}
	  		}

	  		delete(im, tmp);
		}
		
		// ps-processamento 
		stair(im, tmp, NORTH);
		delete(im, tmp);
		stair(im, tmp, SOUTH);
		delete(im, tmp);

		// recupera nveis
		for (int i = 1; i < h-1; i++)
			for (int j = 1; j < w-1; j++) {
				pixel = im.getRGB(j, i);
				cor = new Color(pixel);
				red = cor.getRed();
				if (red > 0) 
					im.setRGB(j, i, new Color(0,0,0).getRGB());
				else 
					im.setRGB(j, i, new Color(255,255,255).getRGB());
			}
	}		// fim de esqueleto

	// nmero de vizinhos 1 do pixel
	public int nays8(BufferedImage im, int r, int c) {
	
		int pixel, red, k = 0;
		Color cor;

		for (int i = r-1; i <= r+1; i++)
			for (int j = c-1; j <= c+1; j++)
				if (i != r || c != j) {
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					red = cor.getRed();
					if (red >= 1)
						k++;
				}

		return k;
	}

	// Conectividade pela contagem de transies preto-branco na vizinhana
	public int connectivity(BufferedImage im, int r, int c) {
		int N = 0, pixel, r1, r2;
		Color cor;

		pixel = im.getRGB(c+1, r);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c+1, r-1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c+1, r-1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c, r-1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0)
			N++;

		pixel = im.getRGB(c, r-1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c-1, r-1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c-1, r-1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c-1, r);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c-1, r);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c-1, r+1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c-1, r+1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c, r+1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c, r+1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c+1, r+1);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		pixel = im.getRGB(c+1, r+1);
		cor = new Color(pixel);
		r1 = cor.getRed();
		pixel = im.getRGB(c+1, r);
		cor = new Color(pixel);
		r2 = cor.getRed();
		if (r1 >= 1 && r2 == 0) 
			N++;

		return N;
	}

	// Apaga pixel em im correspondente a 1 em tmp
	public void delete(BufferedImage im, BufferedImage tmp) {
		int w, h, pixel, red;
		Color cor;

		w = im.getWidth();
		h = im.getHeight();

		// Apaga pixels que esto marcados
		for (int i = 1; i < h-1; i++)
			for (int j = 1; j < w-1; j++) {
				pixel = tmp.getRGB(j, i);
				cor = new Color(pixel);
				red = cor.getRed();
				if (red == 1) {
					im.setRGB(j, i, new Color(0,0,0).getRGB());
					tmp.setRGB(j, i, new Color(0,0,0).getRGB());
				}
			}
	}

	public void stair(BufferedImage im, BufferedImage tmp, int dir) {
		int pixel, r1, r2, r3, r4, r5, r6, r7, r8, r9, w, h;
		Color cor;
		boolean N, S, E, W, NE, NW, SE, SW, C;

		w = im.getWidth();
		h = im.getHeight();

		if (dir == NORTH)
			for (int i = 1; i < h-1; i++)
				for (int j = 1; j < w-1; j++) {
					pixel = im.getRGB(j-1, i-1);
					cor = new Color(pixel);
					r1 = cor.getRed();
					pixel = im.getRGB(j, i-1);
					cor = new Color(pixel);
					r2 = cor.getRed();
					pixel = im.getRGB(j+1, i-1);
					cor = new Color(pixel);
					r3 = cor.getRed();
					pixel = im.getRGB(j-1, i);
					cor = new Color(pixel);
					r4 = cor.getRed();
					pixel = im.getRGB(j, i);
					cor = new Color(pixel);
					r5 = cor.getRed();
					pixel = im.getRGB(j+1, i);
					cor = new Color(pixel);
					r6 = cor.getRed();
					pixel = im.getRGB(j-1, i+1);
					cor = new Color(pixel);
					r7 = cor.getRed();
					pixel = im.getRGB(j, i+1);
					cor = new Color(pixel);
					r8 = cor.getRed();
					pixel = im.getRGB(j+1, i+1);
					cor = new Color(pixel);
					r9 = cor.getRed();
					if (r1 == 1)
						NW = true;
					else
						NW = false;
					if (r2 == 1)
						N = true;
					else
						N = false;
					if (r3 == 1)
						NE = true;
					else
						NE = false;
					if (r4 == 1)
						W = true;
					else
						W = false;
					if (r5 == 1)
						C = true;
					else
						C = false;
					if (r6 == 1)
						E = true;
					else
						E = false;
					if (r7 == 1)
						SW = true;
					else
						SW = false;
					if (r8 == 1)
						S = true;
					else
						S = false;
					if (r9 == 1)
						SE = true;
					else
						SE = false;

					if (dir == NORTH) {
						if (C && !(N && ((E && !NE && !SW && (!W || !S)) || 
							(W && !NW && !SE && (!E || !S)) )) )
								tmp.setRGB(j, i, new Color(0,0,0).getRGB());
						else
							tmp.setRGB(j, i, new Color(1,1,1).getRGB());
					} else if (dir == SOUTH) {
						if (C && !(S && ((E && !SE && !NW && (!W || !N)) || 
							(W && !SW && !NE && (!E || !N)) )) )
								tmp.setRGB(j, i, new Color(0,0,0).getRGB());
						else
							tmp.setRGB(j, i, new Color(1,1,1).getRGB());
					}
	  			}
	}

	// gabarito para ngulos agudos
	public void match_du(BufferedImage im, int r, int c, int k) {

		int[] red = new int[23];
		int col, lin, pixel;
		Color cor;

    int[][] janelaDown = {
			{-2,-2}, {-1,-2}, {0,-2}, {1,-2}, {2,-2}, 
			{-2,-1}, {-1,-1}, {0,-1}, {1,-1}, {2,-1},
			{-2, 0}, {-1, 0}, {0, 0}, {1, 0}, {2, 0},
			{-2, 1}, {-1, 1}, {0, 1}, {1, 1}, {2, 1},
							 {-1, 2}, {0, 2}, {1, 2}
		};
    int[][] janelaUp = {
			{-2, 2}, {-1, 2}, {0, 2}, {1, 2}, {2, 2}, 
			{-2, 1}, {-1, 1}, {0, 1}, {1, 1}, {2, 1},
			{-2, 0}, {-1, 0}, {0, 0}, {1, 0}, {2, 0},
			{-2,-1}, {-1,-1}, {0,-1}, {1,-1}, {2,-1},
							 {-1,-2}, {0,-2}, {1,-2}
		};

		// D1
		for (int v = 0; v < 23; v++){
			col = c + janelaDown[v][0]; //coluna    		
			lin = r + janelaDown[v][1]; //linha
			pixel = im.getRGB(col, lin);
			cor = new Color(pixel);
			red[v] = cor.getRed();
		}

		if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 0 &&
				red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
				red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
				red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
				red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
			  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
		}

		// D2
		if (k >= 2) {
			for (int v = 0; v < 23; v++){
				col = c + janelaDown[v][0]; //coluna    		
				lin = r + janelaDown[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 1 && red[2] == 1 && red[3] == 0 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
			}
		}
		
		// D3
		if (k >= 3) {
			for (int v = 0; v < 23; v++){
				col = c + janelaDown[v][0]; //coluna    		
				lin = r + janelaDown[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 1 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
			}
		}

		// D4
		if (k >= 4) {
			for (int v = 0; v < 23; v++){
				col = c + janelaDown[v][0]; //coluna    		
				lin = r + janelaDown[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 1 && red[2] == 1 && red[3] == 0 &&
					red[4] == 0 && red[5] == 0 && red[6] == 1 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
			}
		}

		// D5
		if (k >= 5) {
			for (int v = 0; v < 23; v++){
				col = c + janelaDown[v][0]; //coluna    		
				lin = r + janelaDown[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 0 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
			}
		}

		// U1
		for (int v = 0; v < 23; v++){
			col = c + janelaUp[v][0]; //coluna    		
			lin = r + janelaUp[v][1]; //linha
			pixel = im.getRGB(col, lin);
			cor = new Color(pixel);
			red[v] = cor.getRed();
		}

		if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 0 &&
				red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
				red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
				red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
				red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
			  red[20]== 0 && red[21]== 0 && red[22]== 0) {
					im.setRGB(c, r, new Color(2,2,2).getRGB());
					return;
		}

		// U2
		if (k >= 2) {
			for (int v = 0; v < 23; v++){
				col = c + janelaUp[v][0]; //coluna    		
				lin = r + janelaUp[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 1 && red[2] == 1 && red[3] == 0 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
						im.setRGB(c, r, new Color(2,2,2).getRGB());
						return;
			}
		}	

		// U3
		if (k >= 3) {
			for (int v = 0; v < 23; v++){
				col = c + janelaUp[v][0]; //coluna    		
				lin = r + janelaUp[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 1 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
						im.setRGB(c, r, new Color(2,2,2).getRGB());
						return;
			}
		}	

		// U4
		if (k >= 4) {
			for (int v = 0; v < 23; v++){
				col = c + janelaUp[v][0]; //coluna    		
				lin = r + janelaUp[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 1 && red[2] == 1 && red[3] == 0 &&
					red[4] == 0 && red[5] == 0 && red[6] == 1 && red[7] == 1 &&
					red[8] == 0 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
						im.setRGB(c, r, new Color(2,2,2).getRGB());
						return;
			}
		}	

		// U5
		if (k >= 5) {
			for (int v = 0; v < 23; v++){
				col = c + janelaUp[v][0]; //coluna    		
				lin = r + janelaUp[v][1]; //linha
				pixel = im.getRGB(col, lin);
				cor = new Color(pixel);
				red[v] = cor.getRed();
			}

			if (red[0] == 0 && red[1] == 0 && red[2] == 1 && red[3] == 1 &&
					red[4] == 0 && red[5] == 0 && red[6] == 0 && red[7] == 1 &&
					red[8] == 1 && red[9] == 0 && red[10]== 0 && red[11]== 0 &&
					red[12]== 0 && red[13]== 0 && red[14]== 0 && red[15]== 0 &&
					red[16]== 0 && red[17]== 0 && red[18]== 0 && red[19]== 0 &&
				  red[20]== 0 && red[21]== 0 && red[22]== 0) {
						im.setRGB(c, r, new Color(2,2,2).getRGB());
						return;
			}
		}	
	}

	/*	Holt's staircase removal stuff	
	public void check (int v1, int v2, int v3) {
		if (!v2 && (!v1 || !v3)) t00 = TRUE;
		if ( v2 && ( v1 ||  v3)) t11 = TRUE;
		if ( (!v1 && v2) || (!v2 && v3) ) {
			t01s = t01;
			t01  = TRUE;
		}
	} */

}
