/***********************************************************************************
FFTMag0.java - Zera a informao sobre magnitude da imagem no domnio de freqncia

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Zera a informao sobre magnitude da imagem no domnio de freqncia

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem com magnitude zerada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FFTMag0 extends JFrame {

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    FFTMag0 fftmag = new FFTMag0(file);

    // Encerra a aplicao clicando no "close"
    fftmag.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    fftmag.setVisible(true);
  }

  public FFTMag0(File file) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
		int w, h;
		JLabel srcL, destL;
      
    // Armazena arquivo imagem numa BufferedImage
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Espectro Amplitude: " + file.getName());

		try {
				dest = computeSpectrum(src);
		} catch (FFTException ffte) {
				System.out.println(ffte);
        System.exit(0);
		}

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		srcL = new JLabel(new ImageIcon(src));
		destL = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(srcL));
		getContentPane().add(new JScrollPane(destL));
		setSize(2*w, h);
	}

  public BufferedImage computeSpectrum(BufferedImage src) 
		throws FFTException {

	    BufferedImage dest = null; 
		  WritableRaster destWR = src.copyData(null); 
			dest = new BufferedImage(src.getColorModel(), destWR, false, null);

			ImageFFT fft = new ImageFFT(dest, ImageFFT.NO_WINDOW);
			fft.transform();

			float constanteMag = fft.getMagnitude(1, 1);
			for (int i = 0; i < src.getWidth(); i++) 
				for (int j = 0; j < src.getHeight(); j++) 
					fft.setMagnitude(j, i, constanteMag);	
			fft.transform();	
			return fft.toImage(dest, 1);	
  }

}
