/***********************************************************************************
FFTPassaAlta.java - Teoria de Fourier

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Teoria de Fourier: ressalta altas freqncias com filtro ideal de passa-alta,
                       em domnio de freqncia.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. raio do filtro.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grafica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FFTPassaAlta extends JFrame {

	public static void main(String args[]) {

		double raio = 0.0;

    // Certifica a passagem da imagem e parmetro
    if (args.length != 2) {
         String msga = "Uso: java -cp . FFTPassaAlta <arquivo_imagem> <raio>";
         System.out.print(msga);
         System.exit(0);
    }

    try {
        raio = Double.parseDouble(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'raio' invalido");
        System.exit(0);
    }

    if (raio < 0.0) {
      String st = "Valor do parametro 'raio' deve ser positivo";
      System.out.println(st);
      System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    FFTPassaAlta fftpa = new FFTPassaAlta(args[0], raio);

    // Encerra a aplicao clicando no "close"
    fftpa.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    fftpa.setVisible(true);
  }

  public FFTPassaAlta(String aFile, double raio) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
		int w, h;
		JLabel srcL, destL;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Altas freq. Fourier: " + file.getName());

		try {
				dest = passaAlta(src, raio);
		} catch (FFTException ffte) {
				System.out.println(ffte);
        System.exit(0);
		}

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		srcL = new JLabel(new ImageIcon(src));
		destL = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(srcL));
		getContentPane().add(new JScrollPane(destL));
		setSize(2*w, h);
	}

  public BufferedImage passaAlta(BufferedImage src, double raio) 
		throws FFTException {

			ImageFFT fft = new ImageFFT(src, ImageFFT.NO_WINDOW);
			fft.transform();
			fft.idealHighPassFilter(raio);
			fft.transform();

			BufferedImage out = fft.toImage(null);
			return out;
  }

}
