/***********************************************************************************
FFTPassaBaixaBW.java - Teoria de Fourier

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Teoria de Fourier: suaviza uma imagem com filtro de passa-baixa de Butterworth,
                       em domnio de freqncia.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ordem do filtro. A ordem  o valor de 'n' atribudo ao expoente do filtro, 
     conforme equao 11.29.  medida que a ordem do filtro cresce, ele se
     aproxima do filtro ideal de passa-baixa.
  3. raio do filtro.

Sada:

  Painel com a imagem original e imagem filtrada com baixas freqncias.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FFTPassaBaixaBW extends JFrame {

	public static void main(String args[]) {

		double	raio = 0.0;
		int			ordem = 1;

    // Certifica a passagem da imagem e parmetros
    if (args.length != 3) {
         String msga = "Uso: java -cp . FFTPassaBaixaBW ";
				 String msgb = "<arquivo_imagem> <ordem> <raio>";
         System.out.print(msga);
         System.out.print(msgb);
         System.exit(0);
    }

    try {
        ordem = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'ordem' invalido");
        System.exit(0);
    }

    if (ordem <= 0) {
        System.out.println("ordem: valor invalido. Ordem >= 1");
        System.exit(0);
    }

    try {
        raio = Double.parseDouble(args[2]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'raio' invalido");
        System.exit(0);
    }

    if (raio < 0.0) {
        System.out.println("valor de 'raio' deve ser positivo");
        System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    FFTPassaBaixaBW fftpb = new FFTPassaBaixaBW(args[0], ordem, raio);

    // Encerra a aplicacao clicando no "close"
    fftpb.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    fftpb.setVisible(true);
  }

  public FFTPassaBaixaBW(String aFile, int ordem, double raio) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
		int w, h;
		JLabel srcL, destL;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Baixas freq. Fourier: " + file.getName());

		try {
				dest = passaBaixaBW(src, ordem, raio);
		} catch (FFTException ffte) {
				System.out.println(ffte);
        System.exit(0);
		}

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		srcL = new JLabel(new ImageIcon(src));
		destL = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(srcL));
		getContentPane().add(new JScrollPane(destL));
		setSize(2*w, h);
	}

  public BufferedImage passaBaixaBW(BufferedImage src, int ordem, double raio) 
		throws FFTException {

			ImageFFT fft = new ImageFFT(src, ImageFFT.NO_WINDOW);
			fft.transform();
			fft.butterworthLowPassFilter(ordem, raio);
			fft.transform();

			BufferedImage out = fft.toImage(null);
			return out;
  }

}
