/***********************************************************************************
FiltroDWMTM.java - Implementa um filtro adaptativo double window-modified trimmed mean

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa um filtro adaptativo double window-modified trimmed mean (DW-MTM)
    Fonte: Myler & Weeks: The pocket handbook of image processing algorithms in C, 
            1993, p. 13.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. valor de Kappa. Inicialmente, pode ser 1.
  3. valor de Sigma. Inicialmente, pode ser 1.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.util.Arrays;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltroDWMTM extends JFrame {

  public static void main(String args[]) {
    double K = 0.0, S = 0.0;

    // Certifica a passagem da imagem e parmetros
    if (args.length != 3) {
			String msga = "Uso: java -cp . FiltroDWMTM <arquivo_imagem> Kappa Sigma";
      System.out.println(msga);
      System.exit(0);
    }

	  // Certifa validade dos parmetros
	  try {
			K = Float.parseFloat(args[1]);
    } catch ( NumberFormatException e ) {
			System.out.println("Parametro 'Kappa' invalido");
			System.exit(0);
    }     

    if (K < 0.0) {
        System.out.println("valor de 'Kappa' deve ser positivo");
        System.exit(0);
    }

	  try {
        S = Float.parseFloat(args[2]);
    } catch ( NumberFormatException e ) {
			System.out.println("Parametro 'Sigma' invalido");
			System.exit(0);
    }     

    if (S < 0.0) {
        System.out.println("valor de 'Sigma' deve ser positivo");
        System.exit(0);
    }

	  // Pega tempo inicial
	  long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando os parmetros
    FiltroDWMTM rd = new FiltroDWMTM(args[0], K, S);

    // Encerra a aplicao clicando no "close"
	  rd.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    rd.setVisible(true);

    // Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.0000");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
  }

  public FiltroDWMTM(String aFile, double K, double S) {
    
	  // Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
	  try {
     		src = ImageIO.read(file);
    } catch(Exception e) {
     		System.out.println("Imagem '" + aFile + "' nao existe.");
     		System.exit(0);
	  }

    // Atribui nome ao frame
    setTitle("Filtro adaptativo DW-MTM: " + file.getName());

	  // Chama funo
    System.out.println("Calculando filtro adaptativo... ");
    dest = filtro(src, K, S);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
  }

  public BufferedImage filtro(BufferedImage src, double Kappa, double Sigma) {

	  int w, h, mediana[], med, tmp, sum, gray, i, j, X, Y, x1, y1, R, tipo; 

	  h = src.getHeight();
	  w = src.getWidth();
    Raster srcR = src.getRaster();
	  tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage outImagem = new BufferedImage(w, h, tipo);
    WritableRaster WR = outImagem.getRaster();
	  mediana = new int[9];
	  R = w;

	  for (Y = 2; Y < h-2; Y++)
		  for (X = 2; X < w-2; X++) {
			  // L os 9 pixels para a matriz mediana
			  tmp = 0;
			  for (y1 = -1; y1 < 2; y1++)
				  for (x1 = -1; x1 < 2; x1++) {
            mediana[tmp] = srcR.getSample(X+x1, Y+y1, 0);
					  tmp++;
				  }
			  // Ordena os valores em ordem crescente
			  /*
			  for (j = 1; j < 9; j++) {
				  tmp = mediana[j];
				  i = j - 1;
				  while (i >= 0 && mediana[i] > tmp) {
					  mediana[i+1] = mediana[i];
					  i--;
				  }
				  mediana[i+1] = tmp;
			  }
			  */
			  // Melhor usar o sort do Java
			  Arrays.sort(mediana);
			  med = mediana[4];
			  sum = 0; 
			  tmp = 0;
			  for (y1 = -2; y1 < 3; y1++)
				  for (x1 = -2; x1 < 3; x1++) {
            gray = srcR.getSample(X+x1, Y+y1, 0);
					  if (gray >= (med - Kappa*Sigma))
						  if (gray <= (med + Kappa*Sigma)) {
							  sum = sum + gray;
							  tmp++;
						  }
				  }
          WR.setSample(X, Y, 0, sum/tmp);
		  }
      return outImagem;
  }     

}
