/***********************************************************************************
FiltroGauss.java - Programa para calcular o filtro de passa-baixa Gaussiano

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Programa para calcular o filtro de passa-baixa Gaussiano usando a classe 
    ConvolveOp do Java2.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Raster;
import java.awt.image.Kernel;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltroGauss extends JFrame {
  
  public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

	  // Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando parmetro
    FiltroGauss filtro = new FiltroGauss(file);

    // Encerra a aplicacao clicando no "close"
    filtro.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    filtro.setVisible(true);

    // Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milissec.");
  }

  public FiltroGauss(File file) {

	  // Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Armazena arquivo imagem numa BufferedImage
    try {
         src = ImageIO.read(file);
    } catch(Exception e) {
         System.out.println("Imagem '" + file + "' nao existe.");
         System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Filtro Gaussiano: " + file.getName());

    // Chama funo
    System.out.println("Calculando filtro Gaussiano... ");
    dest = filtropb(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
  }

  public BufferedImage filtropb(BufferedImage src) {

	  BufferedImage out = null;
	  Raster srcR = src.getRaster();
	  WritableRaster WR = src.getRaster();

    // Define a matriz do filtro
	  float matrizGauss [] = {
			1/16f, 2/16f, 1/16f,
			2/16f, 4/16f, 2/16f,
			1/16f, 2/16f, 1/16f
	  };

    // Define o filtro
	  Kernel filtroGauss = new Kernel(3, 3, matrizGauss);

	  // Cria objeto para convoluo
    ConvolveOp opGauss = new ConvolveOp(filtroGauss);
	  WR = opGauss.filter(srcR, null);

	  out = new BufferedImage(src.getColorModel(), WR, false, null);
	  return out;
  }     

}
