/***********************************************************************************
FiltroLoG.java - Programa para calcular o filtro conjugado Laplace of Gaussian (LoG)

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Programa para calcular o filtro conjugado Laplace of Gaussian (LoG),
    usando a classe ConvolveOp do Java2D.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltroLoG extends JFrame {

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

	  // Pega tempo inicial
    long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    FiltroLoG filtro = new FiltroLoG(file);

    // Encerra a aplicao clicando no "close"
    filtro.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    filtro.setVisible(true);

	  // Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
  }

  public FiltroLoG(File file) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;
      
    // Armazena arquivo imagem numa BufferedImage
 	  try {
     	src = ImageIO.read(file);
    } catch(Exception e) {
     	System.out.println("Imagem '" + file + "' nao existe.");
     	System.exit(0);
	  }

    // Atribui nome ao frame
    setTitle("Laplace do Gaussiano (LoG): " + file.getName());

 	  // Chama funo
    dest = filtroLoG(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
  }

	public BufferedImage filtroLoG(BufferedImage src) {

		BufferedImage out = null;
   	Raster srcR = src.getRaster(); 
    WritableRaster dest1WR = null; 
    WritableRaster dest2WR = null; 

	  // Define matriz do filtro de Gauss
		float matrizGauss [] = {
				1/16f, 2/16f, 1/16f,
				2/16f, 4/16f, 2/16f,
				1/16f, 2/16f, 1/16f
		};

		// Convolve imagem original com filtro de Gauss
		Kernel kGauss = new Kernel(3, 3, matrizGauss);
		ConvolveOp blur = new ConvolveOp(kGauss);
		dest1WR = blur.filter(srcR, null);

		// Coloca resultado do filtro de Gauss em imagem intermediaria
		out = new BufferedImage(src.getColorModel(), dest1WR, false, null);
   	Raster outR = out.getRaster(); 

    // Define matriz do filtro de Laplace
		float matrizLaplace [] = {
				 0f, -1f,  0f,
				-1f,  4f, -1f,
				 0f, -1f,  0f
		};
/*
	  // verso negativa
    float matrizLaplace [] = {
				 0f,  1f,  0f,
				 1f, -4f,  1f,
				 0f,  1f,  0f
		};
*/

		// Convolve imagem intermediaria com filtro de Laplace
		Kernel kLaplace = new Kernel(3, 3, matrizLaplace);
		ConvolveOp sharpe = new ConvolveOp(kLaplace);
		dest2WR = sharpe.filter(outR, null);

		// Gera imagem final
		out = new BufferedImage(src.getColorModel(), dest2WR, false, null);

		return out;
	}     

}
