/***********************************************************************************
FiltroMediana.java - Implementa um filtro de mediana para passa-baixa

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa um filtro de mediana para passa-baixa.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.util.Arrays;
import java.io.IOException;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltroMediana extends JFrame {

  public static void main(String[] args) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o parmetro
    FiltroMediana med = new FiltroMediana(file);

    // Encerra a aplicao clicando no "close"
    med.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    med.setVisible(true);
	}

  public FiltroMediana(File file) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;

    // Armazena arquivo imagem numa BufferedImage
		try {
        src = ImageIO.read(file);
		} catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
		}

    // Atribui nome ao frame
    setTitle("Filtro mediana: " + file.getName());

    dest = medianaOp(src);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  public BufferedImage medianaOp(BufferedImage src) {

		int w, h, tipo, mediana[], tmp, i; 

		h = src.getHeight();
		w = src.getWidth();
		mediana = new int[9];
		tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage outImagem = new BufferedImage(w, h, tipo);
    WritableRaster WR = outImagem.getRaster();
    Raster srcR = src.getRaster();

		for (int Y = 1; Y < h-2; Y++)
			for (int X = 1; X < w-2; X++) {
				// L os 9 pixels para a matriz mediana
				tmp = 0;
				for (int y1 = -1; y1 < 2; y1++)
					for (int x1 = -1; x1 < 2; x1++) {
						mediana[tmp] = srcR.getSample(X+x1, Y+y1, 0);
						tmp++;
					}
					// Ordena os valores em ordem crescente
					/*
					for (int j = 1; j < 9; j++) {
						tmp = mediana[j];
						i = j - 1;
						while (i >= 0 && mediana[i] > tmp) {
							mediana[i+1] = mediana[i];
							i--;
						}
						mediana[i+1] = tmp;
					}
					*/
					// Melhor usar o sort do Java
					Arrays.sort(mediana);
          WR.setSample(X, Y, 0, mediana[4]);
			}
			return outImagem;
  }

}
