/***********************************************************************************
FiltrosDirecionais.java - Implementa vrios filtros direcionais

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa vrios filtros direcionais para deteco de bordas de imagens.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.image.ConvolveOp;
import java.awt.image.Raster;
import java.awt.image.Kernel;
import java.awt.BorderLayout;
import java.awt.Graphics2D;
import java.awt.Graphics;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltrosDirecionais extends JComponent {

	// Define objeto BufferedImage para encapsular a imagem
  BufferedImage src = null;
  int w, h;
  private JComboBox opcoes;

  public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    JFrame f = new JFrame("Filtros Direcionais"); 
    f.getContentPane().add(new FiltrosDirecionais(file));

    // Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(300, 320);
    f.setVisible(true);
  }
  
  public FiltrosDirecionais(File file) {

		int nc;

    // Armazena arquivo imagem numa BufferedImage
	  try {
			  src = ImageIO.read(file);
    } catch (Exception e) {
      	System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

	  // Pega altura e largura da imagem
    w = src.getWidth();
    h = src.getHeight();

    // Define GUI com objetos do Swing
    setLayout(new BorderLayout());

    // Cria um painel para hospedar o combo box
    JPanel controle = new JPanel();

    // Cria o combo box com os nomes das opes
    opcoes = new JComboBox(new String[] {"Imagem", "Norte", 
                              "Sul", "Leste", "Oeste"});

    // Processa mudanca de opo
    opcoes.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
        repaint();
			}
    });
    controle.add(opcoes);
    add(controle, BorderLayout.SOUTH);
	}

  public void paintComponent(Graphics gc) {
		super.paintComponent(gc);
    Graphics2D g2 = (Graphics2D) gc;

    // Recupera a opo do combo box
    String opcao = (String)opcoes.getSelectedItem();

		// Restaura a imagem original
		//
		if (opcao.equals("Imagem")) {
			g2.drawImage(src, 0, 0, null);
    }

		// Filtro Norte
    //
    if (opcao.equals("Norte")) {

			long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      WritableRaster WR = null;
      BufferedImage norteImagem = null;
			
      // Define a matriz do filtro
      float matrizNorte [] = {
				1f,  1f,  1f,
				1f, -2f,  1f,
				-1f, -1f, -1f
			};

      // Define o filtro
			Kernel filtroNorte = new Kernel(3, 3, matrizNorte);

	    // Cria objeto para convoluo
			ConvolveOp norte = new ConvolveOp(filtroNorte);
			WR = norte.filter(srcR, null);
			norteImagem = new BufferedImage(src.getColorModel(), WR, false, null);

			g2.drawImage(norteImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Norte: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro Sul
    //
    if (opcao.equals("Sul")) {

			long eq_time = System.currentTimeMillis();

			Raster srcR = src.getRaster();
      WritableRaster WR = null;
      BufferedImage sulImagem = null;
			
      // Define a matriz do filtro
      float matrizSul [] = {
				-1f, -1f, -1f,
				 1f, -2f,  1f,
				 1f,  1f,  1f
			};

      // Define o filtro
			Kernel filtroSul = new Kernel(3, 3, matrizSul);

	    // Cria objeto para convoluo
			ConvolveOp sul = new ConvolveOp(filtroSul);
			WR = sul.filter(srcR, null);
			sulImagem = new BufferedImage(src.getColorModel(), WR, false, null);

			g2.drawImage(sulImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Sul: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro Leste
    //
    if (opcao.equals("Leste")) {

      long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      WritableRaster WR = null;
      BufferedImage lesteImagem = null;
			
      // Define a matriz do filtro
      float matrizLeste [] = {
				-1f,  1f, 1f,
				-1f, -2f, 1f,
				-1f,  1f, 1f
			};

      // Define o filtro
			Kernel filtroLeste = new Kernel(3, 3, matrizLeste);

	    // Cria objeto para convoluo
			ConvolveOp leste = new ConvolveOp(filtroLeste);
			WR = leste.filter(srcR, null);
			lesteImagem = new BufferedImage(src.getColorModel(), WR, false, null);

			g2.drawImage(lesteImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Leste: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro Oeste
    //
    if (opcao.equals("Oeste")) {

      long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      WritableRaster WR = null;
      BufferedImage oesteImagem = null;
			
      // Define a matriz do filtro
      float matrizOeste [] = {
				1f,  1f, -1f,
				1f, -2f, -1f,
				1f,  1f, -1f
			};

      // Define o filtro
			Kernel filtroOeste = new Kernel(3, 3, matrizOeste);

	    // Cria objeto para convoluo
			ConvolveOp oeste = new ConvolveOp(filtroOeste);
			WR = oeste.filter(srcR, null);
			oesteImagem = new BufferedImage(src.getColorModel(), WR, false, null);

			g2.drawImage(oesteImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Oeste: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }
  }
}
