/***********************************************************************************
FiltrosOrdem.java - Implementa os filtros de ordem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa os filtros de ordem: mximo, mnimo, intervalo e olmpico.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.BorderLayout;
import java.awt.Graphics2D;
import java.awt.Graphics;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class FiltrosOrdem extends JComponent {

  // Define objeto BufferedImage para encapsular a imagem
  BufferedImage src = null;
  int w, h;
  private JComboBox opcoes;

  public static void main(String args[]) {

	  File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    JFrame f = new JFrame("Filtros de Ordem"); 
    f.getContentPane().add(new FiltrosOrdem(file));

    // Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(300, 320);
    f.setVisible(true);
  }
  
  public FiltrosOrdem(File file) {

    // Armazena arquivo imagem numa BufferedImage
 		try {
      src = ImageIO.read(file);
    } catch (Exception e) {
      System.out.println("Imagem '" + file + "' nao existe.");
      System.exit(0);
    }

	  // Pega altura e largura da imagem
    w = src.getWidth();
    h = src.getHeight();

    // Define GUI com objetos do Swing
    setLayout(new BorderLayout());

    // Cria um painel para hospedar o combo box
    JPanel controle = new JPanel();

    // Cria o combo box com os nomes das opes
    opcoes = new JComboBox(new String[] {"Imagem", "Maximo", 
                              "Minimo", "Intervalo", "Olimpico"});

    // Processa mudanca de opo
    opcoes.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
        repaint();
      }
    });
    controle.add(opcoes);
    add(controle, BorderLayout.SOUTH);
  }

  public void paintComponent(Graphics gc) {
	  super.paintComponent(gc);
	  Graphics2D g2 = (Graphics2D) gc;

    // Recupera a opo do combo box
    String opcao = (String)opcoes.getSelectedItem();

	  // Restaura a imagem original
	  //
    if (opcao.equals("Imagem")) {
		  g2.drawImage(src, 0, 0, null);
    }

	  // Filtro de mximo
    //
    if (opcao.equals("Maximo")) {

		  int tipo, ordem[], tmp, nc, maxi;

			long eq_time = System.currentTimeMillis();

			ordem = new int[9];
			tipo = BufferedImage.TYPE_BYTE_GRAY;
			BufferedImage outImagem = new BufferedImage(w, h, tipo);
			WritableRaster WR = outImagem.getRaster();
			Raster srcR = src.getRaster();

			for (int Y = 1; Y < h-1; Y++)
			  for (int X = 1; X < w-1; X++) {
				  // Le os 9 pixels para a matriz de ordem
					tmp = 0;
					maxi = 0;
					for (int y1 = -1; y1 < 2; y1++)
					  for (int x1 = -1; x1 < 2; x1++) {
						  ordem[tmp] = srcR.getSample(X+x1, Y+y1, 0);
							maxi = Math.max(maxi, ordem[tmp]);
							tmp++;
						}
						WR.setSample(X, Y, 0, maxi);
        }

				g2.drawImage(outImagem, 0, 0, null);
        eq_time = System.currentTimeMillis() - eq_time;
        String msg = "Maximo: tempo de execucao ";
        System.out.println( msg + eq_time + " milisseg.");
        return;
	  }

    // Filtro de mnimo
    //
    if (opcao.equals("Minimo")) {

		  int tipo, ordem[], tmp, nc, mini; 

			long eq_time = System.currentTimeMillis();

			ordem = new int[9];
			tipo = BufferedImage.TYPE_BYTE_GRAY;
			BufferedImage outImagem = new BufferedImage(w, h, tipo);
			WritableRaster WR = outImagem.getRaster();
			Raster srcR = src.getRaster();

			for (int Y = 1; Y < h-1; Y++)
				for (int X = 1; X < w-1; X++) {
					// L os 9 pixels para a matriz de ordem
					tmp = 0;
					mini = 255;
					for (int y1 = -1; y1 < 2; y1++)
						for (int x1 = -1; x1 < 2; x1++) {
							ordem[tmp] = srcR.getSample(X+x1, Y+y1, 0);
							mini = Math.min(mini, ordem[tmp]);
							tmp++;
						}
						WR.setSample(X, Y, 0, mini);
				}

			g2.drawImage(outImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Minimo: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }
    
	  // Filtro de intervalo
    //
    if (opcao.equals("Intervalo")) {

		  int tipo, ordem[], tmp, nc, mini, maxi; 

			long eq_time = System.currentTimeMillis();

			ordem = new int[9];
			tipo = BufferedImage.TYPE_BYTE_GRAY;
			BufferedImage outImagem = new BufferedImage(w, h, tipo);
			WritableRaster WR = outImagem.getRaster();
			Raster srcR = src.getRaster();

			for (int Y = 1; Y < h-1; Y++)
				for (int X = 1; X < w-1; X++) {
					// L os 9 pixels para a matriz de ordem
					tmp = 0;
					mini = 255;
					maxi = 0;
					for (int y1 = -1; y1 < 2; y1++)
						for (int x1 = -1; x1 < 2; x1++) {
							ordem[tmp] = srcR.getSample(X+x1, Y+y1, 0);
							mini = Math.min(mini, ordem[tmp]);
							maxi = Math.max(maxi, ordem[tmp]);
							tmp++;
						}
						WR.setSample(X, Y, 0, maxi - mini);
				}

			g2.drawImage(outImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Intervalo: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

    // Filtro olmpico
    //
    if (opcao.equals("Olimpico")) {

		  int tipo, ordem[], tmp, nc, mini, maxi, soma;

			long eq_time = System.currentTimeMillis();

			ordem = new int[9];
			tipo = BufferedImage.TYPE_BYTE_GRAY;
			BufferedImage outImagem = new BufferedImage(w, h, tipo);
			WritableRaster WR = outImagem.getRaster();
			Raster srcR = src.getRaster();

			for (int Y = 1; Y < h-1; Y++)
				for (int X = 1; X < w-1; X++) {
					// Le os 9 pixels para a matriz de ordem
					tmp = 0;
					mini = 255;
					maxi = 0;
					soma = 0;
					for (int y1 = -1; y1 < 2; y1++)
						for (int x1 = -1; x1 < 2; x1++) {
							ordem[tmp] = srcR.getSample(X+x1, Y+y1, 0);
							soma += ordem[tmp];
							mini = Math.min(mini, ordem[tmp]);
							maxi = Math.max(maxi, ordem[tmp]);
							tmp++;
						}
					soma = soma - (maxi + mini);
					WR.setSample(X, Y, 0, (int)soma/9);
			  }

			g2.drawImage(outImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Olimpico: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

  }
}
