/***********************************************************************************
FishEye.java - Implementa um efeito olho de peixe

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa um efeito olho de peixe.

Parmetros de entrada:

  1. nome do arquivo imagem, ponto central e zoom.
    (xc, yc) = ponto central para efeito.
    zoom = usar valores acima de 1.8.
    Adaptado de:  Lyon, D. A. Image processing in Java. Upper Saddle 
                  River, NJ: Prentice Hall, 1999.

Sada:

  Painel com a imagem distorcida.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.*;

public class FishEye extends JFrame {
	static int xc = 0, yc = 0;
	static double zoom = 0.0;

  public static void main(String args[]) {

		if (args.length != 4) {
			String msga = "Uso: java -cp . FishEye <imagem> <xc> <yc> <zoom>";
      System.out.println(msga);
      System.exit(-1);
		}

    try {
			xc = Integer.parseInt(args[1]);
			yc = Integer.parseInt(args[2]);
			zoom = Double.parseDouble(args[3]);
    } catch ( NumberFormatException e ) {
        	System.out.println(e);
    }     

    if (xc < 0 && yc < 0) {
      System.out.println("Valores do centro devem ser positivos");
      System.exit(-1);
    }

    if (zoom <= 0.0) {
      System.out.println("Valor de 'zoom' deve ser positivo e > 0.0");
      System.exit(-1);
    }

    long e_time = System.currentTimeMillis();

    FishEye rd = new FishEye(args[0]);
    rd.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		rd.setSize(300, 200);
    rd.setVisible(true);

    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.0000");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milissec.");
	}

  public FishEye(String aFile) {
	  	
		BufferedImage bimage = null, dest = null;

    // Certifica que imagem existe
    File file = new File(aFile);
    try {
     	bimage = ImageIO.read(file);
    } catch(Exception e) {
     	System.out.println("Imagem '" + aFile + "' nao existe.");
     	System.exit(0);
	 	}

    // Atribui nome ao frame
    setTitle("Efeito olho de peixe: " + file.getName());

    // chama funo
    dest = fishEye(bimage);

		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem 1
		JLabel lsrc = new JLabel(new ImageIcon(bimage));
		getContentPane().add(new JScrollPane(lsrc));

		// Imagem 2
		JLabel img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img2));
	}

  public BufferedImage fishEye(BufferedImage src) {

    int w, h;
		double p[] = new double[2];
		int xp, yp;
		double R;
		double dx, dy, radius, a, u;

    w = src.getWidth();
    h = src.getHeight();
		R = Math.sqrt(w*w/4 + h*h/4);
		// BufferedImage out = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_GRAY);
    BufferedImage out = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);

		for (int x = 0; x < w; x++)
			for (int y = 1; y < h; y++) {
				dx = x - xc;
				dy = y - yc;
				radius = Math.sqrt(dx*dx + dy*dy);
				u = Math.pow(radius, zoom)/R;
				a = Math.atan2(dy, dx);
				p[0] = u*Math.cos(a);
				p[1] = u*Math.sin(a);
				xp = (int)p[0] + xc;
				yp = (int)p[1] + yc;
				if ((xp < w) && (yp < h) && (xp >= 0) && (yp >= 0)) {
					out.setRGB(x, y, src.getRGB(xp, yp));
				}
			}

      return out;
   }     

}
