/***********************************************************************************
FreqEspacial1D.java - Gera imagem sinttica com freqncia espacial e dimenso
                      dim X dim.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Gera imagem sinttica com freqncia espacial 'u' e dimenso dim X dim.

Parmetros de entrada:

    1. 'u'   - freqncia espacial.
    2. 'dim' - dimenso da imagem.
    3. 'A'   - amplitude.
    4. 'Fi'   - ngulo de fase.

Sada:

  Painel com a imagem sinttica.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

public class FreqEspacial1D extends JFrame {

	public static void main(String args[]) {

		int u = 0, dim = 0, A = 0;
		double Fi = 0.0;

    // Certifica a passagem dos parmetros
    if (args.length != 4) {
         String msga = "Uso: java -cp . FreqEspacial1D <u> <dim> <A> <Fi>";
         String msgb = "\t<u>   - frequencia espacial";
         String msgc = "\t<dim> - dimensao da imagem";
         String msgd = "\t<A>   - amplitude";
         String msge = "\t<Fi>  - angulo de fase";
         System.out.println(msga);
         System.out.println(msgb);
         System.out.println(msgc);
         System.out.println(msgd);
         System.out.println(msge);
         System.exit(0);
    }

		// Certifica parmetro de freqncia espacial
    try {
        u = Integer.parseInt(args[0]);
    } catch (NumberFormatException e) {
        System.out.println("Valor da frequencia espacial ('u') invalido");
        System.exit(0);
    }

    if (u <= 0) {
        System.out.println("Frequencia espacial: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Certifica dimenso da imagem
    try {
        dim = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'dimensao' invalido");
        System.exit(0);
    }

    if (dim <= 0) {
        System.out.println("dimensao: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Certifica amplitude
    try {
        A = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro amplitude ('A') invalido");
        System.exit(0);
    }

    if (A <= 0) {
        System.out.println("Amplitude: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Certifica ngulo de fase
    try {
        Fi = Double.parseDouble(args[3]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro angulo de fase ('Fi') invalido");
        System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando o nome do arquivo imagem
    FreqEspacial1D fc = new FreqEspacial1D(u, dim, A, Fi);

    // Encerra a aplicao clicando no "close"
    fc.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    fc.setVisible(true);
	}

  public FreqEspacial1D(int u, int dim, int A, double Fi) {

		// Define objeto BufferedImage para encapsular as imagens
    BufferedImage sintetica = null;
    int nc, w, h, tipo;
      
    // Atribui nome ao frame
    setTitle("Freqncia Espacial 1D");

    tipo = BufferedImage.TYPE_BYTE_GRAY;
    w = h = dim;
		sintetica = new BufferedImage(w, h, tipo);
    WritableRaster sinteticaWR = sintetica.getRaster();

    double fase = Math.toRadians(Fi); 
    for (int i = 0; i < h; i++) {
			for (int j = 0; j < w; j++) {
				nc = 128 + (int)(A*Math.sin((2*Math.PI*u*j)/(w - 1)+fase)); 
        sinteticaWR.setSample(j, i, 0, nc);
      }
    }

    // Define GUI com objetos do Swing
    JLabel limg = new JLabel(new ImageIcon(sintetica));
    getContentPane().add(new JScrollPane(limg));
    setSize(w, h);
  }

}
