/***********************************************************************************
FreqEspacial2D.java - Gera imagem sinttica com freqncia espacial (u, v)

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Gera imagem sinttica com freqncia espacial (u, v) e dimenso dim X dim.

Parmetros de entrada:

    1. 'u'   - freqncia espacial X.
    2. 'v'   - freqncia espacial Y.
    3. 'dim' - dimenso da imagem.
    4. 'A'   - amplitude.
    5. 'Fix' - ngulo de fase na direo X.
    6. 'Fiy' - ngulo de fase na direo Y.

Sada:

  Painel com a imagem sinttica.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

public class FreqEspacial2D extends JFrame {

	public static void main(String args[]) {

		int u = 0, v = 0, dim = 0, A = 0;
		double Fix = 0.0, Fiy = 0.0;

    // Certifica a passagem dos parmetros
    if (args.length != 6) {
         String msga = "Uso: java -cp . FreqEspacial2D <u> <v> <dim> <A> <Fix> <Fiy>";
         String msgb = "\t<u>   - frequencia espacial x";
         String msgc = "\t<v>   - frequencia espacial y";
         String msgd = "\t<dim> - dimensao da imagem";
         String msge = "\t<A>   - amplitude";
         String msgf = "\t<Fix>  - angulo de fase na direcao x";
         String msgg = "\t<Fiy>  - angulo de fase na direcao y";
         System.out.println(msga);
         System.out.println(msgb);
         System.out.println(msgc);
         System.out.println(msgd);
         System.out.println(msge);
         System.out.println(msgf);
         System.out.println(msgg);
         System.exit(0);
    }

		// Certifica parmetro de freqncia espacial direo x
    try {
        u = Integer.parseInt(args[0]);
    } catch (NumberFormatException e) {
        System.out.println("Valor da frequencia espacial ('u') invalido");
        System.exit(0);
    }

    if (u < 0) {
        System.out.println("Frequencia espacial: valor invalido. Deve ser >= 0");
        System.exit(0);
    }

		// Certifica parmetro de freqncia espacial direo y
    try {
        v = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor da frequencia espacial ('v') invalido");
        System.exit(0);
    }

    if (v < 0) {
        System.out.println("Frequencia espacial: valor invalido. Deve ser >= 0");
        System.exit(0);
    }

		// Certifica dimenso da imagem
    try {
        dim = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'dimensao' invalido");
        System.exit(0);
    }

    if (dim <= 0) {
        System.out.println("dimensao: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Certifica amplitude
    try {
        A = Integer.parseInt(args[3]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro Amplitude ('A') invalido");
        System.exit(0);
    }

    if (A <= 0) {
        System.out.println("Amplitude: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Certifica ngulo de fase na direo x
    try {
        Fix = Double.parseDouble(args[4]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro angulo de fase ('Fix') x invalido");
        System.exit(0);
    }

		// Certifica ngulo de fase na direo y
    try {
        Fiy = Double.parseDouble(args[5]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro angulo de fase ('Fiy') y invalido");
        System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando o nome do arquivo imagem
    FreqEspacial2D fc = new FreqEspacial2D(u, v, dim, A, Fix, Fiy);

    // Encerra a aplicao clicando no "close"
    fc.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    fc.setVisible(true);
	}

  public FreqEspacial2D(int u, int v, int dim, int A, double Fix, double Fiy) {

		// Define objeto BufferedImage para encapsular as imagens
    BufferedImage sintetica = null;
    int nc, w, h, tipo;
		double fx, fy, compU, compV;
      
    // Atribui nome ao frame
    setTitle("Freqncia Espacial 2D");

    tipo = BufferedImage.TYPE_BYTE_GRAY;
    w = h = dim;
		sintetica = new BufferedImage(w, h, tipo);
    WritableRaster sinteticaWR = sintetica.getRaster();

    fx = Math.toRadians(Fix); 
    fy = Math.toRadians(Fiy); 
    for (int i = 0; i < h; i++) {
			for (int j = 0; j < w; j++) {
				compU = 2*Math.PI*u*j/(w-1) + fx;
				compV = 2*Math.PI*v*i/(h-1) + fy;
				nc = 128 + (int)(A*Math.sin(compU + compV)); 
        sinteticaWR.setSample(j, i, 0, nc);
      }
    }

    // Define GUI com objetos do Swing
    JLabel limg = new JLabel(new ImageIcon(sintetica));
    getContentPane().add(new JScrollPane(limg));
    setSize(256, 256);
  }

}
