/***********************************************************************************
HistoGrab.java - Exemplo de uso da classe PixelGrabber

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Exemplo de uso da classe PixelGrabber

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com a imagem original e o seu histograma, sobreposto.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.Frame;
import java.awt.Event;
import java.awt.Image;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MediaTracker;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;

public class HistoGrab extends Frame {

  int w, h, hist[] = new int[256], max_hist = 0;
  Image img;

  public static void main(String args[]) {

    // Certifica a passagem da imagem como parmetro
    if (args.length != 1) {
      System.out.println("Uso: java -cp . HistoGrab <arquivo_imagem>");
      System.exit(0);
    }

    // Recupera tempo de processamento
    long e_time = System.currentTimeMillis();

    // Chama o construtor passando o nome do arquivo imagem
    Frame f = new HistoGrab(args[0]);
    f.setVisible(true);

    // Recupera tempo de processamento
    e_time = System.currentTimeMillis() - e_time;
    System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }
  
  public HistoGrab(String imagem) {
    int p, R, G, B, y, iw = 0, ih = 0, pixels[] = new int[1];
    PixelGrabber pg = null;

    // Certifica que a imagem existe
    File tmpFile = new File(imagem);
    if (!tmpFile.exists()) {
       System.out.println("Arquivo '" + imagem + "' nao existe.");
       System.exit(0);
    }

    setTitle("Histograma: " + tmpFile.getName());
    System.out.println("Carregando imagem: `" + imagem + "'");

    // Associa objeto Image com <arquivo_imagem>
    img = Toolkit.getDefaultToolkit().getImage(imagem);

    // Define objeto MediaTracker
    MediaTracker mt = new MediaTracker(this);

    // Certifica carga do <arquivo_imagem> e pixels da imagem
    mt.addImage(img, 0);
    try {
      mt.waitForID(0);
      iw = img.getWidth(null);
      ih = img.getHeight(null);
      pixels = new int[iw*ih];
      pg = new PixelGrabber(img, 0, 0, iw, ih, pixels, 0, iw);
      pg.grabPixels();
    } catch (InterruptedException e) {
      System.out.println("Problema lendo arquivo `" + imagem + "'");
      System.exit(0);
    }

    w = img.getWidth(null);
    h = img.getHeight(null);
    setSize(w, h);

    // Calcula histograma da imagem.
    // Com imagem monocromtica, apenas um sample de pixel
    //  necessario. Usa-se sample red.

    ColorModel cmodel = pg.getColorModel();

    for (int i = 0; i < pixels.length; i++) {
       R = cmodel.getRed(pixels[i]);
       G = cmodel.getGreen(pixels[i]);
       B = cmodel.getBlue(pixels[i]);
       hist[R]++;
    }

    for (int i = 0; i < 256; i++) {
       max_hist = Math.max(hist[i], max_hist);
    }
  }

  public void paint(Graphics g) {
    Graphics2D g2 = (Graphics2D) g;
    int x, y, lasty;

    // desenha a imagem
    g2.drawImage(img, 0, 0, null);

    // desenha o histograma
    x = (w - 256)/2;
    lasty = h - h*hist[0]/max_hist;
    for (int i = 0; i < 256; i++, x++) {
      y = h - h*hist[i]/max_hist;
      g2.setColor(new Color(i, i, i));
      g2.fillRect(x, y, 1, h);
      g2.setColor(Color.red);
      g2.drawLine(x-1, lasty, x, y);
      lasty = y;
    }
  }

  // Encerra a aplicao clicando no "close"
  public boolean handleEvent(Event event) {
    if (event.id == Event.WINDOW_DESTROY)
       System.exit(0);
       return false;
  }
}
