/***********************************************************************************
Histograma.java - Calcula, e mostra, o histograma de uma imagem.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula, e mostra, o histograma de uma imagem.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com o histograma. No  um bom exemplo de programao para desenhar
  um histograma.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.Raster;
import java.awt.image.BufferedImage;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JFrame;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Histograma extends JFrame {

  int w, h, hist[] = new int[256], max_hist = 0;
  // Define objeto BufferedImage para encapsular a imagem
  BufferedImage src;

  public static void main(String args[]) {

	  File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

    // Chama o construtor passando o nome do arquivo imagem
    JFrame f = new Histograma(file);
    f.setSize(300, 280);
    f.setVisible(true);

    // Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }
  
  public Histograma(File file) {
    int p, r, g, b, y, iw = 0, ih = 0, pixels[] = new int[1];

    try {
        src = ImageIO.read(file);
    } catch (Exception e) {
        System.out.println("Erro lendo arquivo: " + file);
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Histograma: " + file.getName());

    iw = src.getWidth();
    ih = src.getHeight();
    Raster srcR = src.getRaster();

    for (int i = 0; i < src.getHeight(); i++)
      for (int j = 0; j < src.getWidth(); j++) {
        y = srcR.getSample(j, i, 0);
        hist[y]++;
      }
		
     for (int i = 0; i < 256; i++) {
				max_hist = Math.max(hist[i], max_hist);
     }
   }

   public void paint(Graphics g) {
     Graphics2D g2 = (Graphics2D) g;
     int x, y;

     g2.setColor(Color.white);
     g2.fillRect(0, 0, 255, 255);
     g2.setColor(Color.black);
     x = 0;
     for (int i = 0; i < 256; i++, x++) {
         y = 256 - 256*hist[i]/max_hist;
         //g2.fillRect(x, y, 1, 255);
         g2.drawLine(x, 256, x, y);
     }
     g2.setColor(Color.blue);
     g2.fillRect( 64, 255, 1, 8);
     g2.fillRect(128, 255, 1, 8);
     g2.fillRect(192, 255, 1, 8);
     g2.drawString(  "0",   5, 270);
     g2.drawString( "64",  60, 270);
     g2.drawString("128", 120, 270);
     g2.drawString("192", 185, 270);
     g2.drawString("256", 245, 270);
     g2.drawString("Pr{x}", 10, 40);
     g2.drawString("x", 270, 256);
     g2.setColor(Color.black);
		 g2.translate(this.getInsets().left, this.getInsets().top);
     g2.drawRect(5, 23, 250, 233);
     return;
   }
}
