/***********************************************************************************
HistogramaFC.java - Calcula o histograma de uma imagem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula, e mostra, o histograma de uma imagem.
     necessrio usar o pacote JFreeChart, verso 0.9.8.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com o histograma. Esta  uma melhor implementao do programa
  Histograma.java.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do JFreeChart
import org.jfree.data.XYSeriesCollection;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.TextTitle;
import org.jfree.chart.plot.Plot;
import org.jfree.data.XYDataset;
import org.jfree.data.XYSeries;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.GradientPaint;
import java.awt.image.Raster;
import java.awt.Color;
import java.awt.Paint;

// pacote Swing
import javax.swing.JFileChooser;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class HistogramaFC {

  public static void main(String[] args) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null;
		int w, h;
		int[] hist = new int[256];
    
		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

    // Armazena arquivo imagem numa BufferedImage
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + file + "' nao existe.");
        System.exit(0);
    }

    Raster srcR = src.getRaster(); 
		w = src.getWidth();
		h = src.getHeight();

		// Calcula o histograma
		for (int i = 0; i < 256; i++) 
			hist[i] = 0;
		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++)
				hist[srcR.getSample(j, i, 0)]++;

		// Cria o arquivo de dados...	
		XYSeries dataset = new XYSeries("Densidade probabilidade");
		for (int i = 0; i < 256; i++) {
			dataset.add(i, hist[i]);
		}

		XYDataset xyDataSet = new XYSeriesCollection(dataset);

		// Cria o grafico...
		JFreeChart chart = ChartFactory.createAreaXYChart("Histograma",
												"nivel de cinza",
												"Frequencia",
												xyDataSet,
												false,				// mostra a legenda FDP
												true,
												true
		);

		// Cria e mostra um frame...
		ChartFrame frame = new ChartFrame("Histograma", chart);

		// Personaliza apresentao do histograma
		// Adiciona sub-ttulo
		TextTitle subtitulo = new TextTitle(file.getName());
		chart.addSubtitle(subtitulo);

		// Cor de fundo do quadro 
		Paint pq = new GradientPaint(0, 0, Color.white, 1000, 0, Color.blue);
		chart.setBackgroundPaint(pq);

		// Cor de fundo da rea do grfico
		Plot plot = chart.getPlot();
		Paint pc = new GradientPaint(0, 0, Color.green, 1000, 0, Color.white);
		plot.setBackgroundPaint(pc);

		frame.pack();
		frame.setVisible(true);
  }
}