/***********************************************************************************
Image2Buffered.java - Convertendo Image para BufferedImage

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Convertendo Image para BufferedImage.

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com imagem e texto sobreposto.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;

// pacotes do AWT
import java.awt.Font;
import java.awt.Frame;
import java.awt.Event;
import java.awt.Image;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MediaTracker;
import java.awt.image.BufferedImage;

public class Image2Buffered extends Frame {
  
  // Define um objeto BufferedImage para encapsular a imagem
  private BufferedImage bimage = null;

  public static void main(String args[]) {

    // Certifica a passagem da imagem como parmetro
    if (args.length != 1) {
      String msga = "Uso: java -cp . Image2Buffered <arquivo_imagem>";
      System.out.println(msga);
      System.exit(-1);
    }

    // Chama o construtor passando o nome do arquivo imagem
    Image2Buffered ib = new Image2Buffered(args[0]);
    ib.setVisible(true);
  }

  public Image2Buffered(String aFile) {
    Image img = null;
    int imgw, imgh, tipo;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    if (!file.exists()) {
      System.out.println("Arquivo '" + aFile + "' nao existe.");
      System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Image2Buffered: " + file.getName());

    // Carrega o arquivo imagem
    try {
      img = readImage(aFile);
    } catch (IOException e) {
      System.out.println("Abortando. Problema com imagem " + aFile);
      System.exit(0);
    }

    // implementa leitura indireta de BufferedImage, usando Image
    imgw = img.getWidth(null);
    imgh = img.getHeight(null);
    tipo = BufferedImage.TYPE_BYTE_GRAY;
    try {
      bimage = returnAsBuffered(img, imgw, imgh, tipo);
    } catch (IOException e) {
      System.out.println("Problema convertendo Image --> BufferedImage.");
      System.exit(0);
    }

    setSize(imgw, imgh);
  }

  public Image readImage(String fileIn) throws IOException {

    Toolkit tk = Toolkit.getDefaultToolkit();

    Image img = tk.getImage(fileIn);
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(img, 0);
    try {
      mt.waitForID(0);
    } catch (InterruptedException e) {
      System.out.println("readImage: problema lendo imagem " + fileIn);
      return null;
    }
      return img;
  }

  public BufferedImage returnAsBuffered(Image imagem, int w, int h, int type) 
      throws IOException {

    BufferedImage bi = null;

    bi = new BufferedImage(w, h, type);
      Graphics2D g2 = bi.createGraphics();
      g2.drawImage(imagem, 0, 0, null);

       // Para a forma original, apagar as quatro linhas abaixo
      g2.setPaint(Color.white);
      g2.setFont(new Font("Century Gothic", Font.PLAIN, 15)); 
      String texto = "Texto sobre Imagem";
      g2.drawString(texto, 10, 15);

      return bi;
   }

   // sobrecarrega o mtodo grfico paint()
   // para desenhar a imagem
   public void paint(Graphics g) {
     Graphics2D g2 = (Graphics2D)g;

     // Translada origem da imagem e desenha
     g2.translate(this.getInsets().left,this.getInsets().top);
     g2.drawImage(bimage, 0, 0, this);
   }

   // Encerra a aplicao clicando no "close"
   public boolean handleEvent(Event event) {
     if (event.id == Event.WINDOW_DESTROY)
        System.exit(0);
        return false;
   }
}
