/***********************************************************************************
JAIConvolve.java - Convoluo de uma imagem usando API JAI

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Convoluo de uma imagem usando API JAI.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ngulo de rotao da figura.

Sada:

  Painel com imagem.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.GridLayout;
import java.awt.geom.AffineTransform;
import java.awt.image.renderable.ParameterBlock;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.PlanarImage;
import javax.media.jai.Interpolation;

public class JAIConvolve extends JFrame {
	
  public static void main(String args[]) {

    float theta = 0;

		// verifica parmetros de entrada
		if (args.length != 2) {
	   	System.out.println("Uso: java JAIConvolve <arquivo_imagem> <angulo>");
	   	System.exit(0);
		}

    // L ngulo
    try {
        theta = Float.parseFloat(args[1]);
    } catch (IllegalArgumentException e) {
        System.out.println("Valor de 'angulo' invalido: " + args[1]);
        System.exit(0);
    }

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
		JFrame f = new JAIConvolve(args[0], theta);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIConvolve(String aFile, float theta) {

		PlanarImage sourceImg = null;
		JLabel img1, img2;

    // Certifica que a imagem existe
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("JAIConvolve imagem: " + tmpFile.getName());

		// Define filtro de passa-alta
		System.out.println("Calculando filtro de passa alta...");
		float[] kernelPa = {
			-1.0f, -1.0f, -1.0f,
			-1.0f,  9.0f, -1.0f,
			-1.0f, -1.0f, -1.0f
		};
		KernelJAI pa_kernel = new KernelJAI(3, 3, kernelPa);
		RenderedOp paImg = JAI.create("convolve", sourceImg, pa_kernel);

		// Rotao da imagem
		System.out.println("Transformacao afim: rotacao");
		RenderedOp interpolImg = interpola(paImg, theta);

		// Pega valores de tamanho e largura das imagens
		int w = sourceImg.getWidth();
		int h = sourceImg.getHeight();

		// Mostra as imagens
		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));
		img2 = new JLabel(new ImageIcon(interpolImg.getAsBufferedImage()));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
  }

	public RenderedOp interpola(PlanarImage src, float theta) {

		// Especifica o mtodo de interpolao
		Interpolation interp = Interpolation.getInstance(Interpolation.INTERP_BICUBIC);

		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);				// imagem para rotao
		pb.add(0.0f);							// ponto x para rotao
		pb.add(0.0f);							// ponto y para rotao
		pb.add(theta);						// ngulo de rotao
		pb.add(interp);						// O mtodo de interpolao
		return JAI.create("rotate", pb, null);
	}

}
