/***********************************************************************************
JAIConvolveV1.java - Convoluo de uma imagem usando API JAI

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Convoluo de uma imagem usando API JAI

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com trs imagens convolvidas com os filtros de passa-alta, passa-baixa
  e de ordem estatstica (mediana).

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.GridLayout;
import java.awt.image.renderable.ParameterBlock;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.PlanarImage;
import javax.media.jai.operator.MedianFilterDescriptor;
import javax.media.jai.operator.MedianFilterShape;

public class JAIConvolveV1 extends JFrame {
	
  public static void main(String args[]) {

		// Checa nome de arquivo na lista de argumentos
		if (args.length != 1) {
	   	System.out.println("Uso: java JAIConvolveV1 <arquivo_imagem>");
	   	System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
		JFrame f = new JAIConvolveV1(args[0]);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIConvolveV1(String aFile) {

		PlanarImage sourceImg = null;
		JLabel img1, img2, img3;

    // Certifica que a imagem existe
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("Convolve imagem: " + tmpFile.getName());

		// 1. Cria filtro de passa-alta
		//		Chama funo com filtro
		RenderedOp sobelH = sobelHoriz(sourceImg);
		// RenderedOp sobelV = sobelVert(sourceImg);
		// RenderedOp gradiM = gradMagnitude(sourceImg);

		// 2. Cria filtro de passa-baixa
		//		Modifique os valores para variar o tamanho do filtro
		RenderedOp boxImg = boxFilter(sourceImg, 5, 5);

		// 3. Cria filtro de ordem estatstica: mediana
		MedianFilterShape tipoF = MedianFilterDescriptor.MEDIAN_MASK_X;
		RenderedOp medImg = filtroMediana(sourceImg, tipoF, 5);

		// Pega valores de tamanho e largura das imagens
		int w = sourceImg.getWidth();
		int h = sourceImg.getHeight();

		// Mostra as imagens
		// Cria gridLayout de 1 x 3
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 3));
		img1 = new JLabel(new ImageIcon(sobelH.getAsBufferedImage()));
		img2 = new JLabel(new ImageIcon(boxImg.getAsBufferedImage()));
		img3 = new JLabel(new ImageIcon(medImg.getAsBufferedImage()));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		getContentPane().add(new JScrollPane(img3));
		setSize(3*w, h);
  }

	public RenderedOp sobelHoriz(PlanarImage src) {

		// Filtro de passa-alta de Sobel
		KernelJAI sobelH = KernelJAI.GRADIENT_MASK_SOBEL_HORIZONTAL;

		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(sobelH);

		return JAI.create("convolve", pb);
	}

	public RenderedOp sobelVert(PlanarImage src) {

		// Filtro de passa-alta de Sobel
		KernelJAI sobelV = KernelJAI.GRADIENT_MASK_SOBEL_VERTICAL;

		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(sobelV);

		return JAI.create("convolve", pb);
	}

	public RenderedOp gradMagnitude(PlanarImage src) {

		// Filtro de passa-alta de Sobel com a magnitude do gradiente
		KernelJAI sobelV = KernelJAI.GRADIENT_MASK_SOBEL_VERTICAL;
		KernelJAI sobelH = KernelJAI.GRADIENT_MASK_SOBEL_HORIZONTAL;

		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(sobelV);
		pb.add(sobelH);

		return JAI.create("GradientMagnitude", pb);
	}

	public RenderedOp boxFilter(PlanarImage src, int w, int h) {

		// Filtro de passa-passa
		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(w);
		pb.add(h);
		pb.add(w/2);
		pb.add(h/2);

		return JAI.create("BoxFilter", pb);
	}

	public RenderedOp filtroMediana(PlanarImage src, 
					MedianFilterShape maskType, int filterSize) {

		// Filtro de ordem estatstica: mediana
		// Cria e adiciona bloco de parmetros
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(maskType);
		pb.add(filterSize);

		return JAI.create("MedianFilter", pb);
	}
}
