/***********************************************************************************
JAIDCT.java - Calcula transformada discreta do co-seno: DCT, IDCT

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula transformada discreta do co-seno: DCT, IDCT

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com quatro imagens: original, DCT, IDCT e original. 

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.SampleModel;
import java.awt.image.DataBuffer;
import java.awt.image.DataBuffer;
import java.awt.image.ColorModel;
import java.awt.RenderingHints;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// pacotes do JAI
import javax.media.jai.PlanarImage;
import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;

public class JAIDCT extends JFrame {

  public static void main(String[] args) {

		// Checa nome de arquivo na lista de argumentos
		if (args.length != 1) {
	   	System.out.println("Uso: java JAIDCT <arquivo_imagem> ");
	   	System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
    JFrame frame = new JAIDCT(args[0]);

		// Encerra a aplicao clicando no "close"
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
  }

  public JAIDCT(String fileIn) {

		JLabel img1, img2, img3, img4;
		int nBandas, w, h;

		PlanarImage sourceImg = null;
    // Certifica que a imagem existe
		try {
			sourceImg = JAI.create("fileload", fileIn);
			System.out.println("Carregando imagem '" + fileIn + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + fileIn + " com problemas.");
			System.exit(0);
		}

		// Informaes da imagem
		nBandas = sourceImg.getNumBands();
		w = sourceImg.getWidth();
		h = sourceImg.getHeight();
		String imageInfo = "Dimensao: " + w + "x" + h + " Bandas: " + nBandas;
 
		// Cria DCT
    PlanarImage dctImg = DCTImage(sourceImg);

		// Cria a inversa de DCT
    PlanarImage idctPI = IDCTImage(dctImg);

		// Para restaurar imagem original,  preciso criar
		// um objeto RenderingHints com leiaute adequado
		PlanarImage dst = RHImagem(sourceImg, idctPI);

		// Atribui nome ao frame
		File tmpFile = new File(fileIn);
		setTitle("Exemplo DCT: " + tmpFile.getName());

		// Mostra as imagens
		System.out.println("Mostrando Imagens");
		System.out.println("Imagem original: \n\t" + imageInfo);

		// Cria gridLayout de 1 x 4
		getContentPane().setLayout(new GridLayout(1, 4));

		// Imagem original
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));

		// Imagem DCT
		img2 = new JLabel(new ImageIcon(dctImg.getAsBufferedImage()));
		
		// Imagem IDCT
		img3 = new JLabel(new ImageIcon(idctPI.getAsBufferedImage()));
		
		// Imagem original
		img4 = new JLabel(new ImageIcon(dst.getAsBufferedImage()));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		getContentPane().add(new JScrollPane(img3));
		getContentPane().add(new JScrollPane(img4));
    setSize(4*w, h);
  }

	// Calcula a DCT da imagem fonte
 	public PlanarImage DCTImage(PlanarImage src) {

		System.out.println("Criando DCT...");
    ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
    return JAI.create("dct", pb);
	}

	// Calcula a IDCT da imagem transformada
 	public PlanarImage IDCTImage(PlanarImage dct) {

		System.out.println("Criando IDCT...");
    ParameterBlock pb = new ParameterBlock();
		pb.addSource(dct);
    return JAI.create("idct", pb, null);
	}

	public PlanarImage RHImagem(PlanarImage src, PlanarImage idft) {

		System.out.println("Transformando IDFT (real) para PlanarImage (byte)...");
		ImageLayout il = new ImageLayout();
    il.setSampleModel(src.getSampleModel());
    il.setColorModel(src.getColorModel());
    RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

    // Converte os dados para byte.
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(idft);
    pb.add(DataBuffer.TYPE_BYTE);
    return JAI.create("format", pb, rh);
	}

}