/***********************************************************************************
JAIDCTBarFC.java - Mostra histograma da imagem usando API JAI e JFreeChart

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Mostra histograma da imagem usando API JAI e JFreeChart.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. 'bin': quantidade de barras do histograma. Neste caso, pode usar valores
            pequenos, como 4 ou 5.

Sada:

  Painel com trs imagens convolvidas com os filtros de passa-alta, passa-baixa
  e de ordem estatstica (mediana).

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do JFreeChart
import org.jfree.data.DefaultCategoryDataset;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.TextTitle;
import org.jfree.chart.plot.Plot;

// pacotes do AWT
import java.awt.Color;
import java.awt.Paint;
import java.awt.GridLayout;
import java.awt.GradientPaint;
import java.awt.image.renderable.ParameterBlock;

// pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.PlanarImage;

public class JAIDCTBarFC {
	
	public static void main(String args[]) {

		int bin = 0;

		// Checa nome de arquivo na lista de argumentos
		if (args.length != 2) {
	   	System.out.println("Uso: java JAIDCTBarFC <arquivo_imagem> <bin>");
	   	System.exit(0);
		}

    try {
        bin = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'bin' invalido");
        System.exit(0);
    }

    if (bin <= 0) {
        System.out.println("bin: valor invalido. Deve ser > 0");
        System.exit(0);
    }

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

    // Chama o construtor
		JAIDCTBarFC f = new JAIDCTBarFC(args[0], bin);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIDCTBarFC(String aFile, int bin) {

		PlanarImage sourceImg = null;

		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Calcula e mostra histograma da imagem
    File file = new File(aFile);
    String fileName = file.getName();
		calculaHisto(sourceImg, fileName, bin);
  }

	public void calculaHisto(PlanarImage src, String fileName, int bin) { 

		String serie = "massa";
		String[] category = new String[bin];

		// Calcula o histograma da imagem
		System.out.println("Calculando histograma da imagem...");

		// Cada "bin" contm a freqncia do nvel de cinza na imagem. Ex.:
		// bin 0 = 13456, bin 1 = 10546 etc. Neste caso, usamos apenas 4
		// nveis de cinza da DCT da imagem fonte.
		double[] bins = DCTImageBins(src, bin);

		// Desenha histograma com JFreeChart
		// Cria o arquivo de dados...	
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		for (int i = 0; i < bin; i++) {
			category[i] = "bin" + (i+1);
		}
		for (int i = 0; i < bin; i++) {
			dataset.addValue(bins[i], serie, category[i]);
		}

		// Cria o grafico...
		String titulo = "Massa de " + bin + " 'bins' da DCT de " + fileName;
		JFreeChart chart = ChartFactory.createVerticalBarChart(
												titulo,
												"'bin' ordenado por freqncia",
												"Massa do 'bin'",
												dataset,
												false,				// mostra a legenda FDP
												true,
												true
		);

		TextTitle subTitulo = new TextTitle("DCT - transformada discreta do co-seno");
		chart.addSubtitle(subTitulo);

		// Cria e mostra um frame...
		ChartFrame frame = new ChartFrame("Histograma", chart);

		// Cor de fundo do quadro 
		Paint pq = new GradientPaint(0, 0, Color.white, 1000, 0, Color.blue);
		chart.setBackgroundPaint(pq);

		// Cor de fundo da area do grafico
		Plot plot = chart.getPlot();
		Paint pc = new GradientPaint(0, 0, Color.green, 1000, 0, Color.white);
		plot.setBackgroundPaint(pc);

		frame.pack();
		frame.setVisible(true);
	}

	// DCTImageBins
	// Fonte: Tariq Rashid
	//				MSc Machine Learning and Data Mining
	//				University of Bristol
	//				Clustering of Fuzzy Image Features
	//				30th September 2003

	public static double[] DCTImageBins(PlanarImage image, int bins) {
		/* create the Discrete Cosine Transform of the image */
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(image);
		RenderedOp dct = JAI.create("dct", pb);

		/* Get the DCT image data. */
		int width = dct.getWidth();
		int height = dct.getHeight();
		int numBands = dct.getSampleModel().getNumBands();
		int dataType = dct.getSampleModel().getDataType();
		double[] dctData = dct.getData().getPixels(0, 0, width, height, (double[])null);

		/* create bins for the magnitude data and create sums */
		int halfway = dctData.length/2;
		int steps = halfway/bins;

		/* calculate total mass for use in later normalisation */
		double total_mass = 0.0;
		
		/* ignore the zeroth point, which is only the average of the signal */
		for (int i = 1; i < halfway; i++) {
			total_mass += Math.abs(dctData[i]);
		}

		/* binning of the dct mass */	
		double dct_binned_array[] = new double[bins];
		for (int b = 0; b < bins; b++) {
			/* use first half of DCT, the second half is not usually informative */
			int start = (steps * b);
			int end = start + steps;
			
			/* for each bin, sum mass */
			double mass = 0.0;
			for (int i = start; i < end; i++) {
				if (i != 0) {
					/* ignore the zeroth point, which is only the average of the signal */
					mass += Math.abs(dctData[i]);
				}
			}
			double normalised_mass = mass/total_mass;
			System.out.println(" for first-half dct bin " + b + ", normalised mass = " 
				+ normalised_mass);
			if (Double.isNaN(normalised_mass) | Double.isInfinite(normalised_mass)) {
				System.out.println("NaN or Infinite - setting normalised_mass to zero");
				normalised_mass = 0.0;
			}
			dct_binned_array[b] = normalised_mass;
		}
		return dct_binned_array;
	}

}
