/***********************************************************************************
JAIDFT.java - Transformada discreta de Fourier com JAI.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Transformada discreta de Fourier com JAI.
    O objetivo desse programa  mostrar o que acontece quando zeramos a informao
    de fase de uma imagem no domnio de freqncia. Ao recompor a imagem sem a 
    informao de fase, aparece uma nova imagem, diferente da original.
    A sada deste programa  a mesma do JAIDFTFase0.java.

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com imagem original e imagem sem informao de fase do domnio de 
  freqncia.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// Pacotes do AWT
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.image.DataBuffer;
import java.awt.geom.AffineTransform;
import java.awt.image.renderable.ParameterBlock;

// Pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// Pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.PlanarImage;
import javax.media.jai.ImageLayout;

public class JAIDFT extends JFrame {
	
  public static void main(String args[]) {

		// Confere nome de arquivo na lista de argumentos
		if (args.length != 1) {
	   	System.out.println("Uso: java JAIDFT <arquivo_imagem>");
	   	System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
		JFrame f = new JAIDFT(args[0]);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIDFT(String aFile) {

		PlanarImage sourceImg = null;
		int w, h, wdst, hdst;
		JLabel img1, img2;

		// Carrega o arquivo com imagem
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Falha ao carregar o arquivo: '" + aFile + "'");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("DFT com fase 0: " + tmpFile.getName());

		w = sourceImg.getWidth();
		h = sourceImg.getHeight();

		// Se imagem tem mais de uma banda, emite aviso
		if (sourceImg.getSampleModel().getNumBands() > 1) {
			String msgA = "\nAviso: imagem com mais de uma banda. ";
			String msgB = "Resultado inesperado!!\n";
			String msgC = "\tPara obter resultado esperado, ";
			String msgD = "transforme imagem para TYPE_BYTE_GRAY.\n";
			System.out.println(msgA+msgB+msgC+msgD);
		}

	  // Cadeia de modelagem DFT
    PlanarImage idft = criaGrafoDFT(sourceImg);

		// Para restaurar imagem original,  preciso criar
		// um objeto RenderingHints com leiaute adequado
		PlanarImage dst = imagemRH(sourceImg, idft);

		// Mostra as imagens
		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem original
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));

		// Imagem original, apos DFT e IDFT
		img2 = new JLabel(new ImageIcon(dst.getAsBufferedImage()));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));

		// Pega valores de tamanho e largura das imagens
		setSize(2*w, h);
  }

	public PlanarImage criaGrafoDFT(PlanarImage src) {

		// Calcula a DFT da imagem fonte. Duas imagens so geradas:
		// 1. espectro de amplitude (magnitude)
		// 2. espectro de fase
		PlanarImage dft  = imagemDFT(src);

		// Armazena a amplitude na imagem 'mag'
		PlanarImage mag  = imagemMagnitude(dft);

		// Armazena a fase na imagem 'fase'
		PlanarImage fase  = imagemFase(dft);

		// Zera o espectro de fase
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(fase);
		pb.add(new double[] {0.0});
		PlanarImage fase0 = JAI.create("multiplyConst", pb);

		// Armazena os dois espectros na imagem complexa 'complex'
		// Este passo  necessario para executar a inversa da DFT
		PlanarImage complex  = imagemPolar2Complex(mag, fase0);

		// Retorna a IDFT da imagem complexa
    return imagemInversaDFT(complex);
	}

	// Calcula a transformada direta de Fourier
	public PlanarImage imagemDFT(PlanarImage src) {

		System.out.println("Criando DFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		return JAI.create("dft", pb);
	}

	// Calcula a imagem com magnitude da imagem DFT
	public PlanarImage imagemMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitude", pb);
	}

	// Calcula a imagem com fase da imagem DFT
	public PlanarImage imagemFase(PlanarImage dft) {

		System.out.println("Criando imagem fase...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("phase", pb);
	}

	// Calcula imagem complexa 
	public PlanarImage imagemPolar2Complex(PlanarImage mag, PlanarImage phase) {

		System.out.println("Criando imagem complexa...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(mag);
		pb.addSource(phase);
		return JAI.create("polarTocomplex", pb);
	}

	// Calcula a imagem inversa da DFT
	public PlanarImage imagemInversaDFT(PlanarImage dft) {

		System.out.println("Criando imagem IDFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("idft", pb);
	}

	public PlanarImage imagemRH(PlanarImage src, PlanarImage idft) {

		System.out.println("Transformando IDFT (real) para PlanarImage (byte)...");
		ImageLayout il = new ImageLayout();
    il.setSampleModel(src.getSampleModel());
    il.setColorModel(src.getColorModel());
    RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

    // Converte os dados para byte.
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(idft);
    pb.add(DataBuffer.TYPE_BYTE);
    return JAI.create("format", pb, rh);
	}
}
