/***********************************************************************************
JAIDFTFiltroIdeal.java - Implementa filtro ideal no domnio de freqncia

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa filtro ideal no domnio de freqncia usando transformada discreta
    de Fourier (DFT).

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. raio do filtro. Um valor no intervalo [0,1]. Para filtros de passa-baixa, 
     quanto mais prximo de zero for o raio, mais 'borrada' ser a imagem. Para
     filtros de passa-alta, quanto mais prximo de 1 for o raio, menos 
     efeito de borda ser percebido.
  3. tipo do filtro: 
     1 = passa-baixa.
     2 = passa-alta.

Sada:

  Painel com imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// Pacotes do AWT
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.SampleModel;
import java.awt.image.renderable.ParameterBlock;

// Pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// Pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.TiledImage;
import javax.media.jai.PlanarImage;
import javax.media.jai.ImageLayout;
import javax.media.jai.operator.DFTDescriptor;

public class JAIDFTFiltroIdeal extends JFrame {
	
  public static void main(String args[]) {

		double raio = 0.0f;
		int tFiltro = 0;
		// Checa nome de arquivo na lista de argumentos
		if (args.length != 3) {
			String msgA = "Uso: java JAIDFTFiltroIdeal <arquivo_imagem>";
			String msgB = " <raio> <tipo do Filtro>";
	   	System.out.println(msgA + msgB);
	   	System.exit(0);
		}

    try {
        raio = Double.parseDouble(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'raio' invalido");
        System.exit(0);
    }

    try {
        tFiltro = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
        String str = "Valor do parametro tipo de filtro ('tFiltro') invalido";
        System.out.println(str);
        System.exit(0);
    }

		if (tFiltro <= 0)	{
        String str = "Tipo de filtro: 1 (passa-baixa) ou 2 (passa-alta)";
        System.out.println(str);
        System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
		JFrame f = new JAIDFTFiltroIdeal(args[0], raio, tFiltro);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIDFTFiltroIdeal(String aFile, double raio, int tFiltro) {

		PlanarImage sourceImg = null, idft = null, dst = null;
		int w, h;
		JLabel img1, img2;

		// Carrega o arquivo com imagem
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("Fourier: filtro ideal: " + tmpFile.getName());

		w = sourceImg.getWidth();
		h = sourceImg.getHeight();
		raio = Math.min(Math.min(w, h), raio);

		if (sourceImg.getSampleModel().getNumBands() > 1) {
			String msgA = "\nAviso: imagem com mais de uma banda. ";
			String msgB = "Resultado inesperado!!\n";
			String msgC = "\tPara obter resultado esperado, ";
			String msgD = "transforme imagem para TYPE_BYTE_GRAY.\n";
			System.out.println(msgA+msgB+msgC+msgD);
		}

		idft = criaGrafoDFT(sourceImg, raio, tFiltro);

		// Para restaurar imagem original,  preciso criar
		// um objeto RenderingHints com leiaute adequado
		dst = imagemRH(sourceImg, idft);

		// Mostra as imagens
		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem original
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));

		// Imagem filtrada, apos DFT e IDFT
		img2 = new JLabel(new ImageIcon(dst.getAsBufferedImage()));

		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));

		// Pega valores de tamanho e largura das imagens
		setSize(2*w, h);
  }

	public PlanarImage criaGrafoDFT(PlanarImage a, double raio, int tFiltro) {

		// Calcula a DFT da imagem: dftA = DFT(a)
		PlanarImage dftA  = imagemDFT(a);

		// Separa componentes do espectro de freqncia: fase e magnitude
		PlanarImage fase  = imagemFase(dftA);
		PlanarImage mag   = imagemMagnitude(dftA);

		// Cria TiledImage, para permitir manipulao de pixels
		TiledImage magTI  = criaTI(mag);
		PlanarImage magF  = null;
		PlanarImage complex = null;

		if (tFiltro == 1){
			// Filtro de passa-baixas
			magF  = filtroIdealPassaBaixa(magTI, raio);
		}	else {
			// Filtro de passa-altas
			magF  = filtroIdealPassaAlta(magTI, raio);
		}

		// Converte de polar para complexo
		complex = imagemPolar2Complex(magF, fase);

		// Retorna a IDFT da imagem complexa
    return imagemInversaDFT(complex);
	}

	public TiledImage filtroIdealPassaBaixa(TiledImage ti, double raio) {

		int width, height;

		width = ti.getWidth();
		height = ti.getHeight();
		int u2 = width/2;
    int v2 = height/2;
    int su, sv;
    double r, rmax = Math.min(u2, v2);

    if (raio < 0.0 || raio > 1.0) {
      System.out.println("Raio do filtro deve estar no intervalo [0.0,1.0] ");
			System.exit(0);
		}

		for (int v = 0; v < height; v++) {
      sv = shift(v, v2) - v2;
      for (int u = 0; u < width; u++) {
        su = shift(u, u2) - u2;
        r = Math.sqrt(su*su + sv*sv)/rmax;
        if (r > raio)
					ti.setSample(u, v, 0, 0.0f);
      }
    }
		return ti;
	}

	public TiledImage filtroIdealPassaAlta(TiledImage ti, double raio) {

		int width, height;

		width = ti.getWidth();
		height = ti.getHeight();
		int u2 = width/2;
    int v2 = height/2;
    int su, sv;
    double r, rmax = Math.min(u2, v2);

    if (raio < 0.0 || raio > 1.0) {
      System.out.println("Raio do filtro deve estar no intervalo [0.0,1.0] ");
			System.exit(0);
		}

    for (int v = 0; v < height; v++) {
      sv = shift(v, v2) - v2;
      for (int u = 0; u < width; u++) {
        su = shift(u, u2) - u2;
        r = Math.sqrt(su*su + sv*sv)/rmax;
        if (r < raio)
					ti.setSample(u, v, 0, 0.0f);
      }
    }
		return ti;
	}

  public int shift(int ponto, int centro) {
    return (ponto >= centro ? ponto-centro : ponto + centro);
  }

	public TiledImage criaTI(PlanarImage src) {
		SampleModel sm = src.getSampleModel();
		ColorModel  cm = src.getColorModel();

		TiledImage ti = new TiledImage(src.getMinX(), src.getMinY(),
																	 src.getWidth(), src.getHeight(),
																	 src.getTileGridXOffset(),
																	 src.getTileGridYOffset(), sm, cm);
		ti.setData(src.copyData());
		return ti;
	}

	// Calcula a transformada direta de Fourier
	public PlanarImage imagemDFT(PlanarImage src) {

		System.out.println("Criando DFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		return JAI.create("dft", pb);
	}

	// Multiplica duas imagens complexas
	public PlanarImage imagemMultiplyComplex(PlanarImage dftA, PlanarImage dftB) {

		System.out.println("Multiplicando imagens complexas...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dftA);
		pb.addSource(dftB);
		return JAI.create("MultiplyComplex", pb);
	}

	// Calcula a imagem com magnitude da imagem DFT
	public PlanarImage imagemMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitude", pb);
	}

	// Calcula o espectro de potncia
	public PlanarImage imagemQuadradoMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem quadrado da magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitudeSquared", pb);
	}

	// Calcula a imagem com fase da imagem DFT
	public PlanarImage imagemFase(PlanarImage dft) {

		System.out.println("Criando imagem fase...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("phase", pb);
	}

	// Calcula imagem complexa 
	public PlanarImage imagemPolar2Complex(PlanarImage mag, PlanarImage phase) {

		System.out.println("Criando imagem complexa...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(mag);
		pb.addSource(phase);
		return JAI.create("polarTocomplex", pb);
	}

	// Calcula a translao peridica de uma imagem
	public PlanarImage imagemDeslocaUV(PlanarImage src, int shiftU, int shiftV) {

		System.out.println("Calculando translacao periodica...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(shiftU);
		pb.add(shiftV);
		return JAI.create("periodicShift", pb);
	}

	// Calcula a imagem inversa da DFT
	public PlanarImage imagemInversaDFT(PlanarImage dft) {

		System.out.println("Criando imagem IDFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("idft", pb);
	}

	public PlanarImage imagemRH(PlanarImage src, PlanarImage idft) {

		System.out.println("Transformando IDFT (real) para PlanarImage (byte)...");
		ImageLayout il = new ImageLayout();
    il.setSampleModel(src.getSampleModel());
    il.setColorModel(src.getColorModel());
    RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

    // Converte os dados para byte.
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(idft);
    pb.add(DataBuffer.TYPE_BYTE);
    return JAI.create("format", pb, rh);
	}

}
