/***********************************************************************************
JAIDFTMag0.java - Elimina a informao de magnitude de uma image no domnio de 
                  freqncia.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Elimina a informao de magnitude de uma image no domnio de freqncia.

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com imagem original e imagem original sem a informao da magnitude 
  do domnio de freqncia.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// Pacotes do AWT
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.image.DataBuffer;
import java.awt.image.ColorModel;
import java.awt.image.SampleModel;
import java.awt.image.renderable.ParameterBlock;

// Pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JFrame;

// Pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.TiledImage;
import javax.media.jai.PlanarImage;
import javax.media.jai.ImageLayout;
import javax.media.jai.operator.DFTDescriptor;

public class JAIDFTMag0 extends JFrame {
	
  public static void main(String args[]) {

		int raio = 0, tFiltro = 0;
		// Checa nome de arquivo na lista de argumentos
		if (args.length != 1) {
			String msg = "Uso: java JAIDFTMag0 <arquivo_imagem>";
	   	System.out.println(msg);
	   	System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor
		JFrame f = new JAIDFTMag0(args[0]);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIDFTMag0(String aFile) {

		PlanarImage sourceImg = null;
		int w, h;
		JLabel img1, img2;

		// Carrega o arquivo com imagem
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("Fourier com magnitude 0: " + tmpFile.getName());

		w = sourceImg.getWidth();
		h = sourceImg.getHeight();

		// Se imagem tem mais de uma banda, emite aviso
		if (sourceImg.getSampleModel().getNumBands() > 1) {
			String msgA = "\nAviso: imagem com mais de uma banda. ";
			String msgB = "Resultado inesperado!!\n";
			String msgC = "\tPara obter resultado esperado, ";
			String msgD = "transforme imagem para TYPE_BYTE_GRAY.\n";
			System.out.println(msgA+msgB+msgC+msgD);
		}

		// Cadeia de modelagem DFT
    PlanarImage idft = criaGrafoDFT(sourceImg);

		// Para restaurar imagem original,  preciso criar
		// um objeto RenderingHints com leiaute adequado
		PlanarImage dst = imagemRH(sourceImg, idft);

		// Mostra as imagens
		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem original
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));

		// Imagem filtrada, aps DFT e IDFT
		img2 = new JLabel(new ImageIcon(dst.getAsBufferedImage()));

		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));

		// Pega valores de tamanho e largura das imagens
		setSize(2*w, h);
  }

	public PlanarImage criaGrafoDFT(PlanarImage a) {

		// Calcula a DFT da imagem: dftA = DFT(a)
		PlanarImage dftA  = imagemDFT(a);

		// Separa componentes do espectro de freqncia: fase e magnitude
		PlanarImage fase  = imagemFase(dftA);
		PlanarImage mag   = imagemMagnitude(dftA);

		// Cria TiledImage, para permitir manipulao de pixels
		TiledImage mag0 = criaTI(mag);

		// Substitui informao de magnitude
		mag0 = zeraMagnitude(mag0);
		
		// Converte de polar para complexo
		PlanarImage complex = imagemPolar2Complex(mag0, fase);

		// Retorna a IDFT da imagem complexa
    return imagemInversaDFT(complex);
	}

	public TiledImage zeraMagnitude(TiledImage ti) {

		double pixel;
		int width, height;

		width = ti.getWidth();
		height = ti.getHeight();

		// Substitui os valores de freqncia por uma constante
		pixel =	ti.getSample(1, 1, 0);
		for (int u = 0; u < height; u++)
			for (int v = 0; v < width; v++) {
					ti.setSample(v, u, 0, pixel);
			}
		return ti;
	}

	public TiledImage criaTI(PlanarImage src) {
		SampleModel sm = src.getSampleModel();
		ColorModel  cm = src.getColorModel();

		TiledImage ti = new TiledImage(src.getMinX(), src.getMinY(),
																	 src.getWidth(), src.getHeight(),
																	 src.getTileGridXOffset(),
																	 src.getTileGridYOffset(), sm, cm);
		ti.setData(src.copyData());
		return ti;
	}

	// Calcula a transformada direta de Fourier
	public PlanarImage imagemDFT(PlanarImage src) {

		System.out.println("Criando DFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		return JAI.create("dft", pb);
	}

	// Multiplica duas imagens complexas
	public PlanarImage imagemMultiplyComplex(PlanarImage dftA, PlanarImage dftB) {

		System.out.println("Multiplicando imagens complexas...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dftA);
		pb.addSource(dftB);
		return JAI.create("MultiplyComplex", pb);
	}

	// Calcula a imagem com magnitude da imagem DFT
	public PlanarImage imagemMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitude", pb);
	}

	// Calcula o espectro de potncia
	public PlanarImage imagemQuadradoMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem quadrado da magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitudeSquared", pb);
	}

	// Calcula a imagem com fase da imagem DFT
	public PlanarImage imagemFase(PlanarImage dft) {

		System.out.println("Criando imagem fase...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("phase", pb);
	}

	// Calcula imagem complexa 
	public PlanarImage imagemPolar2Complex(PlanarImage mag, PlanarImage phase) {

		System.out.println("Criando imagem complexa...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(mag);
		pb.addSource(phase);
		return JAI.create("polarTocomplex", pb);
	}

	// Calcula a translao peridica de uma imagem
	public PlanarImage imagemDeslocaUV(PlanarImage src, int shiftU, int shiftV) {

		System.out.println("Calculando translacao periodica...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(shiftU);
		pb.add(shiftV);
		return JAI.create("periodicShift", pb);
	}

	// Calcula a imagem inversa da DFT
	public PlanarImage imagemInversaDFT(PlanarImage dft) {

		System.out.println("Criando imagem IDFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("idft", pb);
	}

	public PlanarImage imagemRH(PlanarImage src, PlanarImage idft) {

		System.out.println("Transformando IDFT (real) para PlanarImage (byte)...");
		ImageLayout il = new ImageLayout();
    il.setSampleModel(src.getSampleModel());
    il.setColorModel(src.getColorModel());
    RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

    // Converte os dados para byte.
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(idft);
    pb.add(DataBuffer.TYPE_BYTE);
    return JAI.create("format", pb, rh);
	}

}
