/***********************************************************************************
JAIDFTViewMag.java - Visualiza a magnitude de uma imagem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Visualiza a magnitude (domnio de freqncia) de uma imagem.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. 'shift' - desloca, ou no, centro da imagem de freqncia.

Sada:

  Painel com imagem original e imagem magnitude do domnio de freqncia.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// Pacotes do AWT
import java.awt.GridLayout;
import java.awt.image.DataBuffer;
import java.awt.image.renderable.ParameterBlock;

// Pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JFrame;

// Pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.TiledImage;
import javax.media.jai.PlanarImage;
import javax.media.jai.ImageLayout;

public class JAIDFTViewMag extends JFrame {
	
  public static void main(String args[]) {

		int shift = 0;		// sem deslocamento do espectro
		// Checa nome de arquivo e shift na lista de argumentos
		if (args.length != 2) {
			String msg = "Uso: java JAIDFTViewMag <arquivo_imagem> <shiftMag: 0|1>";
	   	System.out.println(msg);
	   	System.exit(0);
		}

    try {
        shift = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'shift' invalido");
        System.exit(0);
    }

    if ((shift < 0) || (shift > 1)) {
        System.out.println("shift: {0, 1}");
        System.exit(0);
    }

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor
		JFrame f = new JAIDFTViewMag(args[0], shift);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIDFTViewMag(String aFile, int shift) {

		PlanarImage sourceImg = null;
		JLabel img1, img2;
		int w, h, wdst, hdst;

		// Carrega o arquivo com imagem
		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Atribui nome ao frame
		File tmpFile = new File(aFile);
		setTitle("Espectro de magnitude: " + tmpFile.getName());

		w = sourceImg.getWidth();
		h = sourceImg.getHeight();

	  // Cadeia de modelagem DFT
		PlanarImage dst = criaCadeiaDFT(sourceImg, shift); 
  	wdst = dst.getWidth();
		hdst = dst.getHeight();

		// Mostra as imagens
		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem original
		img1 = new JLabel(new ImageIcon(sourceImg.getAsBufferedImage()));

		// Imagem original, apos DFT e IDFT
		img2 = new JLabel(new ImageIcon(dst.getAsBufferedImage()));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));

		// Pega valores de tamanho e largura das imagens
		setSize(2*w, h);
  }

	public PlanarImage criaCadeiaDFT(PlanarImage src, int shift) {

		// Calcula a DFT da imagem fonte. Duas imagens so geradas:
		// 1. espectro de amplitude (magnitude)
		// 2. espectro de fase
		PlanarImage dft  = imagemDFT(src);

		// Armazena a amplitude na imagem 'mag'
		PlanarImage mag  = imagemMagnitude(dft);

		// Retorna a magnitude em forma visvel
    return viewMag(mag, shift);
	}

	// Calcula a transformada direta de Fourier
	public PlanarImage imagemDFT(PlanarImage src) {

		System.out.println("Criando DFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		return JAI.create("dft", pb);
	}

	// Calcula a imagem com magnitude da imagem DFT
	public PlanarImage imagemMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitude", pb);
	}

	// Calcula o espectro de potncia
	public PlanarImage imagemQuadradoMagnitude(PlanarImage dft) {

		System.out.println("Criando imagem quadrado da magnitude...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("magnitudeSquared", pb);
	}

	// Calcula a imagem com fase da imagem DFT
	public PlanarImage imagemFase(PlanarImage dft) {

		System.out.println("Criando imagem fase...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("phase", pb);
	}

	// Calcula imagem complexa 
	public PlanarImage imagemPolar2Complex(PlanarImage mag, PlanarImage phase) {

		System.out.println("Criando imagem complexa...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(mag);
		pb.addSource(phase);
		return JAI.create("polarTocomplex", pb);
	}

	// Calcula a translao peridica de uma imagem
	public PlanarImage imagemDeslocaUV(PlanarImage src, int shiftU, int shiftV) {

		System.out.println("Calculando translacao periodica...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(shiftU);
		pb.add(shiftV);
		return JAI.create("periodicShift", pb);
	}

	// Calcula a imagem inversa da DFT
	public PlanarImage imagemInversaDFT(PlanarImage dft) {

		System.out.println("Criando imagem IDFT...");
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(dft);
		return JAI.create("idft", pb);
	}

	public PlanarImage viewMag(PlanarImage magnitude, int shiftMag) {

		int dataType = 0, w, h;
		double[] max, min;
		double maximo = 65535, scale, offset;

		// 1. Calcula Log da imagem, para visualizar
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(magnitude);
		PlanarImage magLog = JAI.create("Log", pb);

		// 2. Calcula valores extremos (min, max) para re-escalar nveis de cinza
		pb = new ParameterBlock();
		pb.addSource(magLog);
		PlanarImage magExt = JAI.create("Extrema", pb);

		// 3. Re-escala nveis de cinza da imagem para o intervalo 0-255
		max = (double[]) magExt.getProperty("maximum");
		min = (double[]) magExt.getProperty("minimum");

		if ( dataType == DataBuffer.TYPE_BYTE )
			maximo = 255;
		
		scale = maximo/(max[0] - min[0]);
		offset = maximo*min[0]/(min[0] - max[0]);
		pb = new ParameterBlock();
		pb.addSource(magExt);
		pb.add( new double[] {scale} );
		pb.add( new double[] {offset} );
		PlanarImage magRes = JAI.create("Rescale", pb);

		// 4. Formata a imagem para visualizacao
		pb = new ParameterBlock();
		pb.addSource(magRes);
		pb.add( new Integer(dataType) );
		PlanarImage magForm = JAI.create("format", pb);

		// 5. Se necessrio, desloca o espectro de amplitude 
		//		para o centro da imagem
		if (shiftMag != 0)	{
				w = magnitude.getWidth()/2;
				h = magnitude.getHeight()/2;
				return imagemDeslocaUV(magForm, w, h);
		} else
				return magForm;
	}

}
