/***********************************************************************************
JAIEstat.java - Mostra estatstica e histograma da imagem usando a API JAI e 
                JFreeChart.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Mostra estatstica e histograma da imagem usando a API JAI e JFreeChart.

Parmetros de entrada:

  1. nome do arquivo imagem.

Sada:

  Painel com histograma da imagem e estatsticas da imagem na janela DOS.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do JFreeChart
import org.jfree.data.XYSeriesCollection;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.TextTitle;
import org.jfree.chart.plot.Plot;
import org.jfree.data.XYDataset;
import org.jfree.data.XYSeries;

// Pacotes do AWT
import java.awt.Color;
import java.awt.Paint;
import java.awt.GridLayout;
import java.awt.GradientPaint;
import java.awt.image.renderable.ParameterBlock;

// Pacotes do JAI
import javax.media.jai.JAI;
import javax.media.jai.Histogram;
import javax.media.jai.RenderedOp;
import javax.media.jai.PlanarImage;

public class JAIEstat {
	
	double[] mean;
	double min, max;

	public static void main(String args[]) {

		// Checa nome de arquivo na lista de argumentos
		if (args.length != 1) {
	   	System.out.println("Uso: java JAIEstat <arquivo_imagem>");
	   	System.exit(0);
		}

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

    // Chama o construtor
		JAIEstat f = new JAIEstat(args[0]);

		// Pega tempo final
		e_time = System.currentTimeMillis() - e_time;
		System.out.println("Tempo de execucao: " + e_time + " milisseg.");
  }

  public JAIEstat(String aFile) {

		PlanarImage sourceImg = null;

		try {
			sourceImg = JAI.create("fileload", aFile);
			System.out.println("Carregando imagem '" + aFile + "'");
		} catch (Exception e) {
			System.out.println("Arquivo " + aFile + " com problemas.");
			System.exit(0);
		}

		// Calcula a mdia, valor mnimo e mximo dos nveis de cinza da imagem
		calculaEstat(sourceImg);

		// Calcula e mostra histograma da imagem
    File file = new File(aFile);
    String fileName = file.getName();
		calculaHisto(sourceImg, fileName);
  }

	public void calculaEstat(PlanarImage src) { 

		double [][] extrema;
		int nBandas, w, h;

		ParameterBlock pb = new ParameterBlock();
		pb.addSource(src);
		pb.add(null);					// calcula mdia de toda a imagem
		pb.add(1);						// amostra cada pixel em x
		pb.add(1);						// amostra cada pixel em y

		// Calcula a mdia da banda 0
		RenderedOp mediaOp = JAI.create("mean", pb, null);
		mean = (double[]) mediaOp.getProperty("mean");

		// Calcula valores mximo e mnimo da banda 0
		RenderedOp mm = JAI.create("extrema", pb);
		extrema = (double[][])mm.getProperty("extrema");
		min = extrema[0][0];
		max = extrema[1][0];

		// Informaes da imagem
		nBandas = src.getNumBands();
		w = src.getWidth();
		h = src.getHeight();

		System.out.println("Estatisticas da imagem:");
		System.out.println("\n\tmin = " + min + "\n\tmax = " + max +
								 "\n\tmedia = " + mean[0] + "\n\tbandas = " + nBandas +
								 "\n\tw = " + w + "\n\th = " + h + "\n");
	}

	public void calculaHisto(PlanarImage src, String fileName) { 

		// Calcula o histograma da imagem
		System.out.println("Calculando histograma da imagem...");

		// Cria bloco de params para histograma
		ParameterBlock pbh = new ParameterBlock();
		pbh.addSource(src);
		pbh.add(null);					// ROI = null, histo de toda a imagem
		pbh.add(1);							// amostra cada pixel x da imagem
		pbh.add(1);							// amostra cada pixel y da imagem
		pbh.add(new int[] {256});
		pbh.add(new double[] {0.0});
		pbh.add(new double[] {256.0});

		// Realiza a operao de histograma
		RenderedOp dst = JAI.create("histogram", pbh);

		// Recupera os dados do histograma
		Histogram hist = (Histogram) dst.getProperty("histogram");

		// Cada "bin" contm a freqncia do nvel de cinza na imagem. Ex.:
		// bin 0 = 13456, bin 1 = 10546 etc. Neste caso, acumularemos todos os
		// nveis de cinza (256) da imagem fonte.
		int[] bins = hist.getBins(0);

		// Desenha histograma com JFreeChart
		// Cria o arquivo de dados...	
		XYSeries dataset = new XYSeries("Densidade probabilidade");
		for (int i = 0; i < 256; i++) {
			dataset.add(i, bins[i]);
		}

		XYDataset xyDataSet = new XYSeriesCollection(dataset);

		// Cria o grfico...
		String titulo = "Histograma de " + fileName;
		JFreeChart chart = ChartFactory.createAreaXYChart(titulo,
												"nivel de cinza",
												"Frequencia",
												xyDataSet,
												false,				// mostra a legenda FDP
												true,
												true
		);

		// Cria e mostra um frame...
		ChartFrame frame = new ChartFrame("Histograma", chart);

		// Personaliza apresentao do histograma
		// Adiciona sub-ttulo
	  DecimalFormat df = new DecimalFormat("0.00");
		String medStr	= String.valueOf(df.format(mean[0]));
		String minStr	= String.valueOf(min);
		String maxStr	= String.valueOf(max);
		String estat = "media = " + medStr + " min = " + minStr + " max = " + maxStr;
		TextTitle subtitulo = new TextTitle(estat);
		chart.addSubtitle(subtitulo);

		// Cor de fundo do quadro 
		Paint pq = new GradientPaint(0, 0, Color.white, 1000, 0, Color.blue);
		chart.setBackgroundPaint(pq);

		// Cor de fundo da rea do grfico
		Plot plot = chart.getPlot();
		Paint pc = new GradientPaint(0, 0, Color.green, 1000, 0, Color.white);
		plot.setBackgroundPaint(pc);

		frame.pack();
		frame.setVisible(true);
	}
}
