/***********************************************************************************
Labeling.java - Implementa operao de rotulao de objetos de uma imagem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa operao de rotulao de objetos de uma imagem.

Parmetros de entrada:

  1. nome do arquivo imagem. Esse arquivo deve conter apenas objetos simples.
     Como exemplo, use o arquivo binZadeh.png ou blobs.gif, que se encontram no 
     diretrio do captulo 10.
  2. nmero de vizinhos, deve ser 4 ou 8.

Sada:

  Painel com imagem original e imagem 'rotulada'.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.GridLayout;
import java.awt.Point;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Labeling extends JFrame {

	// Conectividade - 4 ou 8.
	private int conectividade;

	// Armazena posio relativa dos vizinhos do pixel
	private Point[] delta;

	// Largura da imagem
	private int width;

	// Altura da imagem
	private int height;

	public static void main(String args[]) {

		int viz = 0;

		// Certifica a passagem da imagem
		if (args.length != 2) {
			String msg = "Uso: java -cp . Labeling <arquivo_imagem> <#vizinhos>";
			System.out.println(msg);
			System.exit(0);
		}

		try {
        viz = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro '#vizinhos' invalido");
        System.exit(0);
    }

    if ((viz != 4) && (viz != 8)) {
        System.out.println("#vizinhos: valor invalido. Deve ser 4 ou 8");
        System.exit(0);
    }

		// Pega tempo inicial
		long e_time = System.currentTimeMillis();

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando os parmetros
		JFrame f = new Labeling(args[0], viz);

    // Encerra a aplicao clicando no "close"
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		// Pega tempo final
    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.00");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
	}

	public Labeling(String aFile, int vizinhos) {
		
    // Define objeto BufferedImage para encapsular a imagem
		BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;

    // Certifica que a imagem existe
		File file = new File(aFile);
		try {
				src = ImageIO.read(file);
		} catch (Exception e) {
        System.out.println("Erro lendo arquivo: " + file);
				System.exit(0);
    }

		// Atribui nome ao frame
		setTitle("Labeling: " + file.getName());

		// Chama funo
		dest = labelOp(src, vizinhos);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

	public BufferedImage labelOp(BufferedImage src, int conectNumber) {

		// Vetor para vizinhos conectado-4
		Point[] CONEXAO_4 = {
			new Point( 1,  0),
			new Point( 0, -1),
			new Point(-1,  0),
			new Point( 0,  1)
		};

		// Vetor para vizinhos conectado-8
		Point[] CONEXAO_8 = {
			new Point( 1,  0),
			new Point( 1, -1),
			new Point( 0, -1),
			new Point(-1, -1),
			new Point(-1,  0),
			new Point(-1,  1),
			new Point( 0,  1),
			new Point( 1,  1)
		};

		// Nmero mximo de regies
		int MAX_REGIOES = 255;

    if (conectNumber == 4) {
      conectividade = 4;
      delta = CONEXAO_4;
    } else if (conectNumber == 8) {
      conectividade = 8;
      delta = CONEXAO_8;
    }

    width = src.getWidth();
    height = src.getHeight();
		int tipo = BufferedImage.TYPE_BYTE_GRAY;
		BufferedImage dest = new BufferedImage(width, height, tipo);
    WritableRaster out = dest.getRaster();
		WritableRaster in = src.copyData(null);

    int n = 1;
    for (int y = 0; y < height; ++y)
      for (int x = 0; x < width; ++x)
        if (in.getSample(x, y, 0) > 0) {
          label(in, out, x, y, n);
          ++n;
          if (n > MAX_REGIOES)
            return dest;
        }
		System.out.println("Regioes rotuladas: " + --n);

    return dest;

	} // labelOp

  //
  // Rotula recursivamente pixels com vizinhos-4 ou 8
  // 'in'  imagem a ser rotulada
  // 'out' imagem com regies rotuladas
  // 'x'   coordenada x do pixel
  // 'y'   coordenada y do pixel
  // 'n'   nmero da regio atual
  //

  public void label(WritableRaster in, WritableRaster out,
   int x, int y, int n) {

		int j, k;

		in.setSample(x, y, 0, 0);
    out.setSample(x, y, 0, n*50);
    
		for (int i = 0; i < conectividade; ++i) {
      j = x + delta[i].x;
      k = y + delta[i].y;
      if (inImage(j, k) && in.getSample(j, k, 0) > 0)
        label(in, out, j, k, n);
    }
  }

  //
  // Testa se um pixel pertence  regio rotulada
  // 'x'   coordenada x do pixel
  // 'y'   coordenada y do pixel
  // retorna 'true' se pixel estiver dentro da imagem
  //

  public boolean inImage(int x, int y) {
    return x >= 0 && x < width && y >= 0 && y < height;
  }
}
