/***********************************************************************************
LaplacianOfGauss.java - Este programa calcula os valores de uma distribuio Laplace
                        de Gauss para duas dimenses.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Este programa calcula os valores de uma distribuio Laplace de Gauss para 
    duas dimenses.
    O valor do desvio padro pode ser variado, sendo um parmetro de entrada.
    Ele serve para definir a matriz a ser usada no processo de convoluo conhecido
    como LoG (Laplacian of Guassian).
    O programa gera uma matriz de 10x10.
    A partir dessa matriz, pode-se definir um valor de filtro, ou kernel, de 
    tamanhos que vo de 3x3 at 9x9.

Parmetros de entrada:

  1. valor do desvio padro desejado.

Sada:

  Uma matriz 10x10 com os valores de LoG.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.text.DecimalFormat;

public class LaplacianOfGauss {

	public static void main(String args[]) {

		double dp = 1.0, k, m, val;

    if (args.length != 1) {
			String msga = "Uso: java -cp . LaplacianOfGauss <dp>";
      System.out.println(msga);
      System.exit(0);
    }
		
		try {
			dp = Double.parseDouble(args[0]);
    } catch ( NumberFormatException e ) {
     	System.out.println(e);
    }     

		k = 1/(Math.PI*dp*dp*dp*dp);
		m = 1/(2*dp*dp);
		
    DecimalFormat df = new DecimalFormat("0.0000");
		for (double x = -2; x < 3; x += 0.5) {
			for (double y = -2; y < 3; y += 0.5) {
				val = -k*(1 - (x*x+y*y)*m)*Math.pow(Math.E, -(x*x+y*y)*m);
				System.out.print(df.format(val) + ";");
			}
			System.out.println();
		}
	}

}