/***********************************************************************************
LogicDyadicOp.java - LogicDyadicOp - realiza operao lgica pixel-a-pixel 

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    LogicDyadicOp - realiza operao lgica pixel-a-pixel do tipo AND, OR e 
    EXCLUSIVE OR (EOR) entre duas imagens e produz uma imagem sada.
    Dois requisitos:
      1. Se as imagens no forem do mesmo tamanho, ento a primeira imagem *DEVE*
         ser menor que a segunda.
      2. As imagens *DEVEM* ser TYPE_BYTE_GRAY.

Parmetros de entrada:

  1. nome do primeiro arquivo imagem.
  2. 'op' - tipo do operador (AND, OR ou EOR). Pode ser maisculo ou minsculo.
  3. nome do segundo arquivo imagem.

Sada:

  Painel com as duas imagens originais e a imagem processada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class LogicDyadicOp extends JFrame {

   public static void main(String args[]) {

      // Certifica a passagem das imagens e do operador
      if (args.length != 3) {
         System.out.print("Uso: java LogicDyadicOp <img1> <op> <img2>");
         System.out.print("\nonde: \n<img1> imagem de entrada");
         System.out.print("\n<op>   operacao AND, OR ou EOR");
         System.out.println("\n<img2> imagem de entrada");
         System.exit(0);
      }

      if (!args[1].toUpperCase().equals("AND")) {
         if (!args[1].toUpperCase().equals("OR")) {
            if (!args[1].toUpperCase().equals("EOR")) {
               String msg = "Erro: operador invalido. Use <AND|OR|EOR>";
               System.out.println(msg);
               System.exit(0);
            }
         }
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
      LogicDyadicOp logic = new LogicDyadicOp(args[0], args[1], args[2]);

      // Encerra a aplicao clicando no "close"
      logic.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      logic.setVisible(true);
   }

   public LogicDyadicOp(String imagem1, String op, String imagem2) {

      // Define objeto BufferedImage para encapsular a imagem
      BufferedImage img1 = null, img2 = null, dest = null;
 			JLabel img1L, img2L, img3L;
			int w1, h1;
      
      // Certifica que imagem 1 existe
      File file1 = new File(imagem1);
      try {
         img1 = ImageIO.read(file1);
      } catch(Exception e) {
         System.out.println("Imagem '" + imagem1 + "' nao existe.");
         System.exit(0);
      }

      // Certifica que imagem 2 existe
      File file2 = new File(imagem2);
      try {
         img2 = ImageIO.read(file2);
      } catch(Exception e) {
         System.out.println("Imagem '" + imagem2 + "' nao existe.");
         System.exit(0);
      }

      // Certifica se tipo de imagem 1  vlido
			if (img1.getType() != BufferedImage.TYPE_BYTE_GRAY) {
         System.out.println("Imagem '" + imagem1 + "' nao eh TYPE_BYTE_GRAY.");
         System.exit(0);
			}

      // Certifica se tipo de imagem 2  vlido
			if (img2.getType() != BufferedImage.TYPE_BYTE_GRAY) {
         System.out.println("Imagem '" + imagem2 + "' nao eh TYPE_BYTE_GRAY.");
         System.exit(0);
			}

      // Certifica se dimenso  vlida
			if ((img1.getWidth() > img1.getWidth()) || 
				  (img1.getHeight() > img2.getHeight())) {
         System.out.println("imagem 1 > imagem 2");
         System.exit(0);
			}

			System.out.println("Imagem1: '" + imagem1);
      System.out.println("Imagem2: '" + imagem2);

      // Atribui nome ao frame
      setTitle("Operao lgica: " + op.toUpperCase());

      // Realiza operao lgica
      if (op.toUpperCase().equals("AND")) {
        dest = opAND(img1, img2);
        System.out.println("AND");
      } else {
         if (op.toUpperCase().equals("OR")) {
           dest = opOR(img1, img2);
           System.out.println("OR");
         } else {
            if (op.toUpperCase().equals("EOR")) {
              dest = opNOT(img1, img2);
              System.out.println("EXCLUSIVE OR");
            }
         }
       }

			w1 = dest.getWidth();
			h1 = dest.getHeight();

			// Cria gridLayout de 1 x 2
			getContentPane().setLayout(new GridLayout(1, 3));
			img1L = new JLabel(new ImageIcon(img1));
			img2L = new JLabel(new ImageIcon(img2));
			img3L = new JLabel(new ImageIcon(dest));
			getContentPane().add(new JScrollPane(img1L));
			getContentPane().add(new JScrollPane(img2L));
			getContentPane().add(new JScrollPane(img3L));
			setSize(3*w1, h1);
   }

   public BufferedImage opAND(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, result, tipo;
      
      w = src1.getWidth();
      h = src1.getHeight();
			tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage dest = new BufferedImage(w, h, tipo);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            result = pixel1 & pixel2;
            if (result > 255)
               result = 255;
            try {
                 destWR.setSample(x, y, 0, result);
            } catch (ArrayIndexOutOfBoundsException e) {
                 System.out.println("Problema escrevendo dest: " +y + ", " + x);
                 System.exit(0);
            }               
         }
      return dest;
   }     

   public BufferedImage opOR(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, result;
      
      w = src1.getWidth();
      h = src1.getHeight();
      BufferedImage dest = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_GRAY);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            result = pixel1 | pixel2;
            if (result > 255)
               result = 255;
            try {
                 destWR.setSample(x, y, 0, result);
            } catch (ArrayIndexOutOfBoundsException e) {
                 System.out.println("Problema escrevendo dest: " +y + ", " + x);
                 System.exit(0);
            }               
         }
      return dest;
   }     

   public BufferedImage opNOT(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, result;
      
      w = src1.getWidth();
      h = src1.getHeight();
      BufferedImage dest = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_GRAY);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            result = pixel1 ^ pixel2;
            if (result < 0)
               result = 0;
            try {
                 destWR.setSample(x, y, 0, result);
            } catch (ArrayIndexOutOfBoundsException e) {
                 System.out.println("Problema escrevendo dest: " +y + ", " + x);
                 System.exit(0);
            }               
         }
      return dest;
   }     

}

