/***********************************************************************************
MathDyadicOp.java - MathDyadicOp - realiza operao aritmtica pixel-a-pixel 

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    MathDyadicOp - realiza operao aritmtica pixel-a-pixel entre duas imagens e
    produz uma imagem sada.
    Dois requisitos:
      1. Se as imagens no forem do mesmo tamanho, ento a primeira imagem *DEVE*
      ser menor que a segunda.
      2. As imagens *DEVEM* ser TYPE_BYTE_GRAY.
	   
Parmetros de entrada:

  1. nome do primeiro arquivo imagem.
  2. 'op' - tipo do operador (operacao: +,-,/,x).
  3. nome do segundo arquivo imagem.

Sada:

  Painel com as duas imagens originais e a imagem processada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class MathDyadicOp extends JFrame {

   public static void main(String args[]) {

      // Certifica a passagem das imagens e do operador
      if (args.length != 3) {
         System.out.print("\nUso: java MathDyadicOp <img1> <op> <img2>");
         System.out.print("\nonde: \n<img1> imagem1");
         System.out.print("\n<op>   operacao: +,-,/,x");
         System.out.println("\n<img2> imagem2");
         System.exit(0);
      }

      if ((!args[1].equals("+")) && (!args[1].equals("-")) 
          && (!args[1].equals("/")) && (!args[1].equals("x"))) {
         System.out.print("Erro: operador invalido. ");
         System.out.println("Opcoes: +,-,/,x");
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
      MathDyadicOp math = new MathDyadicOp(args[0], args[1], args[2]);

      // Encerra a aplicao clicando no "close"
      math.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      math.setVisible(true);
   }

   public MathDyadicOp(String imagem1, String op, String imagem2) {

      // Define objeto BufferedImage para encapsular a imagem
      BufferedImage img1 = null, img2 = null, dest = null;
 			JLabel img1L, img2L, img3L;
			int w1, h1;
      
      // Certifica que as imagens existem
      File file1 = new File(imagem1);
      try {
         img1 = ImageIO.read(file1);
      } catch(Exception e) {
         System.out.println("Imagem '" + imagem1 + "' nao existe.");
         System.exit(0);
      }

      File file2 = new File(imagem2);
      try {
         img2 = ImageIO.read(file2);
      } catch(Exception e) {
         System.out.println("Imagem '" + imagem2 + "' nao existe.");
         System.exit(0);
      }

      // Certifica se tipo de imagem 1  vlido
			if (img1.getType() != BufferedImage.TYPE_BYTE_GRAY) {
         System.out.println("Imagem '" + imagem1 + "' nao eh TYPE_BYTE_GRAY.");
         System.exit(0);
			}

      // Certifica se tipo de imagem 2  vlido
			if (img2.getType() != BufferedImage.TYPE_BYTE_GRAY) {
         System.out.println("Imagem '" + imagem2 + "' nao eh TYPE_BYTE_GRAY.");
         System.exit(0);
			}

      // Certifica se dimenso  vlida
			if ((img1.getWidth() > img1.getWidth()) || 
				  (img1.getHeight() > img2.getHeight())) {
         System.out.println("imagem 1 > imagem 2");
         System.exit(0);
			}

			System.out.println("Imagem1: '" + imagem1);
      System.out.println("Imagem2: '" + imagem2);

      // Atribui nome ao frame
      if (op.equals("+"))
        setTitle("Operao aritmtica: " + "Adiciona");
      if (op.equals("-"))
        setTitle("Operao aritmtica: " + "Subtrai");
      if (op.equals("x"))
        setTitle("Operao aritmtica: " + "Multiplica");
      if (op.equals("/"))
        setTitle("Operao aritmtica: " + "Divide");

      // Realiza operao algbrica
      if (op.equals("+")) {
        dest = adiciona(img1, img2);
      } else {
         if (op.equals("-")) {
           dest = subtrai(img1, img2);
         } else {
            if (op.equals("/")) {
              dest = divide(img1, img2);
            } else {
               if (op.equals("x")) {
                 dest = multiplica(img1, img2);
               }
            }
         }
       }

      // Mostra as imagens
			w1 = dest.getWidth();
			h1 = dest.getHeight();

			// Cria gridLayout de 1 x 2
			getContentPane().setLayout(new GridLayout(1, 3));
			img1L = new JLabel(new ImageIcon(img1));
			img2L = new JLabel(new ImageIcon(img2));
			img3L = new JLabel(new ImageIcon(dest));
			getContentPane().add(new JScrollPane(img1L));
			getContentPane().add(new JScrollPane(img2L));
			getContentPane().add(new JScrollPane(img3L));
			setSize(3*w1, h1);
   }

   public BufferedImage adiciona(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, soma = 0, tipo;
      
      w = src1.getWidth();
      h = src1.getHeight();
			tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage dest = new BufferedImage(w, h, tipo);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

			System.out.println("Adicionando...");
      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            soma = pixel1 + pixel2;
            // trunca valores maiores que 255
            if (soma > 255)
               soma = 255;
            destWR.setSample(x, y, 0, soma);
         }
      return dest;
   }     

   public BufferedImage subtrai(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, sub = 0, tipo;
      
      w = src1.getWidth();
      h = src1.getHeight();
			tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage dest = new BufferedImage(w, h, tipo);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

			System.out.println("Subtraindo...");
      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            sub = pixel1 - pixel2;

						// trunca valores menores que 0
            if (sub < 0)
               sub = 0;
            destWR.setSample(x, y, 0, sub);
         }
      return dest;
   }     

   public BufferedImage divide(BufferedImage src1, BufferedImage src2) {

      int w, h, indice, pixel, tipo;
      double[] res;
      double pixel1 = 0.0, pixel2 = 0.0;
      
      w = src1.getWidth();
      h = src1.getHeight();
      res = new double[w*h];
			tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage dest = new BufferedImage(w, h, tipo);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

			System.out.println("Dividindo...");
      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            indice = y*w + x;
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            
            // Evita divisao por zero
            if (pixel2 == 0)
               pixel2 = 1;
            res[indice] = (pixel1/pixel2);
         }

			// Procedimento para escalonar os valores da diviso
			// no intervalo [0, 255]
			for (int y = 0; y < h; y++)
				for (int x = 0; x < w; x++) {
					indice = y*w+x;
					if ((res[indice] >= 1/255) && (res[indice] <= 1))
						res[indice] = res[indice]*127 + 1; 
					else
						if ((res[indice] > 1) && (res[indice] < 256))
                res[indice] = 128 + (res[indice])/2; 
        }

				// Grava valores escalonados
				for (int y = 0; y < h; y++)
					for (int x = 0; x < w; x++) {
						indice = y*w+x;
            if (res[indice] <= 1)
               pixel = (int)(127*(res[indice]+1));
            else
               pixel = (int)(128+(res[indice]/2));
            destWR.setSample(x, y, 0, pixel);
					}

      return dest;
   }     

   public BufferedImage multiplica(BufferedImage src1, BufferedImage src2) {

      int w, h, pixel1 = 0, pixel2 = 0, multi = 0, tipo;
      
      w = src1.getWidth();
      h = src1.getHeight();
			tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage dest = new BufferedImage(w, h, tipo);

      Raster src1R = src1.getRaster();
      Raster src2R = src2.getRaster();
      WritableRaster destWR = dest.getRaster();

			System.out.println("Multiplicando...");
      for (int y = 0; y < h; y++)
         for (int x = 0; x < w; x++) {
            pixel1 = src1R.getSample(x, y, 0);
            pixel2 = src2R.getSample(x, y, 0);
            multi = pixel1*pixel2;

						// trunca valores maiores que 255
            if (multi > 255)
               multi = 255;
            destWR.setSample(x, y, 0, multi);
         }
      return dest;
   }     

}

