/***********************************************************************************
MediaTrackerIO.java - Segundo mtodo de carga de imagem (modelo push)

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Segundo mtodo de carga de imagem (modelo push): MediaTracker.

Parmetros de entrada:

  1. nome do primeiro arquivo imagem.

Sada:

  Painel com a imagem.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.Frame;
import java.awt.Event;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MediaTracker;

public class MediaTrackerIO extends Frame {

   // Define um objeto Image para encapsular a imagem
   private Image imagem;

   public static void main(String args[]) throws Exception {
      // Certifica a passagem da imagem como parmetro
      if (args.length != 1) {
         System.out.println("Uso: java -cp . MediaTrackerIO <arquivo_imagem>");
         System.exit(0);
      }

      // Certifica que a imagem existe
      boolean existeFile = (new File(args[0])).exists();
      if (!existeFile) {
         System.out.println("Abortando: arquivo " + args[0] + " nao existe.");
         System.exit(0);
      }

      // Chama o construtor passando o nome do arquivo imagem
      MediaTrackerIO mtIO = new MediaTrackerIO(args[0]);
   }

   public MediaTrackerIO(String fileName) {

      // Associa objeto Image com <arquivo_imagem>
      imagem = Toolkit.getDefaultToolkit().getImage(fileName);

      // Define objeto MediaTracker
      MediaTracker mt = new MediaTracker(this);
      mt.addImage(imagem, 0);

      // Certifica carga do <arquivo_imagem>
      try {
         mt.waitForID(0);
      } catch (InterruptedException e) {
         System.out.println("MediaTrackerIO: problema lendo imagem " + fileName);
         System.exit(0);
      }

      // Atribui nome ao frame
      File file = new File(fileName);
      setTitle("MediaTrackerIO: " + file.getName());

      // Define dimenso do Frame
      setSize(imagem.getWidth(null), imagem.getHeight(null));
      setVisible(true);
   }

   // sobrecarrega o metodo grfico paint()
   public void paint(Graphics g) {
     Graphics2D g2 = (Graphics2D)g;

     // Translada origem da imagem e desenha
     g2.translate(this.getInsets().left,this.getInsets().top);
     g2.drawImage(imagem, 0, 0, this);
   }

   // Encerra a aplicao clicando no "close"
   public boolean handleEvent(Event event) {
     if (event.id == Event.WINDOW_DESTROY)
        System.exit(0);
        return false;
   }
}
