/***********************************************************************************
MultivariadaTab.java - Calcula estatstica multivariada para imagens monocromticas.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula estatstica multivariada para imagens monocromticas. Uma variante do
    programa Multivariada.java, com a apresentao das estatsticas numa tabela
    Java.

Parmetros de entrada:

  1. nomes dos arquivos imagens. Pelo menos, trs imagens. 
     Este programa foi implementado visando o uso de bandas de imagens de satlites.
     Para teste, use as imagens sumare3.gif, sumare4.gif e sumare5.gif, disponveis
     no diretrio do captulo 8. Elas correspondem s bandas 3, 4 e 5, respec-
     tivamente, do Landsat 5, sensor Thematic Mapper, da regio da microbacia do 
     crrego Taquara Branca, em Sumar-SP, com ano de aquisio de 1989.

Sada:

  Estatsticas univariadas e multivariadas impressas na janela DOS e valores
  multivariados mostrados em uma tabela Java.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.Font;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class MultivariadaTab extends JFrame {

	private	JPanel		topPanel;
	private	JTable		table;
	private	JScrollPane scrollPane;

  public static void main(String args[]) {

      // Certifica a passagem da imagem
      if (args.length < 3) {
         String msg = "Uso: java -cp . MultivariadaTab <imagem[1|2|3[..|n]>";
         System.out.println(msg);
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
			String[] files = new String[args.length];
			for (int i = 0; i < args.length; i++)
				files[i] = args[i];

      JFrame f = new MultivariadaTab(files);

      // Encerra a aplicao clicando no "close"
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setVisible(true);
  }
	
  public MultivariadaTab(String[] files) {

			// Define objeto BufferedImage para encapsular a imagem
			int nFiles = files.length;
		  System.out.println("nFiles: " + nFiles);
      BufferedImage[] src = new BufferedImage[nFiles];
			File[] fileName = new File[nFiles];
			int[] width, height;

			// Valores de altura e largura para teste de dimenso
			width = new int[nFiles];
			height = new int[nFiles];

			// Certifica que as imagens existem
			for (int i = 0; i < nFiles; i++){
			  System.out.println("arquivo: " + files[i]);
				fileName[i] = new File(files[i]);
				try {
					 src[i] = ImageIO.read(fileName[i]);
				} catch (Exception e) {
					 System.out.println("Erro lendo arquivo: " + fileName[i]);
					 System.exit(0);
				}
				width[i] = src[i].getWidth();
				height[i] = src[i].getHeight();
			}

			for (int i = 0; i < files.length-1; i++){
	      if ((width[i] != width[i+1]) && (height[i] != height[i+1])) {
					System.out.println("Imagens com dimensoes diferentes");
					System.exit(0);
				}
      }
		
      // Atribui nome ao frame
			setTitle("Multivariada");
			setSize(300, 100);
			setBackground(Color.gray);

			// Cria um painel para todos os componentes
			topPanel = new JPanel();
			topPanel.setLayout(new BorderLayout());
			getContentPane().add(topPanel);

      // Chama funo
      imagemEstatMulti(src);
	}

  public void imagemEstatMulti(BufferedImage[] aSrc) {
    
      // Variveis para as bandas
      int[]  min_bandaN, max_bandaN;
      long[] sum_bandaN, sum_sqr_bandaN;
      double[] dVAR_bandaN, dMEAN_bandaN, dDESVP_bandaN, dSECOND_bandaN;

			// Variveis globais
      long[] sum_b1xb2;
			long total = 0;
      double[] dCOVAR_b1xb2, dCORREL_b1xb2, dSUMPROD_b1xb2;
			int nBandas = aSrc.length;
			int nc = 0, nc_m = 0, nc_k = 0;

			// Inicia variveis
			min_bandaN = new int[nBandas];
			max_bandaN = new int[nBandas];
			sum_bandaN = new long[nBandas];
			sum_sqr_bandaN = new long[nBandas];
			dVAR_bandaN = new double[nBandas];
			dMEAN_bandaN = new double[nBandas];
			dDESVP_bandaN = new double[nBandas];
			dSECOND_bandaN = new double[nBandas];
			for (int i = 0; i < nBandas; i++){
				min_bandaN[i] = 255;
				max_bandaN[i] = 0;
				sum_bandaN[i] = 0;
				sum_sqr_bandaN[i] = 0;
				dVAR_bandaN[i] = 0.0d;
				dMEAN_bandaN[i] = 0.0d;
				dDESVP_bandaN[i] = 0.0d;
				dSECOND_bandaN[i] = 0.0d;
			}

			int len = 0;
			for (int i = 1; i < nBandas; i++)	{
				len += (nBandas-i);
			}
			dCOVAR_b1xb2 = new double[len];
			dCORREL_b1xb2 = new double[len];
			dSUMPROD_b1xb2 = new double[len];
			sum_b1xb2 = new long[len];
			for (int i = 0; i < len; i++)	{
				dCOVAR_b1xb2[i] = 0.0d;
				dCORREL_b1xb2[i] = 0.0d;
				dSUMPROD_b1xb2[i] = 0.0d;
			}
			
			Raster srckR = null;
      int h = aSrc[0].getHeight();
      int w = aSrc[0].getWidth();
			for (int k = 0; k < nBandas; k++)	{
				srckR = aSrc[k].getRaster();
	      for (int i = 0; i < h; i++) {
		      for (int j = 0; j < w; j++) {
            // Pega pixels da banda k
						try {
							nc = srckR.getSample(j, i, 0);
						}
						catch (Exception e){
							System.out.println(k+", "+j+", "+i);
							System.exit(0);
						}

						// Acha mximos e mnimos
            min_bandaN[k] = Math.min(nc, min_bandaN[k]);
            max_bandaN[k] = Math.max(nc, max_bandaN[k]);

						// Acumula valores para clculos univariados da banda k
            sum_bandaN[k] += nc;
            sum_sqr_bandaN[k] += nc*nc;
					}
				}
			}

			Raster srcmR = null;
			int m = 0;
			int indiceSum = 0;
			while (m < nBandas - 1) {
				for (int k = m+1; k < nBandas; k++)	{
					srcmR = aSrc[m].getRaster();
					srckR = aSrc[k].getRaster();
					for (int i = 0; i < h; i++) {
						for (int j = 0; j < w; j++) {
							// Pega pixels da banda k e k+1
							nc_m = srcmR.getSample(j, i, 0);
							nc_k = srckR.getSample(j, i, 0);
							// Acumula valores para clculo da covarincia
							sum_b1xb2[indiceSum] += nc_m*nc_k;
						}
					}
					indiceSum++;
				}
				m++;
			}

      // Valores univariados da banda k
			total = w*h;
			for (int k = 0; k < nBandas; k++)	{
				dMEAN_bandaN[k]   = (double)sum_bandaN[k]/total;
	      dSECOND_bandaN[k] = (double)sum_sqr_bandaN[k]/total;
			  dVAR_bandaN[k]    = dSECOND_bandaN[k] - dMEAN_bandaN[k]*dMEAN_bandaN[k];
	      dDESVP_bandaN[k]  = Math.sqrt(dVAR_bandaN[k]);
			}
		
			// Valores multivariados
			// Calcula soma corrigida dos produtos, covarincia e correlao
			m = 0;
			indiceSum = 0;
			while (m < nBandas - 1) {
				for (int k = m+1; k < nBandas; k++)	{
					dSUMPROD_b1xb2[indiceSum] = (double)(sum_b1xb2[indiceSum] - 
														(sum_bandaN[m]*sum_bandaN[k])/total);
					dCOVAR_b1xb2[indiceSum] = (double)(dSUMPROD_b1xb2[indiceSum]/(total - 1));
					dCORREL_b1xb2[indiceSum] = 
												dCOVAR_b1xb2[indiceSum]/(dDESVP_bandaN[m]*dDESVP_bandaN[k]);
					indiceSum++;
				}
				m++;
			}

			// Monta matriz de varincia e covarincia
			double[][] outCov = new double[nBandas][nBandas];

      // diagonal
			for (int k = 0; k < nBandas; k++)
				for (int i = 0; i < nBandas; i++)
					if (k == i)	
						outCov[k][i] = dVAR_bandaN[k];
			
      // matriz triangular inferior
			m = 0;
			for (int k = 0; k < nBandas; k++)
				for (int i = k+1; i < nBandas; i++)
					outCov[k][i] = dCOVAR_b1xb2[m++];
			
      // matriz triangular superior
			m = 0;
			for (int k = 0; k < nBandas; k++)
				for (int i = k+1; i < nBandas; i++)
					outCov[i][k] = dCOVAR_b1xb2[m++];

			// Monta matriz de correlao
			double[][] outCor = new double[nBandas][nBandas];
			// diagonal
			for (int k = 0; k < nBandas; k++)
				for (int i = 0; i < nBandas; i++)
					if (k == i)	
						outCor[k][i] = 1.0d;
			
      // matriz triangular inferior
			m = 0;
			for (int k = 0; k < nBandas; k++)
				for (int i = k+1; i < nBandas; i++)
					outCor[k][i] = dCORREL_b1xb2[m++];
			
      // matriz triangular superior
			m = 0;
			for (int k = 0; k < nBandas; k++)
				for (int i = k+1; i < nBandas; i++)
					outCor[i][k] = dCORREL_b1xb2[m++];

			// Define os formatos de impresso
			String padrao = "##.##";
			DecimalFormat formato = null;
			String[] media_banda = new String[nBandas];
			String[] varia_banda = new String[nBandas];
			String[] desvp_banda = new String[nBandas];
			String[] outCov_banda = new String[nBandas*nBandas];
			String[] outCor_banda = new String[nBandas*nBandas];
			for (int k = 0; k < nBandas; k++)	{
				formato = new DecimalFormat(padrao);
				media_banda[k] = formato.format(dMEAN_bandaN[k]);
				varia_banda[k] = formato.format(dVAR_bandaN[k]);
				desvp_banda[k] = formato.format(dDESVP_bandaN[k]);
			}
			for (int k = 0; k < nBandas; k++)
				for (int i = 0; i < nBandas; i++)
					outCov_banda[k*nBandas+i] = formato.format(outCov[k][i]);
			for (int k = 0; k < nBandas; k++)
				for (int i = 0; i < nBandas; i++)
					outCor_banda[k*nBandas+i] = formato.format(outCor[k][i]);
		
      System.out.println("\nEstatisticas univariadas:"); 
      System.out.print("\nBanda");
			for (int k = 1; k <= nBandas; k++)
				System.out.print("\t"+k);
      System.out.print("\nmedia\t");
			for (int k = 0; k < nBandas; k++)
				System.out.print(media_banda[k] + "\t");
      System.out.print("\nvar.\t");
			for (int k = 0; k < nBandas; k++)
				System.out.print(varia_banda[k] + "\t");
      System.out.print("\ndesvp.\t");
			for (int k = 0; k < nBandas; k++)
				System.out.print(desvp_banda[k] + "\t");
      System.out.print("\nminimo\t");
			for (int k = 0; k < nBandas; k++)
				System.out.print(min_bandaN[k] + "\t");
      System.out.print("\nmaximo\t");
			for (int k = 0; k < nBandas; k++)
				System.out.print(max_bandaN[k] + "\t");

			System.out.println("\n\nEstatisticas multivariadas:");
      System.out.println("\nMatriz de variancia-covariancia");
      System.out.print("\nBanda");
			for (int k = 1; k <= nBandas; k++)
				System.out.print("\t"+k);
			for (int k = 1; k <= nBandas; k++){
				System.out.print("\n"+k);
				for (int i = 0; i < nBandas; i++)
					System.out.print("\t" + outCov_banda[(k-1)*nBandas+i]);
			}

			// Cria nomes das colunas
			String columnNames[] = new String[nBandas];
			for (int k = 1; k <= nBandas; k++)
				columnNames[k-1] = "Banda"+k;

			// Cria dados
			String dataValues[][] =	new String[2*nBandas][nBandas];
			for (int lin = 1; lin <= nBandas; lin++)
				for (int col = 0; col < nBandas; col++)
					dataValues[lin-1][col] = outCov_banda[(lin-1)*nBandas+col];

			for (int lin = nBandas; lin < 2*nBandas; lin++)
				for (int col = 0; col < nBandas; col++)
					dataValues[lin][col] = outCor_banda[(lin-nBandas)*nBandas+col];

			// Cria nova instncia da tabela
			table = new JTable(dataValues, columnNames);

			// Adiciona a tabela a um painel rolvel
			scrollPane = new JScrollPane(table);
			topPanel.add(scrollPane, BorderLayout.CENTER);
			String strTitulo = "Matriz de varincia-covarincia e correlao";
			JLabel lbTitulo = new JLabel(strTitulo, SwingConstants.CENTER);
			lbTitulo.setFont(new Font("Helvetica",Font.PLAIN,12));
			getContentPane().add(lbTitulo, BorderLayout.NORTH);

			System.out.println("\n\nMatriz de correlacao");
      System.out.print("\nBanda");
			for (int k = 1; k <= nBandas; k++)
				System.out.print("\t"+k);
			for (int k = 1; k <= nBandas; k++){
				System.out.print("\n"+k);
				for (int i = 0; i < nBandas; i++)
					System.out.print("\t" + outCor_banda[(k-1)*nBandas+i]);
			}
      System.out.print("\n");
	}
}
