/***********************************************************************************
PassaAlta.java - Implementa vrios algoritmos,com base em altas freqncias da imagem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Implementa vrios algoritmos, com base em altas freqncias da imagem,
    para deteco de bordas e realce de imagens.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem original e imagens filtradas.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.event.ItemListener;
import java.awt.image.ConvolveOp;
import java.awt.event.ItemEvent;
import java.awt.BorderLayout;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.awt.Graphics2D;
import java.awt.Graphics;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class PassaAlta extends JComponent {

	// Define objeto BufferedImage para encapsular a imagem
  BufferedImage src = null;
  int w, h;
  private JComboBox opcoes;

  public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    JFrame f = new JFrame("Altas freqncias"); 
    f.getContentPane().add(new PassaAlta(file));

    // Encerra a aplicao clicando no "close"
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setSize(300, 320);
    f.setVisible(true);
	}
  
  public PassaAlta(File file) {
		int nc;

    System.out.println("Carregando imagem: " + file);

    // Armazena arquivo imagem numa BufferedImage
		try {
      src = ImageIO.read(file);
    } catch (Exception e) {
     	System.out.println("Imagem '" + file + "' nao existe.");
      System.exit(0);
    }

		// Pega altura e largura da imagem
    w = src.getWidth();
    h = src.getHeight();

    // Define GUI com objetos do Swing
    setLayout(new BorderLayout());

    // Cria um painel para hospedar o combo box
    JPanel controle = new JPanel();

    // Cria o combo box com os nomes das opes
    opcoes = new JComboBox(new String[] {"Imagem", "Realce", 
                "Bordas", "Gradiente", "Roberts", "Sobel",
							  "Prewitt", "Frei&Chen", "Laplaciano"});

    // Processa mudana de opo
    opcoes.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
				repaint();
      }
    });
    controle.add(opcoes);
    add(controle, BorderLayout.SOUTH);
	}

  public void paintComponent(Graphics gc) {
		super.paintComponent(gc);
	  Graphics2D g2 = (Graphics2D) gc;

    // Recupera a opo do combo box
    String opcao = (String)opcoes.getSelectedItem();

		// Restaura a imagem original
	  //
    if (opcao.equals("Imagem")) {
			g2.drawImage(src, 0, 0, null);
    }

	  // Realce da imagem
    //
    if (opcao.equals("Realce")) {

			long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      WritableRaster realceWR = null;
      BufferedImage realceImagem = null;

      // Define a matriz do filtro
      float matrizRealce [] = {
				 0f, -1f,  0f,
				-1f,  5f, -1f,
				 0f, -1f,  0f
			};

      // Define o filtro
			Kernel filtroRealce = new Kernel(3, 3, matrizRealce);

			// Cria objeto para convoluo
			ConvolveOp realca = new ConvolveOp(filtroRealce);
			realceWR = realca.filter(srcR, null);

			realceImagem = new BufferedImage(src.getColorModel(), realceWR, false, null);

			g2.drawImage(realceImagem, 0, 0, null);
			eq_time = System.currentTimeMillis() - eq_time;
			String msg = "Realce: tempo de execucao ";
			System.out.println( msg + eq_time + " milisseg.");
			return;
		}

	  // Deteco de bordas. 
    //
    if (opcao.equals("Bordas")) {

			long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      BufferedImage bordasImagem = null;
      WritableRaster bordasWR = null;

      // Define a matriz do filtro
      float matrizBordas [] = {
				-1f, -1f, -1f,
				-1f,  8f, -1f,
				-1f, -1f, -1f
			};

      // Define o filtro
			Kernel filtroBordas = new Kernel(3, 3, matrizBordas);

      // Cria objeto para convoluo
			ConvolveOp bordas = new ConvolveOp(filtroBordas);
			bordasWR = bordas.filter(srcR, null);

			bordasImagem = new BufferedImage(src.getColorModel(), bordasWR, false, null);

			g2.drawImage(bordasImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Deteccao de bordas: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro de gradiente
    //
	  if (opcao.equals("Gradiente")) {

			long eq_time = System.currentTimeMillis();
			int X, Y, pixelOut, tipo, col, lin, pixel[];
			int[][] janela = {
					      {0,-1}, 
				{-1, 0},         {1, 0}, 
						    {0, 1}
			};

			tipo = BufferedImage.TYPE_BYTE_GRAY;
	    BufferedImage gradImagem = new BufferedImage(w, h, tipo);
			
		  Raster srcR = src.getRaster();
			WritableRaster gradWR = gradImagem.getRaster();

			pixel = new int[4];
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					for (int v = 0; v < 4; v++)	{
						col = j + janela[v][0];
						lin = i + janela[v][1];
						pixel[v] = srcR.getSample(col, lin, 0);
					}
					X = pixel[2] - pixel[1];
					Y = pixel[3] - pixel[0];
					// Calcula valor da magnitude do gradiente usando apenas o valor
					// absoluto do gradientes nas direes x e y, em vez da raiz 
					// quadrada da soma dos quadrados. Fiz experincias com ambos os
					// mtodos. Os resultados com o valor absoluto foram melhores que
					// o da raiz quadrada, alm de ser mais rpido.
					pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
					gradWR.setSample(j, i, 0, pixelOut);
				}
			}

			g2.drawImage(gradImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Gradiente: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
		}

	  // Filtro de Roberts - Usa a equao 9.33
	  // 
	  if (opcao.equals("Roberts")) {

			long eq_time = System.currentTimeMillis();
			int  X, Y, pixelOut, tipo, col, lin, pixel[];
		  int[][] janela = {
				{-1,-1}, {0,-1},
				{-1, 0}, {0, 0}
			};

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage bobImagem = new BufferedImage(w, h, tipo);
			
      Raster srcR = src.getRaster();
      WritableRaster bobWR = bobImagem.getRaster();;
			
			// Mtodo alternativo - ver nota abaixo em Sobel
			pixel = new int[4];
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					for (int v = 0; v < 4; v++)	{
						col = j + janela[v][0];
						lin = i + janela[v][1];
						pixel[v] = srcR.getSample(col, lin, 0);
					}
					X = pixel[0] - pixel[3];
					Y = pixel[1] - pixel[2];
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          bobWR.setSample(j, i, 0, pixelOut);
        }
      }

			g2.drawImage(bobImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Roberts: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro de Sobel
    // Como se deve fazer duas convolues, temos que ler os pixels
	  // da imagem. Neste caso, no se usa o objeto ConvolveOp.
	  // Usa equaes 9.29, 9.30 e 9.31 do livro. A equao 9.29  aproximada 
	  // com os valores absolutos, a exemplo do filtro de Roberts.
	  //
	  if (opcao.equals("Sobel")) {
      long eq_time = System.currentTimeMillis();
			int  X, Y, pixelOut, tipo, col, lin, pixel[];
		  int[][] janela = {
				{-1,-1}, {0,-1}, {1,-1}, 
				{-1, 0},         {1, 0}, 
				{-1, 1}, {0, 1}, {1, 1}
			};

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage sobelImagem = new BufferedImage(w, h, tipo);
			
      Raster srcR = src.getRaster();
      WritableRaster sobelWR = sobelImagem.getRaster();
			/* Mtodo convencional, conforme equao 9.8 do livro e o cdigo
				de programa que aparece no incio do item 9.2.2
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
          X = - srcR.getSample(j-1, i-1, 0) + srcR.getSample(j+1, i-1, 0)
							- 2*srcR.getSample(j-1, i, 0) + 2*srcR.getSample(j+1, i, 0) 
							- srcR.getSample(j-1, i+1, 0) + srcR.getSample(j+1, i+1, 0);
					Y = srcR.getSample(j-1, i-1, 0) + 2*srcR.getSample(j, i-1, 0) +
							srcR.getSample(j+1, i-1, 0) 
							- srcR.getSample(j-1, i+1, 0) - 2*srcR.getSample(j, i+1, 0)
							- srcR.getSample(j+1, i+1, 0);
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          sobelWR.setSample(j, i, 0, pixelOut);
        }
      }
			*/
			// Mtodo alternativo
			pixel = new int[8];
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					for (int v = 0; v < 8; v++)	{
						col = j + janela[v][0];
						lin = i + janela[v][1];
						pixel[v] = srcR.getSample(col, lin, 0);
					}
					X = -   pixel[0] +	 pixel[2]
							- 2*pixel[3] + 2*pixel[4]
							-   pixel[5] +   pixel[7];
					Y =   pixel[0] + 2*pixel[1] + pixel[2]
							- pixel[5] - 2*pixel[6] - pixel[7];
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          sobelWR.setSample(j, i, 0, pixelOut);
        }
      }

			g2.drawImage(sobelImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Sobel: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }
	  
	  // Filtro de Prewitt - implementao similar ao Sobel
    //
	  if (opcao.equals("Prewitt")) {

      long eq_time = System.currentTimeMillis();
			int  X, Y, pixelOut, tipo;

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage prewittImagem = new BufferedImage(w, h, tipo);
			
      Raster srcR = src.getRaster();
      WritableRaster prewittWR = prewittImagem.getRaster();
			/* Mtodo convencional, conforme equao 9.8 do livro e o cdigo
			   de programa que aparece no incio do item 9.2.2
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					Y = srcR.getSample(j-1, i-1, 0) - srcR.getSample(j+1, i-1, 0) +
							srcR.getSample(j-1, i, 0)   - srcR.getSample(j+1, i, 0) +
							srcR.getSample(j-1, i+1, 0) - srcR.getSample(j+1, i+1, 0);
          X = -srcR.getSample(j-1, i-1, 0) - srcR.getSample(j, i-1, 0) -
	 						srcR.getSample(j+1, i-1, 0) + srcR.getSample(j-1, i+1, 0) +
							srcR.getSample(j, i+1, 0)   + srcR.getSample(j+1, i+1, 0);
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          prewittWR.setSample(j, i, 0, pixelOut);
        }
      }
			*/
			// Mtodo alternativo
			int col, lin, pixel[];
		  int[][] janela = {
				{-1,-1}, {0,-1}, {1,-1}, 
				{-1, 0},         {1, 0}, 
				{-1, 1}, {0, 1}, {1, 1}
			};

			pixel = new int[8];
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					for (int v = 0; v < 8; v++)	{
						col = j + janela[v][0];
						lin = i + janela[v][1];
						pixel[v] = srcR.getSample(col, lin, 0);
					}
					Y = pixel[0] - pixel[2] + pixel[3] - pixel[4] +
						  pixel[5] - pixel[7];
					X = -pixel[0] - pixel[1] -	pixel[2] + pixel[5] +
						  pixel[6] + pixel[7];
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          prewittWR.setSample(j, i, 0, pixelOut);
        }
			}

			g2.drawImage(prewittImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Prewitt: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro de Frei & Chen - implementao similar ao Sobel
    // Esse filtro gera valores reais, por isso, primeiro guardamos
		// esses valores num double array e depois fazemos a transformao
		// linear para o [0, 255].
	  if (opcao.equals("Frei&Chen")) {

			long eq_time = System.currentTimeMillis();
			int  tipo, col, lin, pixel[];
		  int[][] janela = {
				{-1,-1}, {0,-1}, {1,-1}, 
				{-1, 0},         {1, 0}, 
				{-1, 1}, {0, 1}, {1, 1}
			};
			double pixelOut, X, Y, mini = 10.0e10, maxi = 0.0, 
						 intervalo, dPixels[][];

      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage fcImagem = new BufferedImage(w, h, tipo);
			
      Raster srcR = src.getRaster();
      WritableRaster fcWR = fcImagem.getRaster();
			dPixels = new double[w][h];
			/* Mtodo convencional, conforme equao 9.8 do livro e o cdigo
				 de programa que aparece no incio do item 9.2.2.
				 O cdigo abaixo no est preparado para armazenar os valores
				 do filtro no double array!
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					Y = srcR.getSample(j-1, i-1, 0) - srcR.getSample(j+1, i-1, 0) +
							(int)1.414*srcR.getSample(j-1, i, 0) - 
							(int)1.414*srcR.getSample(j+1, i, 0) +
							srcR.getSample(j-1, i+1, 0) - srcR.getSample(j+1, i+1, 0);
          X = -srcR.getSample(j-1, i-1, 0) - (int)1.414*srcR.getSample(j, i-1, 0) -
							srcR.getSample(j+1, i-1, 0) + srcR.getSample(j-1, i+1, 0) +
							(int)1.414*srcR.getSample(j, i+1, 0) + srcR.getSample(j+1, i+1, 0);
          pixelOut = Math.abs(X) + Math.abs(Y);
					if (pixelOut > 255)	{
						pixelOut = 255;
					}
          fcWR.setSample(j, i, 0, pixelOut);
        }
      }
			*/
			// Mtodo alternativo
			// 1. Armazena valores do filtro no double array
			pixel = new int[8];
			for (int i = 1; i < h - 1; i++) {
				for (int j = 1; j < w - 1; j++) {
					for (int v = 0; v < 8; v++)	{
						col = j + janela[v][0];
						lin = i + janela[v][1];
						pixel[v] = srcR.getSample(col, lin, 0);
					}
					Y = (double)pixel[0] - (double)pixel[2] + 
						  (double)1.414*pixel[3] - (double)1.414*pixel[4] +
							(double)pixel[5] - (double)pixel[7];
					X = -(double)pixel[0] - (double)1.414*pixel[1] -
							(double)pixel[2] + (double)pixel[5] +
						  (double)1.414*pixel[6] + (double)pixel[7];
          dPixels[j][i] = Math.abs(X) + Math.abs(Y);
					mini = Math.min(mini, dPixels[j][i]);
					maxi = Math.max(maxi, dPixels[j][i]);
        }
      }

			// 2. Aplica transformao linear
			intervalo = maxi - mini;
			for (int i = 0; i < h; i++) {
				for (int j = 0; j < w; j++) {
           pixelOut = ((dPixels[j][i] - mini)/(intervalo))*255;
           fcWR.setSample(j, i, 0, (int)pixelOut);
        }
      }

			g2.drawImage(fcImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Frei&Chen: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }

	  // Filtro de Laplace
    //
	  if (opcao.equals("Laplaciano")) {

			long eq_time = System.currentTimeMillis();

      Raster srcR = src.getRaster();
      BufferedImage laplaceImagem = null;
      WritableRaster laplaceWR = null;
			/*			
      // Define a matriz do filtro
      float matrizLaplace [] = {
				 0f, -1f,  0f,
				-1f,  4f, -1f,
				 0f, -1f,  0f
			};
			*/
			// verso negativa
      float matrizLaplace [] = {
				0f,  1f,  0f,
				1f, -4f,  1f,
				0f,  1f,  0f
			};

      // Define o filtro
			Kernel filtroLaplace = new Kernel(3, 3, matrizLaplace);

      // Cria objeto para convoluo
			ConvolveOp laplace = new ConvolveOp(filtroLaplace);
			laplaceWR = laplace.filter(srcR, null);

			laplaceImagem = new BufferedImage(src.getColorModel(), laplaceWR, false, null);

			g2.drawImage(laplaceImagem, 0, 0, null);
      eq_time = System.currentTimeMillis() - eq_time;
      String msg = "Laplace: tempo de execucao ";
      System.out.println( msg + eq_time + " milisseg.");
      return;
    }
 }
	
}
