/***********************************************************************************
PassaBaixa.java - Programa para calcular o filtro de passa-baixa

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Programa para calcular o filtro de passa-baixa usando a classe ConvolveOp do
    Java2D.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. tipo de filtro.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class PassaBaixa extends JFrame {

   public static void main(String args[]) {

      int filtro = 0;

      // Certifica a passagem da imagem e valor teto
      if (args.length != 2) {
         String msga = "Uso: java -cp . PassaBaixa <imagem> <filtro>";
         System.out.println(msga);
         System.out.println("filtro: valores validos -> 1, 2, 3, 4");
         System.exit(0);
      }

      // Certifica que parmetro  legal...
      try {
         filtro = Integer.parseInt(args[1]);
      } catch (NumberFormatException e) {
         System.out.println("Valor do parametro 'filtro' invalido");
         System.exit(0);
      }

      // ... e tem valor vlido
      if ((filtro <= 0) || (filtro > 4)) {
         System.out.println("filtro: valores validos -> 1, 2, 3, 4");
         System.exit(0);
      }
 
		  // Pega tempo inicial
      long e_time = System.currentTimeMillis();

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando os parmetros
      PassaBaixa lpFiltro = new PassaBaixa(args[0], filtro);

      // Encerra a aplicao clicando no "close"
      lpFiltro.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      lpFiltro.setVisible(true);

      // Pega tempo final
      e_time = System.currentTimeMillis() - e_time;
      DecimalFormat df = new DecimalFormat("0.00");
      System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
   }

   public PassaBaixa(String aFile, int filtro) {

      // Define objeto BufferedImage para encapsular a imagem
      BufferedImage src = null, dest = null;
 			JLabel img1, img2;
			int w, h;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
          src = ImageIO.read(file);
      } catch(Exception e) {
          System.out.println("Imagem '" + aFile + "' nao existe.");
          System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Filtro passa-baixa (mdia): " + file.getName());

      // Chama funo
      System.out.println("Calculando filtro de passa-baixa... ");
      dest = filtropb(src, filtro);

			w = dest.getWidth();
			h = dest.getHeight();

			// Cria gridLayout de 1 x 2
			getContentPane().setLayout(new GridLayout(1, 2));
			img1 = new JLabel(new ImageIcon(src));
			img2 = new JLabel(new ImageIcon(dest));
			getContentPane().add(new JScrollPane(img1));
			getContentPane().add(new JScrollPane(img2));
			setSize(2*w, h);
   }

   public BufferedImage filtropb(BufferedImage src, int filtro) {

	  float matrizMedia[] = new float[9];

      Raster srcR = src.getRaster();
      WritableRaster WR = null;
      BufferedImage out = null;

      // Define a matriz do filtro
		  switch (filtro) {
        case 1: matrizMedia[0] = 1/10f;
        				matrizMedia[1] = 1/10f;
								matrizMedia[2] = 1/10f;
								matrizMedia[3] = 1/10f;
								matrizMedia[4] = 2/10f;
								matrizMedia[5] = 1/10f;
								matrizMedia[6] = 1/10f;
								matrizMedia[7] = 1/10f;
								matrizMedia[8] = 1/10f;
								break;
        case 2: matrizMedia[0] = 1/12f;
        				matrizMedia[1] = 1/12f;
								matrizMedia[2] = 1/12f;
								matrizMedia[3] = 1/12f;
								matrizMedia[4] = 4/12f;
								matrizMedia[5] = 1/12f;
								matrizMedia[6] = 1/12f;
								matrizMedia[7] = 1/12f;
								matrizMedia[8] = 1/12f;
								break;
        case 3: matrizMedia[0] = 1/20f;
        				matrizMedia[1] = 1/20f;
								matrizMedia[2] = 1/20f;
								matrizMedia[3] = 1/20f;
								matrizMedia[4] = 12/20f;
								matrizMedia[5] = 1/20f;
								matrizMedia[6] = 1/20f;
								matrizMedia[7] = 1/20f;
								matrizMedia[8] = 1/20f;
								break;
        case 4: matrizMedia[0] = 1/9f;
        				matrizMedia[1] = 1/9f;
								matrizMedia[2] = 1/9f;
								matrizMedia[3] = 1/9f;
								matrizMedia[4] = 1/9f;
								matrizMedia[5] = 1/9f;
								matrizMedia[6] = 1/9f;
								matrizMedia[7] = 1/9f;
								matrizMedia[8] = 1/9f;
								break;
			}

      // Define o filtro
      Kernel filtroMedia = new Kernel(3, 3, matrizMedia);

			// Cria objeto para convoluo
      ConvolveOp blur = new ConvolveOp(filtroMedia);
      WR = blur.filter(srcR, null);

		  out = new BufferedImage(src.getColorModel(), WR, false, null);
		  return out;
  }     

}
