/***********************************************************************************
Polar.java - Implementa um efeito com coordenadas polares.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

  Usar para imagens em nveis de cinza.

  Adaptado de:  Lyon, D. A. Image processing in Java. Upper Saddle River,NJ:Prentice
                Hall, 1999.

Parmetros de entrada:

  1.   (xc, yc) = coordenadas central de aplicao do efeito.

Sada:

  Painel com a imagem original e imagem filtrada.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.GridLayout;

// pacotes Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.*;

public class Polar extends JFrame {
	static int xc = 0, yc = 0;

  public static void main(String args[]) {

		if (args.length != 3) {
			String msga = "Uso: java -cp . Polar <imagem> <xc> <yc>";
      System.out.println(msga);
      System.exit(-1);
    }

		// L valor das coordenadas do centro de efeito
    try {
        xc = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro <xc> invalido");
        System.exit(0);
    }

    try {
        yc = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro <yc> invalido");
        System.exit(0);
    }

    if ((xc < 0) || (yc < 0)) {
        System.out.println("<xc, yc>: valores devem ser positivos");
        System.exit(0);
    }

    long e_time = System.currentTimeMillis();

		Polar rd = new Polar(args[0]);
    rd.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		rd.setSize(300, 200);
    rd.setVisible(true);

    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.0000");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milissec.");
  }

  public Polar(String aFile) {
		BufferedImage bimage = null, dest = null;
      
    System.out.println("Carregando imagem: " + aFile);

    File file = new File(aFile);
    try {
     	bimage = ImageIO.read(file);
    } catch(Exception e) {
     	System.out.println("Imagem '" + aFile + "' nao existe.");
     	System.exit(0);
	 	}

    // Atribui nome ao frame
    setTitle("Efeito coord. polar: " + file.getName());

    // chama funo
    dest = polar(bimage);

		// Cria gridLayout de 1 x 2
		System.out.println("Mostrando Imagens");
		getContentPane().setLayout(new GridLayout(1, 2));

		// Imagem 1
		JLabel lsrc = new JLabel(new ImageIcon(bimage));
		getContentPane().add(new JScrollPane(lsrc));

		// Imagem 2
		JLabel img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img2));
	}

  public BufferedImage polar(BufferedImage src) {

    int w, h, xp, yp;
		double p[] = new double[2];
		double dx, dy, radius, a;

    w = src.getWidth();
    h = src.getHeight();
    BufferedImage out = new BufferedImage(w, h, BufferedImage.TYPE_3BYTE_BGR);

		for (int x = 0; x < w; x++)
			for (int y = 1; y < h; y++) {
				dx = x - xc;
				dy = y - yc;
				radius = Math.sqrt(dx*dx + dy*dy);
				a = (180/Math.PI)*Math.atan2(dy, dx);
				a += radius;
				a = a*Math.PI/180;
				p[0] = radius*Math.cos(a);
				p[1] = radius*Math.sin(a);
				xp = (int)p[0] + xc;
				yp = (int)p[1] + yc;
				if ((xp < w) && (yp < h) && (xp >= 0) && (yp >= 0)) {
					out.setRGB(x, y, src.getRGB(xp, yp));
				}
			}

      return out;
   }     

}
