/***********************************************************************************
ROIMean.java - ROIMean: calcula mdia de uma regio retangular

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    ROIMean: calcula mdia de uma regio retangular de uma imagem.
    ROI (Region Of Interest).

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. 'x' - coordenada x do canto superior esquerdo da ROI.
  3. 'y' - coordenada y do canto superior esquerdo da ROI.
  4. 'w' - largura da ROI.
  5. 'h' - altura da ROI.
  Obs.: Se w e h da ROI so maiores que w e h da imagem, calcula mdia de toda
        imagem.

Sada:

  Mostra valor da mdia da ROI na janela DOS e a subimagem (ROI) em um painel.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.Rectangle;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ROIMean extends JFrame {

   public static void main(String args[]) {
      int x = 0, y = 0,      // canto superior esquerdo da ROI
          w = 1, h = 1;      // altura e largura da ROI

      // Certifica a passagem da imagem e outros parmetros
      if (args.length != 5) {
         String msg = "Uso: java -cp . ROIMean <arquivo_imagem> <x> <y> <w> <h>";
         System.out.println(msg);
         System.exit(0);
      }

      // L parmetros e certifica por valores vlidos
      try {
         x = Integer.parseInt(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Coordenada 'x' invalida: " + args[1]);
         System.exit(0);
      }
      try {
         y = Integer.parseInt(args[2]);
      } catch (IllegalArgumentException e) {
         System.out.println("Coordenada 'y' invalida: " + args[2]);
         System.exit(0);
      }
      try {
         w = Integer.parseInt(args[3]);
      } catch (IllegalArgumentException e) {
         System.out.println("Altura ('h') da ROI invalida: " + args[3]);
         System.exit(0);
      }
      try {
         h = Integer.parseInt(args[4]);
      } catch (IllegalArgumentException e) {
         System.out.println("Largura ('w') da ROI invalida: " + args[4]);
         System.exit(0);
      }

      // Assume valor padro
      if (x < 0)
         x = 0;
      if (y < 0)
         y = 0;
      if (x > w)
         x = 0;
      if (y > h)
         y = 0;

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

			// Chama o construtor
      ROIMean rm = new ROIMean(args[0], x, y, w, h);

      // Encerra a aplicao clicando no "close"
      rm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      rm.setSize(200, 200);
      rm.setVisible(true);
   }

   public ROIMean(String aFile, int x, int y, int w, int h) {
      // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage bimage = null;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         bimage = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Mdia da ROI: " + file.getName());

      // Certifica se dimenses esto corretas
      if (w > bimage.getWidth())
         w = bimage.getWidth() - x;

      if (h > bimage.getHeight())
         h = bimage.getHeight() - y;

      // Se w e h so maiores que imagem, calcula mdia de toda imagem
      if ((w > bimage.getWidth()) && (h > bimage.getHeight())) {
         w = bimage.getWidth();
         h = bimage.getHeight();
         x = 0;
         y = 0;
      }

      // Chama funo
      Rectangle roi = new Rectangle(x, y, w, h);
      double media = meanValue(bimage, roi);
      DecimalFormat df = new DecimalFormat("0.000");
      System.out.println("\nmedia " + df.format(media));
   }

   public double meanValue(BufferedImage img, Rectangle ROI) {

      BufferedImage biRoi = img.getSubimage(ROI.x, ROI.y, ROI.width, 
				       ROI.height);

      // Define GUI com objetos do Swing
      JLabel limg = new JLabel(new ImageIcon(biRoi));
      getContentPane().add(new JScrollPane(limg));
      setSize(biRoi.getWidth(), biRoi.getHeight());

      // Chama outro construtor de meanValue com BufferedImage ROI
      return meanValue(biRoi);
   }

   public double meanValue(BufferedImage src) {
      double sum = 0.0;

      Raster raster = src.getRaster();
      for (int i = 0; i < src.getHeight(); i++) 
        for (int j = 0; j < src.getWidth(); j++)
            sum += raster.getSample(j, i, 0);
      return sum/(src.getWidth()*src.getHeight());
   }     

}

