/***********************************************************************************
ROIMeanMouse.java - ROIMeanMouse: calcula mdia de uma regio de interesse (ROI) 
                    retangular.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    ROIMeanMouse: calcula mdia de uma regio de interesse (ROI) retangular; as 
    coordenadas so lidas com o mouse. As coordenadas do canto superior esquerdo
    (CE) e inferior direito (CD) da ROI so fornecidas pelo mouse. 
    Clicar uma vez, com o boto esquerdo do mouse, em CE e uma vez em CD.
    Ver desenho.

    +--------------------------------+
    |                                |
    |           Imagem               |
    |    CE                          |
    |      +------------             |
    |      |            |            |
    |      |    ROI     |            |
    |      |            |            |
    |      |            |            |
    |       ------------+            | 
    |                   CD           |
    |                                |
    |                                |
    +--------------------------------+

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.
  2. coordenadas dos cantos superior esquerdo e direito da ROI, fornecidas
     com o mouse.

Sada:

  Painel com a imagem original e valor da mdia da ROI na janela DOS.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.event.*;
import java.awt.Rectangle;
import java.awt.image.Raster;
import java.awt.BorderLayout;
import java.awt.image.BufferedImage;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ROIMeanMouse extends JFrame {

	int x1, y1, x2, y2;
	JLabel labelCoordenadas = new JLabel();
	String coordenadas = "";
	boolean first = true;
  BufferedImage imagem = null;

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor
    JFrame rm = new ROIMeanMouse(file);

    // Encerra a aplicao clicando no "close"
    rm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    rm.setVisible(true);
  }

  public ROIMeanMouse(File file) {

    // Armazena arquivo imagem numa BufferedImage
    try {
       imagem = ImageIO.read(file);
    } catch(Exception e) {
       System.out.println("Imagem '" + file + "' nao existe.");
       System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Mdia da ROI: " + file.getName());

    // Define GUI com objetos do Swing
		JLabel labelImagem = new JLabel(new ImageIcon(imagem));
    add(labelImagem, BorderLayout.NORTH);
    setSize(imagem.getWidth(), imagem.getHeight());

		// Adiciona evento do mouse: imprime coordenadas do ponto clicado
    add(labelCoordenadas, BorderLayout.SOUTH);

		MouseListener mlst = new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				int xmin, ymin, xmax, ymax, w, h;
				if (evt.getClickCount() == 1)
					coordenadas = evt.getX() + ", " + evt.getY();
					labelCoordenadas.setText(coordenadas);
					if (first)	{
						x1 = evt.getX();
						y1 = evt.getY();
						first = false;
					} else {
						x2 = evt.getX();
						y2 = evt.getY();
						// Chama funo
						xmin = Math.min(x1, x2);
						ymin = Math.min(y1, y2);
						xmax = Math.max(x1, x2);
						ymax = Math.max(y1, y2);
						w = xmax - xmin;
						h = ymax - ymin;
				    Rectangle roi = new Rectangle(xmin, ymin, w, h);
				    double media = meanValue(imagem, roi);
				    DecimalFormat df = new DecimalFormat("0.000");
				    System.out.println("Media ROI = " + df.format(media));
						first = true;
					}
			}
		};

		labelImagem.addMouseListener(mlst);

  }

  public double meanValue(BufferedImage img, Rectangle ROI) {

      BufferedImage biRoi = img.getSubimage(ROI.x, ROI.y, ROI.width, 
				       ROI.height);

      // Chama outro construtor de meanValue com BufferedImage ROI
      return meanValue(biRoi);
   }

   public double meanValue(BufferedImage src) {
      double sum = 0.0;

      Raster raster = src.getRaster();
      for (int i = 0; i < src.getHeight(); i++) 
        for (int j = 0; j < src.getWidth(); j++)
            sum += raster.getSample(j, i, 0);
      return sum/(src.getWidth()*src.getHeight());
   }     
}
