/***********************************************************************************
Region.java - Crescimento de regio

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Crescimento de regio

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. valor da coordenada central x para crescimento da regio.
  3. valor da coordenada central y para crescimento da regio.
  4. conectividade, ou vizinhana. Valores vlidos: 4 ou 8.
  5. limiar, valor no intervalo inteiro [0, 255]. Esse valor representa um
     intervalo de variao dos valores dos pixels vizinhos do pixel central
     (x, y) e que sero considerados pertencentes  regio. Por exemplo, se o 
     valor do pixel em (x, y) for 14, ento um limiar de 8 implica que a regio
     constar de todos os pixels, vizinhos ao pixel central (x, y), com valores 
     no intervalo [6, 22] (14 - 8 = 6 e 14 + 8 = 22).

Sada:

  Painel com a imagem original e imagens filtradas.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.util.ArrayList;
import java.awt.GridLayout;
import java.awt.Point;
import java.util.List;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Region extends JFrame {

	public static void main(String args[]) {

		int x = 0, y = 0, conect = 4, limiar = 0;

    // Certifica a passagem da imagem e parmetros
    if (args.length != 5) {
         String msga = "Uso: java -cp . Region <arquivo_imagem> <x>";
         String msgb = " <y> <conectividade> <limiar>";
         System.out.print(msga);
         System.out.print(msgb);
         System.exit(0);
    }

    try {
        x = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'x' invalido");
        System.exit(0);
    }

    if (x < 0) {
        System.out.println("coordenada x: valor invalido. x >= 0");
        System.exit(0);
    }

    try {
        y = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'y' invalido");
        System.exit(0);
    }

    if (y < 0) {
        System.out.println("coordenada y: valor invalido. y >= 0");
        System.exit(0);
    }

    try {
        conect = Integer.parseInt(args[3]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'conect' invalido");
        System.exit(0);
    }

    if ((conect != 4) && (conect != 8)) {
        System.out.println("conectividade: valor invalido. c = 4 ou 8");
        System.exit(0);
    }

    try {
        limiar = Integer.parseInt(args[4]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'limiar' invalido");
        System.exit(0);
    }

    if ((limiar < 0) && (limiar > 256)) {
        System.out.println("limiar: valor invalido. Valido [0, 255]");
        System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    Region regiao = new Region(args[0], x, y, conect, limiar);

    // Encerra a aplicao clicando no "close"
    regiao.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    regiao.setVisible(true);
  }

  public Region(String aFile, int x, int y, int conect, int limiar) {

		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
		int w, h;
		JLabel srcL, destL;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Crescimento de regio: " + file.getName());

		List seeds = new ArrayList();
		seeds.add(new Point(x, y));

		// Definimos a regio deseja de acordo com 'limiar' a partir
		// do ponto passado em 'seeds.'
		RegionGrower grower = new RegionGrower(src, seeds, conect, limiar);

		// Esse mtodo coloca 1 em locais da imagem que satisfazem o
		// critrio da regio deseja, dentro de 'limiar.'
		grower.growToCompletion();
		System.out.println(grower.getNumIterations() + " iteracoes");

		// getRegionImage retorna uma imagem com 1 nas posies cujos valores
		// estavam dentro do 'limiar' e 0 nas outras.
		dest = grower.getRegionImage();

		// Como o valor 1 retornado na imagem  muito pequeno para
		// visualizar, trocamos esse valor por 255.
		w = dest.getWidth();
		h = dest.getHeight();
    WritableRaster raster = dest.getRaster();
    for (int i = 0; i < h; ++i)
      for (int j = 0; j < w; ++j) 
        if (raster.getSample(j, i, 0) > 0)
          raster.setSample(j, i, 0, 255);
		/* Se voc quiser gravar o arquivo, modificar o diretrio
		try {
        System.out.println("Gravando arquivo 'imgRegion.png' em 'C:\\temp\\'");
				ImageIO.write(dest, "png", new File("C:\\temp\\imgRegion.png"));
		} catch (IOException e) {
				System.out.println("Problema gravando arquivo ");
        System.exit(0);
		}
		*/

		// Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		srcL = new JLabel(new ImageIcon(src));
		destL = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(srcL));
		getContentPane().add(new JScrollPane(destL));
		setSize(2*w, h);
	}
}
