/***********************************************************************************
Regress.java - Calcula coeficiente de correlao 'r' e coeficiente de determinao 
               'R**2'

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula coeficiente de correlao de Pearson 'r' e coeficiente de determinao
    'R**2' entre duas bandas monocromticas. Para teste, usar uma combinao das 
    imagens sumare3.gif, sumare4.gif e sumare5.gif (diretrio do captulo 8).

Parmetros de entrada:

  1. nome do arquivo imagem correspondente  varivel x (independente).
  1. nome do arquivo imagem correspondente  varivel y (dependente).

Sada:

  Desenha o diagrama de espalhamento e escreve a equao de regresso.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.BasicStroke;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.Color;

// pacote Swing para interface grfica
import javax.swing.JFrame;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class Regress extends JFrame {

   BufferedImage imagem_X, imagem_Y;
	
   int p_X, y_X, w_X = 0, h_X = 0,
       p_Y, y_Y, w_Y = 0, h_Y = 0,
       iMaxBanda1, iMaxBanda2, Obs, iLin, iCol;
   double a, b, r, Rsqr;

   public static void main(String args[]) {

      // Certifica a passagem da imagem
      if (args.length != 2) {
         System.out.println("Uso: java -cp . Regress <imagem_X> <imagem_Y>");
         System.exit(0);
      }

      // Pega tempo inicial
      double e_time = System.currentTimeMillis();

      // Chama o construtor passando as imagens
      JFrame f = new Regress(args[0], args[1]);
      f.setSize(300, 280);
      f.setVisible(true);

      // Encerra a aplicao clicando no "close"
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      // Pega tempo final
      e_time = System.currentTimeMillis() - e_time;
      e_time = e_time*0.001;
      String padrao = "##.###";
      DecimalFormat formato = new DecimalFormat(padrao);
      String tempo = formato.format(e_time); 
      System.out.println("Tempo de execucao: " + tempo + " milisseg. ");
   }
  
   public Regress(String aFile1, String aFile2) {
      double wt, t, SxoSS, SumX, SumY, Sigma2, Sigma, Ep, P, SumTsqr,
             SumSigma, SigmaDat, SigmaA, SigmaB, q, Chi2, MeanX,
             MeanY, Xt, Yt, SumXsqr, SumYsqr, SumXY, Tmp,
             x1, y1, x2, y2;
		
      // Certifica que a imagem existe
      File file1 = new File(aFile1);
      if (!file1.exists()) {
         System.out.println("Arquivo '" + aFile1 + "' nao existe.");
         System.exit(0);
      }

      try {
         imagem_X = ImageIO.read(file1);
      } catch (Exception e) {
         System.out.println("Erro lendo arquivo: " + file1);
         System.exit(0);
      }

      // Certifica que a imagem existe
      File file2 = new File(aFile2);
      if (!file2.exists()) {
         System.out.println("Arquivo '" + aFile2 + "' nao existe.");
         System.exit(0);
      }

      try {
         imagem_Y = ImageIO.read(file2);
      } catch (Exception e) {
         System.out.println("Erro lendo arquivo: " + file2);
         System.exit(0);
      }

      w_X = imagem_X.getWidth();
      h_X = imagem_X.getHeight();
      w_Y = imagem_Y.getWidth();
      h_Y = imagem_Y.getHeight();
      if ((w_X != w_Y) || (h_X != h_Y)) {
         System.out.println("Imagens com dimensoes diferentes.");
         System.exit(0);
      }
		
      // Atribui nome ao frame
      setTitle("Regresso: " + file1.getName() + ", " + file2.getName());

      iMaxBanda1 = 0;
      iMaxBanda2 = 0;
      b = 0.0;
      SumX = 0.0;
      SumY = 0.0;
      SumTsqr = 0.0;
      Sigma2 = 0.0;
      Ep = 0.0;
      Obs = w_X*h_X;

      Raster srcX = imagem_X.getRaster();
      Raster srcY = imagem_Y.getRaster();

      for (iLin = 0; iLin < h_Y; iLin++) {
         for (iCol = 0; iCol < w_Y; iCol++) {
             p_Y = srcY.getSample(iCol, iLin, 0);
             Ep = Ep + p_Y;
             P = p_Y*p_Y;
             Sigma2 = Sigma2 + P;
         }
      }

      Sigma2 = (double)(Sigma2 - Ep*Ep/Obs)/(Obs - 1);
      Sigma  = Math.sqrt(Sigma2);
      for (iLin = 0; iLin < h_X; iLin++) {
         for (iCol = 0; iCol < w_X; iCol++) {

             // Pixel da varivel independente
             p_X = srcX.getSample(iCol, iLin, 0);
             SumX = SumX + p_X;

             // Pixel da varivel dependente
             p_Y = srcY.getSample(iCol, iLin, 0);
             SumY = SumY + p_Y;
             iMaxBanda1 = Math.max(p_X, iMaxBanda1);
             iMaxBanda2 = Math.max(p_Y, iMaxBanda2);
         }
      }
      SumSigma = (double)Obs;
      SxoSS = SumX/SumSigma;
      for (iLin = 0; iLin < h_Y; iLin++) { 
         for (iCol = 0; iCol < w_Y; iCol++) {
             p_Y = srcY.getSample(iCol, iLin, 0);
             p_X = srcX.getSample(iCol, iLin, 0);
             t = p_X - SxoSS;
             SumTsqr = SumTsqr + t*t;
             b = b + t*p_Y;
         }
      }
      b = b/SumTsqr;
      a = (SumY - SumX*b)/SumSigma;
      SigmaA = Math.sqrt((1.0 + SumX*SumX/(SumSigma*SumTsqr))/SumSigma);
      SigmaB = Math.sqrt(1.0/SumTsqr);
      Chi2 = 0.0;
      for (iLin = 0; iLin < h_X; iLin++) {
         for (iCol = 0; iCol < w_X; iCol++) {
             p_X = srcX.getSample(iCol, iLin, 0);
             p_Y = srcY.getSample(iCol, iLin, 0);
             Tmp = p_Y - a - b*p_X;
             Chi2 = Chi2 + Tmp*Tmp;
         }
      }
      q = 1.0;
      SigmaDat = (double)Math.sqrt(Chi2/(Obs - 2));
      SigmaA = SigmaA*SigmaDat;
      SigmaB = SigmaB*SigmaDat;
      
      // Coeficiente de correlao linear - r
      MeanX = 0.0;
      MeanY = 0.0;
      SumXsqr = 0.0;
      SumYsqr = 0.0;
      SumXY = 0.0;
      for (iLin = 0; iLin < h_X; iLin++) {
         for (iCol = 0; iCol < w_X; iCol++) {
             p_X = srcX.getSample(iCol, iLin, 0);
             p_Y = srcY.getSample(iCol, iLin, 0);
             MeanX = MeanX + (double)p_X;
             MeanY = MeanY + (double)p_Y;
         }
      }
      MeanX = (double)MeanX/Obs;
      MeanY = (double)MeanY/Obs;
      for (iLin = 0; iLin < h_X; iLin++) {
         for (iCol = 0; iCol < w_X; iCol++) {
             p_X = srcX.getSample(iCol, iLin, 0);
             p_Y = srcY.getSample(iCol, iLin, 0);
             Xt = (double)p_X - MeanX;
             Yt = (double)p_Y - MeanY;
             SumXsqr = SumXsqr + Xt*Xt;
             SumYsqr = SumYsqr + Yt*Yt;
             SumXY = SumXY + Xt*Yt;
         }
      }
      r = SumXY/Math.sqrt(SumXsqr*SumYsqr);
      Rsqr = (SumXY*SumXY)/(SumXsqr*SumYsqr);
      String padrao = "##.###";
      String out1 = null;
      DecimalFormat formato = new DecimalFormat(padrao);
			String regressr = formato.format(r);
      String regressA = formato.format(a); 
      double R100 = Rsqr*100;
      String regressR = formato.format(R100);
			if (r >= 0) {
				String regressB = formato.format(b);
				out1 = "\n\ty = " + regressA + " + " + regressB + "x";
			} else {
				String regressB = formato.format(Math.abs(b));
				out1 = "\n\ty = " + regressA + " - " + regressB + "x";
			}      
      String out2 = "\n\tr = " + regressr + ", R2 = " + regressR + "%";
      System.out.println("\nEquacao de regressao:"); 
      System.out.println(out1); 
      System.out.println("\nCoef. de correlacao (r) e de determinacao (R**2):");
      System.out.println(out2 + "\n"); 
   }

   public void paint(Graphics g) {
      Graphics2D g2 = (Graphics2D) g;
      int x, y, winW, winH, taxaX, taxaY;
      double x1, y1, x2, y2;

      // Traa o grfico de espalhamento entre as duas imagens.
      // Para reduzir o nmero de iteraes, os pixels so amostrado em 1/5.
		
      winW = getSize().width;
      winH = getSize().height;

      Raster srcX = imagem_X.getRaster();
      Raster srcY = imagem_Y.getRaster();

      taxaX = (int)winW/iMaxBanda1;
      taxaY = (int)winH/iMaxBanda2;
      iLin = 0;
      iCol = 0;
      while (iLin < h_X) {
         while (iCol < w_X) {
            p_X = srcX.getSample(iCol, iLin, 0);
            p_Y = srcY.getSample(iCol, iLin, 0);
            y = winH - p_Y*taxaY;
            x = p_X*taxaX;
            drawPoint(x, y, g2);
            iCol += 5;
         }
         iLin += 5;
         iCol = 0;
      }
		
      // Desenha reta ajustada
      x1 = 0.0;
      y1 = a + b*x1;
      x2 = iMaxBanda1;
      y2 = a + b*x2;
      g2.setPaint(Color.yellow);
      g2.setStroke(new BasicStroke(3));
      g2.drawLine((int)x1*taxaX, (int)(winH - y1*taxaY), (int)x2*taxaX, 
                  (int)(winH - y2*taxaY));
		
      // Prepara variveis para escrever equao de regresso
      String padrao = "##.###";
      String out1 = null;
      DecimalFormat formato = new DecimalFormat(padrao);
      String regressA = formato.format(a); 
      String regressr = formato.format(r);
      double R100 = Rsqr*100;
      String regressR = formato.format(R100);
			if (r >= 0) {
				String regressB = formato.format(b);
				out1 = "y = " + regressA + " + " + regressB + "x";
			} else {
				String regressB = formato.format(Math.abs(b));
				out1 = "y = " + regressA + " - " + regressB + "x";
			}
      String out2 = "r = " + regressr + ", R2 = " + regressR + "%";
      g2.setPaint(Color.black);
      g2.drawString(out1, (int)winW/3, getInsets().top + 10); 
      g2.drawString(out2, (int)winW/3, getInsets().top + 25); 
      return;
   }
	
   public void drawPoint(int x, int y, Graphics2D gContxt) {
      int r = 2;
		
      Rectangle point = new Rectangle(x-r, y-r, 2*r, 2*r);
      gContxt.setPaint(Color.red);
      gContxt.fill(point);
      gContxt.setPaint(Color.black);
      gContxt.draw(point);
   }

}
