/***********************************************************************************
RegressPG.java - Calcula coeficiente de correlao 'r' e coeficiente de determinao
                 'R**2'

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula coeficiente de correlao de Pearson 'r' e coeficiente de determinao
    'R**2' entre duas bandas monocromticas. Para teste, usar uma combinao das 
    imagens sumare3.gif, sumare4.gif e sumare5.gif (diretrio do captulo 8).
    Implementa o objeto PixelGrabber.

Parmetros de entrada:

  1. nome do arquivo imagem correspondente  varivel x (independente).
  1. nome do arquivo imagem correspondente  varivel y (dependente).

Sada:

  Desenha o diagrama de espalhamento e escreve a equao de regresso.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.Image;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.MediaTracker;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;

// pacote Swing para interface grfica
import javax.swing.JFrame;

public class RegressPG extends JFrame {

   // Define objeto BufferedImage para encapsular a imagem
   Image imagem_X, imagem_Y;
	
   int p_X, y_X, w_X = 0, h_X = 0,
       p_Y, y_Y, w_Y = 0, h_Y = 0,
       iMaxBanda1, iMaxBanda2, Obs, iLin, iCol;
   int pixels1[] = new int[1], pixels2[] = new int[1];
   PixelGrabber pg1 = null, pg2 = null;
   double a, b, r, Rsqr;

   public static void main(String args[]) {

      // Certifica a passagem da imagem
      if (args.length != 2) {
         System.out.println("Uso: java -cp . RegressPG <imagem_X> <imagem_Y>");
         System.exit(0);
      }

      // Pega tempo inicial
      double e_time = System.currentTimeMillis();
     
      // Chama o construtor passando as imagens
      JFrame f = new RegressPG(args[0], args[1]);
      f.setSize(300, 280);
      f.setVisible(true);

      // Encerra a aplicao clicando no "close"
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      // Pega tempo final
      e_time = System.currentTimeMillis() - e_time;
      e_time = e_time*0.001;
      String padrao = "##.###";
      DecimalFormat formato = new DecimalFormat(padrao);
      String tempo = formato.format(e_time); 
      System.out.println("Tempo de execucao: " + tempo + " milisseg. ");
   }
  
   public RegressPG(String aFile1, String aFile2) {
      double wt, t, SxoSS, SumX, SumY, Sigma2, Sigma, Ep, P, SumTsqr,
             SumSigma, SigmaDat, SigmaA, SigmaB, q, Chi2, MeanX,
             MeanY, Xt, Yt, SumXsqr, SumYsqr, SumXY, Tmp,
             x1, y1, x2, y2;
		
      // Certifica que a imagem 1 existe
      File file1 = new File(aFile1);
      if (!file1.exists()) {
         System.out.println("Arquivo '" + aFile1 + "' nao existe.");
         System.exit(0);
      }

      // Certifica que a imagem 2 existe
      File file2 = new File(aFile2);
      if (!file2.exists()) {
         System.out.println("Arquivo '" + aFile2 + "' nao existe.");
         System.exit(0);
      }

      // Associa objeto Image com arquivos de imagem
      imagem_X = Toolkit.getDefaultToolkit().getImage(aFile1);
      imagem_Y = Toolkit.getDefaultToolkit().getImage(aFile2);

      // Define objeto MediaTracker para imagem 1
      MediaTracker mt1 = new MediaTracker(this);
      mt1.addImage(imagem_X, 0);
      try {
         mt1.waitForID(0);
         w_X = imagem_X.getWidth(null);
         h_X = imagem_X.getHeight(null);
         pixels1 = new int[w_X*h_X];
         pg1 = new PixelGrabber(imagem_X, 0, 0, w_X, h_X, pixels1, 0, w_X);
         pg1.grabPixels();
      } catch (InterruptedException e) {
         System.out.println("Problema lendo arquivo `" + aFile1 + "'");
         System.exit(0);
      }

      // Define objeto MediaTracker para imagem 2
      MediaTracker mt2 = new MediaTracker(this);
      mt2.addImage(imagem_Y, 0);
      try {
         mt2.waitForID(0);
         w_Y = imagem_Y.getWidth(null);
         h_Y = imagem_Y.getHeight(null);
         pixels2 = new int[w_Y*h_Y];
         pg2 = new PixelGrabber(imagem_Y, 0, 0, w_Y, h_Y, pixels2, 0, w_Y);
         pg2.grabPixels();
      } catch (InterruptedException e) {
         System.out.println("Problema lendo arquivo `" + aFile2 + "'");
         System.exit(0);
      }

      if ((w_X != w_Y) || (h_X != h_Y)) {
         System.out.println("Imagens com dimensoes diferentes.");
         System.exit(0);
      }
		
      // Atribui nome ao frame
      setTitle("Regresso: " + file1.getName() + ", " + file2.getName());

      iMaxBanda1 = 0;
      iMaxBanda2 = 0;
      b = 0.0;
      SumX = 0.0;
      SumY = 0.0;
      SumTsqr = 0.0;
      Sigma2 = 0.0;
      Ep = 0.0;
      Obs = w_X*h_X;

      ColorModel cmodel1 = pg1.getColorModel();
      ColorModel cmodel2 = pg2.getColorModel();

      for (int i = 0; i < pixels2.length; i++) {
         p_Y = cmodel2.getRed(pixels2[i]);
         Ep = Ep + p_Y;
         P = p_Y*p_Y;
         Sigma2 = Sigma2 + P;
      }

      Sigma2 = (double)(Sigma2 - Ep*Ep/Obs)/(Obs - 1);
      Sigma  = Math.sqrt(Sigma2);
      for (int i = 0; i < pixels1.length; i++) {
         // Pixel da varivel independente
         p_X = cmodel1.getRed(pixels1[i]);
         SumX = SumX + p_X;

         // Pixel da varivel dependente
         p_Y = cmodel2.getRed(pixels2[i]);
         SumY = SumY + p_Y;
         iMaxBanda1 = Math.max(p_X, iMaxBanda1);
         iMaxBanda2 = Math.max(p_Y, iMaxBanda2);
      }
      SumSigma = (double)Obs;
      SxoSS = SumX/SumSigma;
      for (int i = 0; i < pixels2.length; i++) {
         p_Y = cmodel2.getRed(pixels2[i]);
         p_X = cmodel1.getRed(pixels1[i]);
         t = p_X - SxoSS;
         SumTsqr = SumTsqr + t*t;
         b = b + t*p_Y;
      }
      b = b/SumTsqr;
      a = (SumY - SumX*b)/SumSigma;
      SigmaA = Math.sqrt((1.0 + SumX*SumX/(SumSigma*SumTsqr))/SumSigma);
      SigmaB = Math.sqrt(1.0/SumTsqr);
      Chi2 = 0.0;
      for (int i = 0; i < pixels1.length; i++) {
         p_X = cmodel1.getRed(pixels1[i]);
         p_Y = cmodel2.getRed(pixels2[i]);
         Tmp = p_Y - a - b*p_X;
         Chi2 = Chi2 + Tmp*Tmp;
      }
      q = 1.0;
      SigmaDat = (double)Math.sqrt(Chi2/(Obs - 2));
      SigmaA = SigmaA*SigmaDat;
      SigmaB = SigmaB*SigmaDat;
      
      // Coeficiente de correlacao linear - r
      MeanX = 0.0;
      MeanY = 0.0;
      SumXsqr = 0.0;
      SumYsqr = 0.0;
      SumXY = 0.0;
      for (int i = 0; i < pixels1.length; i++) {
         p_X = cmodel1.getRed(pixels1[i]);
         p_Y = cmodel2.getRed(pixels2[i]);
         MeanX = MeanX + (double)p_X;
         MeanY = MeanY + (double)p_Y;
      }
      MeanX = (double)MeanX/Obs;
      MeanY = (double)MeanY/Obs;
      for (int i = 0; i < pixels1.length; i++) {
         p_X = cmodel1.getRed(pixels1[i]);
         p_Y = cmodel2.getRed(pixels2[i]);
         Xt = (double)p_X - MeanX;
         Yt = (double)p_Y - MeanY;
         SumXsqr = SumXsqr + Xt*Xt;
         SumYsqr = SumYsqr + Yt*Yt;
         SumXY = SumXY + Xt*Yt;
      }
      r = SumXY/Math.sqrt(SumXsqr*SumYsqr);
      Rsqr = (SumXY*SumXY)/(SumXsqr*SumYsqr);
      String padrao = "##.###";
      DecimalFormat formato = new DecimalFormat(padrao);
      String regressA = formato.format(a); 
      String regressB = formato.format(b);
      String regressr = formato.format(r);
      double R100 = Rsqr*100;
      String regressR = formato.format(R100);
      String out1 = "\n\ty = " + regressA + " + " + regressB + "x";
      
      String out2 = "\n\tr = " + regressr + ", R2 = " + regressR + "%";
      System.out.println("\nEquacao de regressao:"); 
      System.out.println(out1); 
      System.out.println("\nCoef. de correlacao (r) e de determinacao (R**2):");
      System.out.println(out2 + "\n"); 
   }

   public void paint(Graphics g) {
      Graphics2D g2 = (Graphics2D) g;
      int x, y, winW, winH, taxaX, taxaY, iLin, iCol;
      double x1, y1, x2, y2;

      // Traa o grfico de espalhamento entre as duas imagens.
      // Para reduzir o nmero de iteraes, os pixels so amostrado em 1/5.
		
      winW = getSize().width;
      winH = getSize().height;
      ColorModel cmodel1 = pg1.getColorModel();
      ColorModel cmodel2 = pg2.getColorModel();

      taxaX = (int)winW/iMaxBanda1;
      taxaY = (int)winH/iMaxBanda2;
      iLin = 0;
      iCol = 0;
      while (iLin < h_X) {
         while (iCol < w_X) {
            p_X = cmodel1.getRed(pixels1[iLin*w_X+iCol]);
            p_Y = cmodel2.getRed(pixels2[iLin*w_X+iCol]);
            y = winH - p_Y*taxaY;
            x = p_X*taxaX;
            drawPoint(x, y, g2);
            iCol += 5;
         }
         iLin += 5;
         iCol = 0;
      }
		
      // Desenha reta ajustada
      x1 = 0.0;
      y1 = a + b*x1;
      x2 = iMaxBanda1;
      y2 = a + b*x2;
      g2.setPaint(Color.yellow);
      g2.setStroke(new BasicStroke(3));
      g2.drawLine((int)x1*taxaX, (int)(winH - y1*taxaY), (int)x2*taxaX, 
                  (int)(winH - y2*taxaY));
		
      // Prepara variveis para escrever equao de regresso
      String padrao = "##.###";
      DecimalFormat formato = new DecimalFormat(padrao);
      String regressA = formato.format(a); 
      String regressB = formato.format(b);
      String regressr = formato.format(r);
      double R100 = Rsqr*100;
      String regressR = formato.format(R100);
      String out1 = "y = " + regressA + " + " + regressB + "x";
      String out2 = "r = " + regressr + ", R2 = " + regressR + "%";
      g2.setPaint(Color.black);
      g2.drawString(out1, (int)winW/3, getInsets().top + 10); 
      g2.drawString(out2, (int)winW/3, getInsets().top + 25); 
      return;
   }
	
   public void drawPoint(int x, int y, Graphics2D gContxt) {
      int r = 2;
		
      Rectangle point = new Rectangle(x-r, y-r, 2*r, 2*r);
      gContxt.setPaint(Color.red);
      gContxt.fill(point);
      gContxt.setPaint(Color.black);
      gContxt.draw(point);
   }

}
