/***********************************************************************************
ResolucaoEspacial.java - Simula baixas resolues para uma imagem.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Simula baixas resolues para uma imagem.
    Observaes: O programa pega a menor dimenso, entre largura e altura, e 
                 trabalha com uma imagem quadrada na menor dimenso. Exemplo,
                 imagem 370X270 => 270X270.
                 Alm disso, ele pega a regio da imagem com a maior potncia de 2.
                 Neste caso, 256x256.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. resoluo desejada para a imagem.

Sada:

  Painel com imagem na resoluo fornecida.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;

// pacote Swing para interface grfica
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ResolucaoEspacial extends JFrame {

   public static void main(String args[]) {

      // Certifica a passagem da imagem e da resoluo como parmetro
      if (args.length != 2) {
         String msg = "Uso: java -cp . ResolucaoEspacial <arquivo_imagem> <res>";
         System.out.println(msg);
         System.exit(-1);
      }

      // L parmetro de resoluo e certifica por valor vlido
      int res = 0;
      try {
         res = Integer.parseInt(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Valor de 'resolucao' invalido: " + args[1]);
         System.exit(0);
      }

      if ((res != 8) && (res != 16) && (res != 32) && (res != 64) && (res != 128)) {
         String msg = "Valor invalido. Deve ser potencia de 2 entre 8 e 128";
         System.out.println(msg);
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor passando o nome do arquivo imagem
      JFrame sr = new ResolucaoEspacial(args[0], res);

      // Encerra a aplicao clicando no "close"
      sr.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      sr.setVisible(true);
   }

   public ResolucaoEspacial(String aFile, int resolucao) {

      // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage src = null, copia = null, dest = null;
      int w, h, ladoMenor, dimPot2;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         src = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Simula resoluo espacial: " + file.getName());

      w = src.getWidth();
      h = src.getHeight();

      // Pega menor valor entre w e h
      ladoMenor = Math.min(w, h);

      // Qual potncia de 2?
      dimPot2 = potencia2(ladoMenor);

      // Primeiro, copia regio da imagem original em imagem
      // com dimenso de potncia de 2
      copia = copiaImagem(dimPot2, src);

      // Segundo, simula resoluo
      dest = mediaBloco(resolucao, copia);

      // Define GUI com objetos do Swing
      JLabel view = new JLabel(new ImageIcon(dest));
      getContentPane().add(new JScrollPane(view));

      // Define tamanho do Frame
      setSize(w, h);
   }

	 public int potencia2(int x) {

      if (x < 16) {
         return (8);
      } else {
         if (x < 32) {
           return (16);
         } else {
            if (x < 64) {
              return (32);
           } else {
              if (x < 128) {
                return (64);
              } else {
                 if (x < 256)
                   return(128);
                 else
                   return(256);
              } // 64
           } // 32
         } // 16
      } // 8
   }

   public BufferedImage copiaImagem(int vertice, BufferedImage src) {
      int pixel;
      
      BufferedImage destImage =
           new BufferedImage(vertice, vertice, BufferedImage.TYPE_BYTE_GRAY);

      Raster srcR = src.getRaster();
      WritableRaster destWR = destImage.getRaster();

      // Copia a regio quadrada, vrtice x vrtice, 
      // de srcR em destWR 
      for (int y = 0; y < vertice; y++)
         for (int x = 0; x < vertice; x++) {
            pixel = srcR.getSample(x, y, 0);
            destWR.setSample(x, y, 0, pixel);
         }
      return destImage;
   }     

   public BufferedImage mediaBloco(int res, BufferedImage src) {
      float sum;
      int average, blockArea, width, height, resX, resY;
      
      width = src.getWidth();
      height = src.getHeight();
      resX = width/res;
      resY = resX;
      BufferedImage destImage =
           new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);

      Raster srcR = src.getRaster();
      WritableRaster destWR = destImage.getRaster();

      // Faz a simulao de resoluo em blocos
      blockArea = resX*resY;
      for (int y = 0; y < width; y += resX)
         for (int x = 0; x < height; x += resY) {
            sum = 0.0f;
            for (int j = 0; j < resX; j++)
               for (int i = 0; i < resY; i++)
                  sum += srcR.getSample(x+i, y+j, 0);
            average = Math.round(sum/blockArea);
            for (int j = 0; j < resX; j++)
               for (int i = 0; i < resY; i++)
                  destWR.setSample(x+i, y+j, 0, average);
         }
      return destImage;
   }     

}

