/***********************************************************************************
RotacaoIndutiva.java - Este programa implementa o algoritmo RotacaoIndutiva

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Este programa implementa o algoritmo RotacaoIndutiva usando um algoritmo de 
    interpolao linear de ordem-zero ou vizinho mais prximo.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ngulo de rotao.

Sada:

  Imagem com a rotao.

Adaptado de: Efford, N. Digital image processing: a practical introduction
    using Java, 1999.

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class RotacaoIndutiva extends JFrame {

  public static void main(String args[]) {
    double theta = 0.0;

    // Certifica a passagem das imagens e do ngulo de rotao
    if (args.length != 2) {
      String msga = "Uso: java -cp . RotacaoIndutiva <imagem> <angulo>";
      System.out.println(msga);
      System.exit(0);
    }

    // Tempo atual
    long e_time = System.currentTimeMillis();

    // L parmetro e certifica por valor vlido
    try {
      theta = Double.parseDouble(args[1]);
    } catch ( NumberFormatException e ) {
      System.out.println("Valor de angulo invalido: " + args[1]);
      System.exit(0);
    }     

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando parmetros
    JFrame ri = new RotacaoIndutiva(args[0], theta);

    // Encerra a aplicao clicando no "close"
    ri.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    ri.setVisible(true);

    e_time = System.currentTimeMillis() - e_time;
    DecimalFormat df = new DecimalFormat("0.0000");
    System.out.println("Tempo de execucao: " + df.format(e_time) + " milisseg.");
  }

  public RotacaoIndutiva(String aFile, double angulo) {
    // Define um objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 	  JLabel destL;
		int w, h;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
      src = ImageIO.read(file);
    } catch(Exception e) {
      System.out.println("Imagem '" + aFile + "' nao existe.");
      System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Rotao indutiva: " + file.getName());

    // Realiza operao de rotao na imagem
    dest = rotacao(src, angulo);

  	w = dest.getWidth();
		h = dest.getHeight();

		destL = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(destL));
		setSize(w, h);
  }

  public BufferedImage rotacao(BufferedImage src, double theta) {

    // Para imagens com nveis de cinza
    int w, h, tipo;

    w = src.getWidth();
    h = src.getHeight();
    tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage out = new BufferedImage(w, h, tipo);

    Raster srcR = src.getRaster();
    WritableRaster outWR = out.getRaster();

    double a0 = Math.cos(theta*Math.PI/180.0);
    double b0 = Math.sin(theta*Math.PI/180.0);
    double a1 = -b0,
    b1 =  a0;

    int rx, ry;
    for (int y = 0; y < h; y++)
      for (int x = 0; x < w; x++) {
        rx = (int) Math.round(a0*x + a1*y);
        ry = (int) Math.round(b0*x + b1*y);
        if (rx >= 0 && rx < w && ry >= 0 && ry < h) {
          try {
            outWR.setSample(rx, ry, 0, srcR.getSample(x, y, 0));
          } catch (ArrayIndexOutOfBoundsException e) {
            String str = "Problema de indice na imagem de saida : ";
            System.out.println(str + y + ", " + x);
            System.exit(0);
          }
        }
      }
    return out;
  }     

}
