/***********************************************************************************
RuidoSaltPepper.java - Simula rudo sal-e-pimenta

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Simula rudo sal-e-pimenta.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. porcentagem de rudo.
     Ateno! Valor aconselhvel para a porcentagem: .01 <= percent <= .05

Sada:

  Painel com imagem original e com rudo.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class RuidoSaltPepper extends JFrame {

  public static void main(String[] args) {

		double percent = 0.0;

		// Certifica a passagem da imagem e porcentagem
    if(args.length != 2) {
			String msga = "Uso: java -cp . RuidoSaltPepper";
			String msgb = " <arquivo_imagem> <%: [0.0, 1.0]>";
			System.err.println(msga + msgb);
			System.exit(0);
		}

		// Certifa validade dos parmetros
    try {
				percent = Double.parseDouble(args[1]);
    } catch (NumberFormatException e) {
        System.out.println("Valor do parametro 'porcentagem' invalido");
        System.exit(0);
    }

		if ((percent < 0.0 ) || (percent > 1.0)) {
				System.out.println("Porcentagem deve estar em [0.0, 1.0]");
        System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
    JFrame ruido = new RuidoSaltPepper(args[0], percent);

    // Encerra a aplicao clicando no "close"
    ruido.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    ruido.setVisible(true);
	}

  public RuidoSaltPepper(String aFile, double percent) {

    // Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;

		// Certifica que a imagem existe
		File file = new File(aFile);
		try {
         src = ImageIO.read(file);
		} catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
		}
     
    // Atribui nome ao frame
    setTitle("Rudo 'salt&pepper': " + file.getName());

    dest = saltpepperOp(src, percent);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}


  public BufferedImage saltpepperOp(BufferedImage src, double percent) {

		int length, n, xmin, xmax, ymin, ymax, h, w, tipo, nc;
		double rx, ry;

		h = src.getHeight();
		w = src.getWidth();
		n = (int)(percent*w*h);
		xmin = 0;
		xmax = w - 1;
		ymin = 0;
		ymax = h - 1;

		tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage outImagem = new BufferedImage(w, h, tipo);
    Raster srcR = src.getRaster();
    WritableRaster outWR = outImagem.getRaster();

		// Copia a imagem original
		for (int Y = 0; Y < h; Y++)
			for (int X = 0; X < w; X++) {
				nc = srcR.getSample(X, Y, 0);
        outWR.setSample(X, Y, 0, nc);
			}

		// Simula rudo
		for (int i = 0; i < n/2; i++) {
			rx = Math.random()*(xmax);
			ry = Math.random()*(ymax);
			outWR.setSample((int)rx, (int)ry, 0, (byte)255);
			rx = Math.random()*(xmax);
			ry = Math.random()*(ymax);
      outWR.setSample((int)rx, (int)ry, 0, (byte)0);
		}

		// Salva arquivo formato PNG
		/* Se voc quiser gravar o arquivo, modificar o diretrio
		try {
      System.out.println("Salvando arquivo 'ruidoS&P.png' em 'C:\\temp\\'");
			ImageIO.write(outImagem, "png", new File("C:\\temp\\ruidoS&P.png"));
		} catch (IOException e) {
			System.out.println("Problema gravando arquivo ");
      System.exit(0);
		}
    */	
		return outImagem;
  }

}
