/***********************************************************************************
RuidoUniforme.java - Simula rudo uniforme

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Simula rudo uniforme.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. nvel de cinza inferiot (nc1) e superior (nc2).
     Como exemplo, usar nc1 = 10 e nc2 = 40.

Sada:

  Painel com imagem original e com rudo.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class RuidoUniforme extends JFrame {

	public static void main(String[] args) {

		int nc1 = 0, nc2 = 0;

		// Certifica a passagem da imagem e parmetros
		if (args.length != 3) {
		   String msg = "Uso: java -cp . RuidoUniforme <arquivo_imagem> <nc1> <nc2>";
		   System.err.println(msg);
		   System.exit(0);
		}

		// Certifa validade dos parmetros
		try {
			nc1 = Integer.parseInt(args[1]);
		} catch (NumberFormatException e) {
			System.out.println("Valor do parametro 'nc1' invalido");
			System.exit(0);
		}

		try {
			nc2 = Integer.parseInt(args[2]);
		} catch (NumberFormatException e) {
			System.out.println("Valor do parametro 'nc2' invalido");
			System.exit(0);
		}

	  if ((nc1 < 0) || (nc2 < 0)) {
			System.out.println("Valores de 'nc1' e 'nc2' devem ser positivos");
			System.exit(0);
	  }

	  if ((nc1 == nc2)) {
			System.out.println("Valores de 'nc1' e 'nc2' devem ser diferentes");
			System.exit(0);
	  }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor passando os parmetros
		JFrame ruido = new RuidoUniforme(args[0], nc1, nc2);

		// Encerra a aplicao clicando no "close"
		ruido.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		ruido.setVisible(true);
  }

	public RuidoUniforme(String aFile, int nc1, int nc2) {

		// Define objeto BufferedImage para encapsular a imagem
		BufferedImage src = null, dest = null;
 		JLabel img1, img2;
		int w, h;

		// Certifica que a imagem existe
		File file = new File(aFile);
		try {
			src = ImageIO.read(file);
		} catch(Exception e) {
			System.out.println("Imagem '" + aFile + "' nao existe.");
			System.exit(0);
		}
     
		// Atribui nome ao frame
		setTitle("Rudo uniforme: " + file.getName());

		dest = uniformeOp(src, nc1, nc2);

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

  public BufferedImage uniformeOp(BufferedImage src, int nc1, int nc2) {

		double range, randonBright;
		int w, h, nc, tipo, v;

		h = src.getHeight();
		w = src.getWidth();

		tipo = BufferedImage.TYPE_BYTE_GRAY;
		BufferedImage outImagem = new BufferedImage(w, h, tipo);
		WritableRaster outWR = outImagem.getRaster();
		Raster srcR = src.getRaster();
		Raster outR = outImagem.getRaster();

		if (nc1 > nc2) {
			int tmp = nc2;
			nc2 = nc1;
			nc1 = tmp;
		}
		range = nc2 - nc1;

		// Copia a imagem original
		for (int Y = 0; Y < h; Y++)
			for (int X = 0; X < w; X++) {
				nc = srcR.getSample(X, Y, 0);
				outWR.setSample(X, Y, 0, nc);
			}

		// Simula rudo
		for (int Y = 0; Y < h; Y++)
			for (int X = 0; X < w; X++) {
				randonBright = Math.random()*range;
				nc = outR.getSample(X, Y, 0) ;
				v = (nc & 0xff) + (int)randonBright;
				if ((v < 0))
					v = 0;
				if ((v > 255))
					v = 255;
				outWR.setSample(X, Y, 0, v);
			}

		// Salva arquivo formato PNG
		/* Se voc quiser gravar o arquivo, modificar o diretrio
		try {
      System.out.println("Salvando imagem 'ruidoUniforme.png' em 'C:\\temp\\'");
			ImageIO.write(outImagem, "png", new File("C:\\temp\\ruidoUniforme.png"));
		} catch (IOException e) {
			System.out.println("Problema gravando arquivo ");
			System.exit(0);
		}
    */	
		return outImagem;
	}

}
