/***********************************************************************************
ShapeMean.java - Calcula a mdia de uma regio de interesse (ROI) genrica

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    ShapeMean: calcula a mdia de uma regio de interesse (ROI) genrica, tipo
               poligonal; as coordenadas so lidas com o mouse.
               1. Um click do boto esquerdo do mouse define as coordenadas (x, y)
                  dos vrtices do polgono.
               2. Um click duplo do mesmo boto fecha o polgono, finalizando o 
                  processo de entrada dos vrtices.
               3. Vrias ROIs podem ser desenhadas na imagem.
               4. Para ver os valores das coordenadas dos vrtices do polgono,
                   necessrio aumentar a janela que mostra a imagem escolhida,
                  pois o programa imprime esses valores abaixo da imagem.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.
  2. coordenadas da ROI poligonal, fornecidas com o mouse.

Sada:

  Painel com a imagem original e valor da mdia da ROI na janela DOS.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.util.Vector;
import java.lang.Integer;
import java.io.IOException;
import java.util.Enumeration;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.Shape;
import java.awt.Point;
import java.awt.Color;
import java.awt.event.*;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.BorderLayout;
import java.awt.image.Raster;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ShapeMean extends JFrame {

	// Vector - armazena os vrtices do polgono lidos com o mouse
	Vector pontos = new Vector();

	// Label para imprimir as coordenadas dos vrtices
	JLabel labelCoordenadas = new JLabel();
	String coordenadas = "";

	// roi = identificador do nmero da regio. Usado para
	//			 identificar o arquivo de sada.
	int roi = 1;
  BufferedImage imagem = null, copiaImagem = null;

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		// Chama o construtor
    JFrame rm = new ShapeMean(file);

    // Encerra a aplicao clicando no "close"
    rm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    rm.setVisible(true);

		// Executa 'garbage collector' para liberar recursos usados.
		System.gc();
  }

  public ShapeMean(final File file) {

    // Armazena arquivo imagem numa BufferedImage
    try {
       imagem = ImageIO.read(file);
    } catch(Exception e) {
       System.out.println("Imagem '" + file + "' nao existe.");
       System.exit(0);
    }

		// Copia imagem fonte
    WritableRaster destWR = imagem.copyData(null); 
		copiaImagem = new BufferedImage(imagem.getColorModel(), 
																		destWR, false, null);

		// Atribui nome ao frame
    setTitle("Mdia da ROI: " + file.getName());

    // Define GUI com objetos do Swing
		JLabel labelImagem = new JLabel(new ImageIcon(copiaImagem));
    getContentPane().add(labelImagem, BorderLayout.NORTH); 
    setSize(imagem.getWidth(), imagem.getHeight());

		// Adiciona evento do mouse: mostra coordenadas do ponto clicado
    getContentPane().add(labelCoordenadas, BorderLayout.SOUTH);

		MouseListener mlst = new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				if (evt.getClickCount() == 1) {
					coordenadas = evt.getX() + ", " + evt.getY();
					labelCoordenadas.setText(coordenadas);
					pontos.addElement(new Point(evt.getX(), evt.getY()));
					desenhaROI();
				}
				if (evt.getClickCount() == 2) {
					fechaROI();
					meanValue(file);
					// Libera rea do Vector pontos
					pontos.clear();
					// Incrementa o identificador da ROI
					roi++;
				}
			}	
		};
		labelImagem.addMouseListener(mlst);
  }

	// MTODOS PRIVADOS
	private void desenhaROI() {
		Point pt;
		Ellipse2D elipse;
		GeneralPath path = new GeneralPath();

		// Define contexto grfico na imagem cpia (copiaImagem)
		// para desenhar os vrtices e segmentos de linhas.
		Graphics2D g2 = copiaImagem.createGraphics();
		g2.setColor(Color.red);

		// Acessa primeiro vrtice
		pt = (Point) pontos.get(0);	

		// Move para o primeiro vrtice
		path.moveTo((float) pt.x,(float) pt.y);
		for (int i = 0; i < pontos.size(); i++){
			pt = (Point) pontos.get(i);	
			// Define crculo de raio 3 nos vrtices
			elipse = new Ellipse2D.Float(pt.x, pt.y, 3, 3);
			// Desenha os crculos preenchidos
			g2.fill(elipse);
			// Desenha segmento de reta entre dois vrtices
			path.lineTo((float) pt.x,(float) pt.y);
		}
		// Desenha o polgono
		g2.draw(path);
		repaint();
	}

	private void fechaROI() {
		Point pt;
		GeneralPath path = new GeneralPath();
		Ellipse2D elipse;

		// Define contexto grfico na imagem cpia (copiaImagem)
		// para desenhar os vrtices e segmentos de linhas.
		Graphics2D g2 = copiaImagem.createGraphics();
		
		g2.setColor(Color.red);

		// Acessa primeiro vrtice
		pt = (Point) pontos.get(0);	

		// Move para o primeiro vrtice
		path.moveTo((float) pt.x,(float) pt.y);
		pt = (Point) pontos.get(pontos.size()-1);	
		// Define crculo de raio 3 nos vrtices
		elipse = new Ellipse2D.Float(pt.x, pt.y, 3, 3);
		// Desenha os crculos preenchidos
		g2.fill(elipse);
		// Desenha segmento de reta entre dois vrtices
		path.lineTo((float) pt.x,(float) pt.y);
		// Desenha o polgono
		g2.draw(path);
		repaint();
	}

	private void meanValue(File file) {

		double sum = 0.0, media = 0.0;
		int conta = 0;
		int xmin, xmax,	ymin, ymax, tipo, pixel;

		// Prepara imagem para leitura de pixels
		Raster raster = imagem.getRaster();

		// Define figura geomtrica genrica ou polgono
		GeneralPath gp = new GeneralPath(GeneralPath.WIND_EVEN_ODD);

		// Desenha polgono - cria contexto grfico
		Graphics2D g2 = copiaImagem.createGraphics();

		// 1. move para ponto inicial
		Point p = new Point();
		p = (Point)pontos.get(0);
		gp.moveTo((float)p.x, (float)p.y);

		// 2. desenha os segmentos de reta
		for (int i = 1; i < pontos.size(); i++) {
			p = (Point)pontos.get(i);
			gp.lineTo((float)p.x, (float)p.y);
		}

		// 3. fecha o polgono
		gp.closePath();

		// Acha mximos e mnimos para definir retngulo envolvente
		// do polgono
		p = (Point)pontos.get(0);
		xmin = p.x; xmax = p.x;
		ymin = p.y; ymax = p.y;

		for (int i = 1; i < pontos.size(); i++) {
			p = (Point)pontos.get(i);
			xmin = (int)Math.min(xmin, p.x);
			ymin = (int)Math.min(ymin, p.y);
			xmax = (int)Math.max(xmax, p.x);
			ymax = (int)Math.max(ymax, p.y);
		}

		// Prepara imagem para gravar polgono
		tipo = BufferedImage.TYPE_BYTE_GRAY;
    BufferedImage dest = new BufferedImage((xmax-xmin), (ymax-ymin), tipo);
    WritableRaster destWR = dest.getRaster();

		// Percorre todo o retngulo envolvente do polgono;
		// se o ponto estiver dentro do polgono, l valor do
		// pixel e acumula soma
		for (int i = ymin; i < ymax; i++)
			for (int j = xmin; j < xmax; j++) {
				if (gp.contains((double)j, (double)i)) {
					pixel = raster.getSample(j, i, 0);
					sum += pixel;
					conta++;
          destWR.setSample(j-xmin, i-ymin, 0, pixel);
				}
			}
		media = sum/conta;
    DecimalFormat df = new DecimalFormat("0.000");
    System.out.println("Media ROI = " + df.format(media));
		
		// Salva a imagem no formato "png", que preserva os 
		// valores exatos acima. O formato "jpg" acrescenta 
		// efeitos esprios s cores (valores) acima definidas. 
		/* Se voc quiser gravar o arquivo, modificar o diretrio
		String saida = null;
		try	{
			// Recupera o nome da imagem, sem extenso
			String nomeImg = file.getName();
			String search = ".";
			int i = nomeImg.indexOf(search);
			saida = nomeImg.substring(0, i);
			saida += "_ROI_" + String.valueOf(roi) + ".png";
			System.out.println("Criando imagem '" + saida + "' em 'C:\\temp\\'");
			ImageIO.write(dest, "png", new File("C:\\temp\\" + saida));		
		} catch (IOException e) {
				System.out.println("Problema gravando arquivo: " + saida);
        System.exit(0);
		}
    */
	}

}
