/***********************************************************************************
SwingDisplay.java - Gerando uma GUI com Swing

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Gerando uma GUI com Swing.
    Para ver os valores das coordenadas dos vrtices do polgono,  necessrio 
    aumentar a janela que mostra a imagem escolhida, pois o programa imprime esses 
    valores abaixo da imagem.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com a imagem escolhida e valores das coordenadas do ponto clicado com o
  mouse, abaixo da imagem.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.BorderLayout;
import java.awt.event.*;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class SwingDisplay extends JFrame {

	JLabel labelCoordenadas = new JLabel();
	String coordenadas = "";

	// Define um objeto BufferedImage para encapsular a imagem
  private BufferedImage imagem;

  public static void main(String args[]) throws Exception {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    JFrame jf = new SwingDisplay(file);

    // Encerra a aplicao clicando no "close"
    jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    jf.setVisible(true);
  }

  public SwingDisplay(File file) {

    // Armazena arquivo imagem numa BufferedImage
    try {
       imagem = ImageIO.read(file);
    } catch(Exception e) {
       System.out.println("Imagem '" + file + "' nao existe.");
       System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("SwingDisplay: " + file.getName());

    // Define GUI com objetos do Swing
		JLabel labelImagem = new JLabel(new ImageIcon(imagem));
    add(labelImagem, BorderLayout.NORTH);

		// Adiciona evento do mouse: imprime coordenadas do ponto clicado
    add(labelCoordenadas, BorderLayout.SOUTH);
		MouseListener mlst = new MouseAdapter() {
			
			public void mouseClicked(MouseEvent evt) {
				if (evt.getClickCount() == 1)
					coordenadas = evt.getX() + ", " + evt.getY();
					labelCoordenadas.setText(coordenadas);
			}
		};
		labelImagem.addMouseListener(mlst);
    setSize(imagem.getWidth(), imagem.getHeight());
   }
}

