/***********************************************************************************
TAfim1.java - TAfim1: uso da classe AffineTransform - exemplo 1

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    TAfim1 - uso da classe AffineTransform. Exemplo 1:
             gira uma imagem por um ngulo theta graus em torno do seu eixo.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ngulo de rotao.

Sada:

  Imagem com rotao.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.Graphics2D;
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class TAfim1 extends JFrame {

   public static void main(String args[]) {

      double theta = 0.0;

      // Certifica a passagem da imagem e ngulo
      if (args.length != 2) {
         String msga = "Uso: java -cp . TAfim1 <arquivo_imagem> <angulo>";
         System.out.println(msga);
         System.exit(0);
      }

      // L ngulo
      try {
         theta = Double.parseDouble(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Valor de 'angulo' invalido: " + args[1]);
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor
      JFrame ta1 = new TAfim1(args[0], theta);

      // Encerra a aplicao clicando no "close"
      ta1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      ta1.setVisible(true);
   }

   public TAfim1(String aFile, double angulo) {

      // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage src = null, dest = null;
 			JLabel destL;
			int w, h;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         src = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Rotao: " + file.getName());

      // ngulos so especificados em radianos
      angulo = Math.toRadians(angulo); 

      // define ponto ncora
      double x = src.getWidth()/2.0;
      double y = src.getHeight()/2.0;
      AffineTransform atGira = AffineTransform.getRotateInstance(angulo, x, y);

      // Realiza transformao
      dest = tAfim(src, atGira);

			w = dest.getWidth();
			h = dest.getHeight();

			destL = new JLabel(new ImageIcon(dest));
			getContentPane().add(new JScrollPane(destL));
			setSize(w, h);
   }

   protected BufferedImage tAfim(BufferedImage bi, AffineTransform af) {

      int w = bi.getWidth();
      int h = bi.getHeight();
      int tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage imagemTx = new BufferedImage(w, h, tipo);
      Graphics2D cg = imagemTx.createGraphics();
      cg.drawImage(bi, af, this);
      return imagemTx;
   }

}
