/***********************************************************************************
TAfim2.java - TAfim2: uso da classe AffineTransformOp - exemplo 2

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    TAfim2 - uso da classe AffineTransformOp. Exemplo 2:
             gira uma imagem por um ngulo theta graus em torno do seu eixo.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ngulo de rotao.

Sada:

  Imagem com rotao.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class TAfim2 extends JFrame {

   public static void main(String args[]) {

      double theta = 0.0;
      int tipo_interp = 0;

      // Certifica a passagem da imagem e demais parmetros
      if (args.length != 3) {
         String msga = "Uso: java -cp . TAfim2 <arquivo_imagem> <angulo> <interp>";
         System.out.println(msga);
         System.out.println("\t<interp>:");
         System.out.println("\t1 = vizinho mais proximo");
         System.out.println("\t2 = bilinear");
         System.exit(0);
      }

      // L ngulo
      try {
         theta = Double.parseDouble(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Valor de 'angulo' invalido: " + args[1]);
         System.exit(0);
      }

      // L tipo de interpolador
      try {
         tipo_interp = Integer.parseInt(args[2]);
      } catch (IllegalArgumentException e) {
         System.out.println("Parametro de 'interpolador' invalido: " + args[2]);
         System.exit(0);
      }

      // Certifica se valores de interpolador so vlidos
      if ((tipo_interp != 1) && (tipo_interp != 2)) {
         System.out.println("Tipo de 'interpolador' invalido. Deve ser:");
         System.out.println("\t1 = vizinho mais proximo");
         System.out.println("\t2 = bilinear");
         System.exit(0);
      }

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor
      JFrame ta2 = new TAfim2(args[0], theta, tipo_interp);

      // Encerra a aplicao clicando no "close"
      ta2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      ta2.setVisible(true);
   }

   public TAfim2(String aFile, double angulo, int interpolador) {

      // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage bimage = null, dest = null;
      int interp = 1;
 			JLabel destL;
			int w, h;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         bimage = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Rotao: " + file.getName());

      // Realiza transformao

      // define tipo de interpolador
      switch (interpolador) {
         case 1: interp = AffineTransformOp.TYPE_NEAREST_NEIGHBOR;
         break;
         case 2: interp = AffineTransformOp.TYPE_BILINEAR;
         break;
      }

      dest = tAfim(bimage, angulo, interp);

			w = dest.getWidth();
			h = dest.getHeight();
			destL = new JLabel(new ImageIcon(dest));
			getContentPane().add(new JScrollPane(destL));
			setSize(w, h);
   }

   protected BufferedImage tAfim(BufferedImage src, double angulo, int interpol) {

      // ngulos so especificados em radianos
      angulo = Math.toRadians(angulo); 

      // define ponto ncora
      double x = src.getWidth()/2.0;
      double y = src.getHeight()/2.0;

      AffineTransform af = AffineTransform.getRotateInstance(angulo, x, y);
      AffineTransformOp afOp = new AffineTransformOp(af, interpol);

      BufferedImage dst = afOp.filter(src, null);
      return dst;
   }

}
