/***********************************************************************************
TLinear.java - Transformao linear: g(x, y) = a*f(x, y) + b

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Transformao linear: g(x, y) = a*f(x, y) + b
    Para obter o negativo da imagem f(x, y), fazer a = -1 e b = 255.
    Este programa implementa a funo de duas maneiras: 
        (1) pixel a pixel.
        (2) usando a LUT.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. ganho ('a').
  3. contraste ('b').

Sada:

  Imagem original e imagem transformada linearmente.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.ByteLookupTable;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class TLinear extends JFrame {

  public static void main(String args[]) {
    int b = 0;
    float a = 1;

    // Certifica a passagem das imagens e do operador
    if (args.length != 3) {
       String msga = "Uso: java -cp . TLinear <imagem> <ganho> <contraste>";
       System.out.println(msga);
       System.exit(0);
    }

    try {
        a = Float.parseFloat(args[1]);
    } catch ( NumberFormatException e ) {
        System.out.println("Valor de 'ganho' invalido " + args[1]);
				System.exit(0);
    }     

    try {
        b = Integer.parseInt(args[2]);
    } catch ( NumberFormatException e ) {
        System.out.println("Valor de 'contraste' invalido " + args[2]);
				System.exit(0);
    }     

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem e parmetros
	  JFrame linear = new TLinear(args[0], a, b);

    // Encerra a aplicao clicando no "close"
    linear.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    linear.setVisible(true);
  }

  public TLinear(String aFile, float ganho, int contraste) {

	  // Define objeto BufferedImage para encapsular a imagem
    BufferedImage dest = null, src = null;
 		JLabel img1, img2;
		int w, h;
      
    // Certifica que a imagem existe
    File file = new File(aFile);
    try {
          src = ImageIO.read(file);
    } catch(Exception e) {
          System.out.println("Imagem '" + aFile + "' nao existe.");
          System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Transf. linear: " + file.getName());

    // Chama funo
	  System.out.println("Calculando transformacao linear da imagem... ");
    dest = rescale(src, ganho, contraste);

    // Cria gridLayout de 1 x 2
    System.out.println("Mostrando Imagens");
    getContentPane().setLayout(new GridLayout(1, 2));

		w = dest.getWidth();
		h = dest.getHeight();

    // Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

/*
   // re-mapeia imagem pixel a pixel
   public BufferedImage rescale(BufferedImage src, float gain, float bias) {

      int w, h, tipo;

      w = src.getWidth();
      h = src.getHeight();
      tipo = BufferedImage.TYPE_BYTE_GRAY;
      BufferedImage out = new BufferedImage(w, h, tipo);
      WritableRaster outWR = out.getRaster();
      Raster srcR = src.getRaster();

      System.out.println("Sem LookupOp");
      for (int i = 0; i < h; i++)
         for (int j = 0; j < w; j++)
             outWR.setSample(j, i, 0, minmax(gain*srcR.getSample(j, i, 0) + bias));
      return out;
   }     
*/
  
   // re-mapeia imagem usando LUT
   public BufferedImage rescale(BufferedImage src, float gain, float bias) {

      BufferedImage out = null;
   	  Raster srcR = src.getRaster(); 
      WritableRaster outWR = src.copyData(null); 

      System.out.println("Com LookupOp");
      byte[] table = new byte[256];
      for (int i = 0; i < 256; i++)
          table[i] = (byte)minmax(gain*i + bias);
      ByteLookupTable rescaleTable = new ByteLookupTable(0, table);
      LookupOp rescaleOp = new LookupOp(rescaleTable, null);
      outWR = rescaleOp.filter(srcR, outWR);
			out = new BufferedImage(src.getColorModel(), outWR, false, null);

	  return out;
   }     
  
   public int minmax(float valor) {
      return Math.min(Math.max(Math.round(valor), 0), 255);
   }

}
