/***********************************************************************************
Univariada.java - Calcula as estatsticas univariadas

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    Calcula as estatsticas univariadas de uma imagem monocromtica.

Parmetros de entrada:

  1. nome do arquivo imagem, escolhido na janela interativa.

Sada:

  Painel com imagem escolhida e estatsticas univariadas na janela DOS.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.text.DecimalFormat;

// pacotes do AWT
import java.awt.image.BufferedImage;
import java.awt.image.Raster;

// pacotes do Swing
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JFileChooser;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.*;

public class Univariada extends JFrame {

	public static void main(String args[]) {

		File file = null;
		int result;

    // Comandos Swing para abrir janela com arquivos.
		JFileChooser fileChooser = new JFileChooser(".");
		fileChooser.setDialogTitle("Abre imagem");
		result = fileChooser.showOpenDialog(null);
		if (result == JFileChooser.APPROVE_OPTION) {
			file = fileChooser.getSelectedFile();
		} else {
      System.out.println("Escolher imagem.");
			System.exit(0);
		}

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

    // Chama o construtor passando o nome do arquivo imagem
    JFrame f = new Univariada(file);
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setVisible(true);
	}

  public Univariada(File file) {
		int w, h;
      
    // Define um objeto BufferedImage para encapsular a imagem
    BufferedImage bimage = null;

    // Armazena arquivo imagem numa BufferedImage
    try {
			bimage = ImageIO.read(file);
    } catch(Exception e) {
			System.out.println("Imagem '" + file + "' nao existe.");
      System.exit(0);
		}

    // Atribui nome ao frame
    setTitle("Estatisticas da Imagem: " + file.getName());

    w = bimage.getWidth();
    h = bimage.getHeight();

    // Chama funo
    double[] stats = moment(bimage);

		// Define os formatos de impresso
		String padrao = "##.###";
		DecimalFormat formato = null;
		formato = new DecimalFormat(padrao);
		String media  = formato.format(stats[0]);
		String desvio = formato.format(stats[2]);
		String desmed = formato.format(stats[1]);
		String varian = formato.format(stats[3]);
		String skewne = formato.format(stats[4]);
		String kurtos = formato.format(stats[5]);

    // Imprime valor na janela DOS
    System.out.print("\nmedia         = " + media);
    System.out.print("\ndesvio padrao = " + desvio);
    System.out.print("\ndesvio medio  = " + desmed);
    System.out.print("\nvariancia     = " + varian);
    System.out.print("\nskewness      = " + skewne);
    System.out.print("\nkurtosis      = " + kurtos);
    System.out.print("\nminimo        = " + (int)(stats[6]));
    System.out.print("\nmaximo        = " + (int)(stats[7]));
    System.out.print("\n");

    // Define tamanho do Frame
    setSize(w, h);
  
    // mostra a imagem
    JLabel lsrc = new JLabel(new ImageIcon(bimage));
    getContentPane().add(new JScrollPane(lsrc));
	}

  public double[] moment(BufferedImage src) {
      
		double[] stats = new double[8];
    /*
      stats[0] = ave
      stats[1] = adev
      stats[2] = sdev
      stats[3] = var
      stats[4] = skew
      stats[5] = curt
      stats[6] = valor mnimo
      stats[7] = valor mximo
    */
    double ep = 0.0, sum, p;
    int w, h, N, pixel;

    Raster raster = src.getRaster();

    for (int i = 0; i < 8; i++) 
			stats[i] = 0.0;
      stats[6] = 255.0;

      // Passo 1: Calcula a mdia, mnimo e mximo
      sum = 0.0;
      w = src.getWidth();
      h = src.getHeight();
      N = w*h;
      for (int i = 0; i < h; i++) 
        for (int j = 0; j < w; j++) {
					pixel = raster.getSample(j, i, 0); 
					sum += pixel;
          stats[6] = Math.min(stats[6], pixel);
          stats[7] = Math.max(stats[7], pixel);
        }
      stats[0] = (double)sum/N;

      // Passo 2: calcula segundo, terceiro e quarto momentos
      //          do desvio da mdia
      for (int i = 0; i < h; i++) 
        for (int j = 0; j < w; j++) {
					sum = raster.getSample(j, i, 0) - stats[0];
					stats[1] += Math.abs(sum);
          ep += sum;
					stats[3] += (p = sum*sum);
					stats[4] += (p *= sum);
					stats[5] += (p *= sum);
        }
      stats[1] /= N;
      stats[3] = (stats[3] - ep*ep/N)/(N-1);
      stats[2] = Math.sqrt(stats[3]);
      if (stats[3] > 0) {
         stats[4] /= (N*(stats[3])*(stats[2]));
         stats[5] = (stats[5])/(N*stats[3]*stats[3]) - 3.0;
      } else
         System.out.println("Sem skew/kurtosis. Variancia = 0.");
      return(stats);
   }     

}

