/***********************************************************************************
ZoomImagem.java - Amplia ou reduz tamanho da imagem

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

    ZoomImagem: amplia ou reduz tamanho da imagem.

Parmetros de entrada:

  1. nome do arquivo imagem.
  2. fator de mudana de escala.
  3. tipo de mudana: ampliao ('1') ou reduo ('2').

Sada:

  Painel com a imagem ampliada/reduzida.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.io.File;

// pacotes do AWT
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class ZoomImagem extends JFrame {

   public static void main(String args[]) {
      int fator = 1, tipoOp = 1;

      // Certifica a passagem da imagem e outros parmetros
      if (args.length != 3) {
         String msga = "Uso: java -cp . ZoomImagem <arquivo_imagem> <p1> <p2>";
         String msgb = "\tp1 = fator de ampliacao/reducao, p2 = tipo de operacao";
         String msgc = "\tSe p2 =  1, amplia imagem";
         String msgd = "\tSe p2 =  2, reduz imagem";
         System.out.println(msga);
         System.out.println(msgb);
         System.out.println(msgc);
         System.out.println(msgd);
         System.exit(0);
      }

      // L parmetros e certifica por valores validos
      try {
         fator = Integer.parseInt(args[1]);
      } catch (IllegalArgumentException e) {
         System.out.println("Fator de ampliacao//reducao invalido: " + args[1]);
         System.exit(0);
      }

      try {
         tipoOp = Integer.parseInt(args[2]);
      } catch (IllegalArgumentException e) {
         System.out.println("Tipo de operacao invalido: " + args[2]);
         System.exit(0);
      }

      // Assume valor padro
      if (fator <= 0)
         fator = 1;
      if ((tipoOp <= 0) || (tipoOp > 2))
         tipoOp = 1;

			// Mostra JFrame decorado pelo Swing
			JFrame.setDefaultLookAndFeelDecorated(true);

      // Chama o construtor
      JFrame zoomImg = new ZoomImagem(args[0], fator, tipoOp);

      // Encerra a aplicao clicando no "close"
      zoomImg.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      zoomImg.setVisible(true);
   }

   public ZoomImagem(String aFile, int aFator, int aTipoOp) {

      // Define um objeto BufferedImage para encapsular a imagem
      BufferedImage bimage = null, dest = null;
      
      // Certifica que a imagem existe
      File file = new File(aFile);
      try {
         bimage = ImageIO.read(file);
      } catch(Exception e) {
         System.out.println("Imagem '" + aFile + "' nao existe.");
         System.exit(0);
      }

      // Atribui nome ao frame
      setTitle("Amplia/Reduz imagem: " + file.getName());

      // Aumenta/Reduz imagem pelo fator
      dest = zoomImage(bimage, aFator, aTipoOp);

      // Define GUI com objetos do Swing
      JLabel limg = new JLabel(new ImageIcon(dest));
      getContentPane().add(new JScrollPane(limg));
      setSize(bimage.getWidth(), bimage.getHeight());
   }

   public BufferedImage zoomImage(BufferedImage image, int fator, int tipoOp) {
      BufferedImage bi = null;
      int w, h;

      switch(tipoOp) {
        // Amplia pelo fator
        case 1:
           w = fator*image.getWidth();
           h = fator*image.getHeight();
           bi = new BufferedImage(w, h, image.getType());
           for (int j = 0; j < h; j++)
              for (int i = 0; i < w; i++)
                 bi.setRGB(i, j, image.getRGB(i/fator, j/fator));
           break;
        // Reduz pelo fator
        case 2: 
           w = (int)image.getWidth()/fator;
           h = (int)image.getHeight()/fator;
           bi = new BufferedImage(w, h, image.getType());
           for (int j = 0; j < h; j++)
              for (int i = 0; i < w; i++)
                 bi.setRGB(i, j, image.getRGB(i*fator, j*fator));
           break;
      }
      return bi;
   }

}
