/*******************************************************************************
GUIMain.java - Gera uma interface grfica de usurio (Graphical User Interface).

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do software 'Processamento de Imagens Digitais'

Descrio:

  Programa para gerar uma interface grafica de usuario (GUI). Objeto do pacote
  'classes'.

  GUIMain.java requer a classe GUIInternalFrame.java

Parmetros de entrada:

  No se aplica.

Sada:

  Interface grfica de usurio.

Desenvolvido por:

    Jos Iguelmar Miranda

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

package classes;

// pacote genrico
import java.io.File;

// pacotes Swing para interface grfica
import javax.swing.JMenu;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;

// pacotes do AWT
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.Dimension;
import java.awt.Toolkit;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;

public class GUIMain extends JFrame
	implements ActionListener {
  
	JDesktopPane desktop;
  private JFileChooser fc;
  BufferedImage img1 = null, img2 = null;

  public GUIMain() {
		super("Processa Imagens");

    // Janela principal: 
		// - 50 pixels de distncia das bordas do monitor
		// - largura e altura com metade do tamanho e altura
		//	 do monitor
		int inset = 50;
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setBounds(inset, inset, screenSize.width/2  - inset*2,
                     screenSize.height/2 - inset*2);

    // Define a GUI com um painel especfico
		desktop = new JDesktopPane();

		// Cria primeira 'janela'
    setContentPane(desktop);
    setJMenuBar(createMenuBar());

    // Torna 'dragging' mais rpido.
    desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
	}

  protected JMenuBar createMenuBar() {
    	
    JMenu menu;
    JMenuItem menuItem;
    JMenuBar menuBar = new JMenuBar();

		// Define o primeiro elemento do menu
    menu = new JMenu("Arquivo");
    menu.setMnemonic(KeyEvent.VK_A);
    menuBar.add(menu);

    // Define o primeiro item do menu Arquivo
    menuItem = new JMenuItem("Abrir");
    menuItem.setMnemonic(KeyEvent.VK_N);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_N, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("abrir");
    menuItem.addActionListener(this);
    menu.add(menuItem);

    // Define o segundo item do menu Arquivo
    menuItem = new JMenuItem("Sair");
    menuItem.setMnemonic(KeyEvent.VK_Q);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_Q, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("sair");
    menuItem.addActionListener(this);
    menu.add(menuItem);

		/*
		menu = new JMenu("Editar");
    menu.setMnemonic(KeyEvent.VK_D);
    menuBar.add(menu);
    */    

		// Define o segundo item do menu
    menu = new JMenu("Processar");
		menu.setMnemonic(KeyEvent.VK_P);
    menuBar.add(menu);
        
    // Define o primeiro item do menu Processar
    menuItem = new JMenuItem("Fuzzy Binariza");
		menuItem.setMnemonic(KeyEvent.VK_F);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_F, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("bin");
    menuItem.addActionListener(this);
    menu.add(menuItem);

    // Define o segundo item do menu Processar
    menuItem = new JMenuItem("Binariza com limiar");
    menuItem.setMnemonic(KeyEvent.VK_L);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_L, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("limiar");
    menuItem.addActionListener(this);
    menu.add(menuItem);

    // Define o terceiro item do menu Processar
    menuItem = new JMenuItem("Filtro Mediana");
    menuItem.setMnemonic(KeyEvent.VK_M);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_M, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("mediana");
    menuItem.addActionListener(this);
    menu.add(menuItem);

    // Define o quatro item do menu Processar
    menuItem = new JMenuItem("Filtro Eroso");
    menuItem.setMnemonic(KeyEvent.VK_A);
    menuItem.setAccelerator(KeyStroke.getKeyStroke(
    KeyEvent.VK_A, ActionEvent.ALT_MASK));
    menuItem.setActionCommand("erosao");
    menuItem.addActionListener(this);
    menu.add(menuItem);

    return menuBar;
	}

  // Atende solicitao da seleo no menu
  public void actionPerformed(ActionEvent e) {
		if ("abrir".equals(e.getActionCommand())) {
			BufferedImage img = AbrirImage();
      createFrame(img);
    } else if ("bin".equals(e.getActionCommand())) { // Fuzzy binariza
				GUIInternalFrame Fr = (GUIInternalFrame) desktop.getSelectedFrame();
			if ((img1 != null) && (Fr != null)) {
	    	ZIntensifica  gB = new ZIntensifica();
				BufferedImage img2 = gB.zadeh(Fr.img1, 3);
       	createFrame(img2);
			} else 
				JOptionPane.showMessageDialog(null, "Favor carregar imagem!", "Imagem", 
								JOptionPane.WARNING_MESSAGE);
      } else if ("limiar".equals(e.getActionCommand())) { // Binariza com limiar
        	GUIInternalFrame Fr = (GUIInternalFrame) desktop.getSelectedFrame();
				if ((img1 != null) && (Fr != null)) {
					String limiar = JOptionPane.showInputDialog(null, "Entre valor do limiar: ");
					int teto = Integer.parseInt(limiar);
	    		Binariza binariza = new Binariza();
					BufferedImage img2 = binariza.binarizaOp(Fr.img1, teto);
       		createFrame(img2);
				} else 
					JOptionPane.showMessageDialog(null, "Favor carregar imagem!", "Imagem", 
								JOptionPane.WARNING_MESSAGE);
      } else if ("mediana".equals(e.getActionCommand())) { // Filtro mediana
       		GUIInternalFrame Fr = (GUIInternalFrame) desktop.getSelectedFrame();
					if ((img1 != null) && (Fr != null)) {
        		FiltroMediana mediana = new FiltroMediana();
        		BufferedImage img2 = mediana.medianaOp(Fr.img1);
        		createFrame(img2);
					} else 
							JOptionPane.showMessageDialog(null, "Favor carregar imagem!", "Imagem", 
										JOptionPane.WARNING_MESSAGE);
			} else if ("erosao".equals(e.getActionCommand())) { // Filtro morfolgico de eroso
       		GUIInternalFrame Fr = (GUIInternalFrame) desktop.getSelectedFrame();
					if ((img1 != null) && (Fr != null)) {
        		Erode morfErode = new Erode();
        		BufferedImage img2 = morfErode.erodeOp(Fr.img1);
        		createFrame(img2);
					} else
							JOptionPane.showMessageDialog(null, "Favor carregar imagem!", "Imagem", 
										JOptionPane.WARNING_MESSAGE);
      } else { // quit
					quit();
      }
    }

    // Cria novo frame interno 
    protected void createFrame(BufferedImage im) {

			// Se imagem foi escolhida, mostra imagem...
			if (im != null) {
				GUIInternalFrame frame = new GUIInternalFrame();
				frame.setImage(im);
				frame.setVisible(true);
				desktop.add(frame);
				try {
					  frame.setSelected(true);
				} catch (java.beans.PropertyVetoException e) {
						System.out.println("Problema em GUIMain:: frame.setSelected");
						System.exit(0);
				}
			}
			// ...seno, espera por seleo
		}

    // Abre imagem usando dialogFile
    protected BufferedImage AbrirImage() {

			int returnVal;

			File file = null;
     	img1 = null;
    	if (fc == null) {
				fc = new JFileChooser();
        fc.addChoosableFileFilter(new FiltrosImg());
        fc.setAcceptAllFileFilterUsed(false);
    	}
      returnVal = fc.showDialog(desktop,"Abre arquivo");

      if (returnVal == JFileChooser.APPROVE_OPTION) {
        file = fc.getSelectedFile(); 
      }

			try {
        	img1 = ImageIO.read(file);
			} catch(Exception e) {
        	System.out.println("Imagem nao escolhida");
					JOptionPane.showMessageDialog(null, "Favor carregar imagem!", "Imagem", 
							JOptionPane.WARNING_MESSAGE);
					img1 = null;
			}
	    return img1;
    }

    // Sai da aplicao
    protected void quit() {
			System.exit(0);
    }

		// Cria GUI e mostra.
    private static void createAndShowGUI() {

			// Decorao da janela
      JFrame.setDefaultLookAndFeelDecorated(true);

      // Cria e inicializa a janela
      GUIMain frame = new GUIMain();

			// Fecha aplicao
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			// Mostra a janela
      frame.setVisible(true);
    }

    public static void main(String[] args) {
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					createAndShowGUI();
        }
      });
    }
}